function [decdata,decdata_std,cutpoint,member,count] = bin(data,key,no_deciles,aggregate_type,nodestr);
%
% [bindata,bindata_std,cutpoint,member,count] = bin(data,key,no_deciles,aggregate_type)
%
% aggregate_type is a function that operates on the data 
% key == horizontal array with the column ids of data to use as keys
% 
% EXAMPLE
%
% bin(rand(100,2),[1],99,{'sum','sum'});
%
% Note that the prototypes will then come out as sums, (so won't be very intelligable)
% It will be more common to use something like
%
% bin(rand(100,2),[1],10,{'mean','sum'});
%
% SEE ALSO
%
% decile1(data,key,nobins,aggregation_functions): will bin the data according to the distribution of values
% bin    (data,key,nobins,aggregation_functions): will bin the data using nobins fixed bins between min and max of range
%  

if nargin<5,
  nodestr = [ones(size(data,2),1) * ['Dimension '] num2str([1:size(data,2)]')];
end;

% Check for more deciles than data
% --------------------------------

if no_deciles>=size(data,1),
  fprintf('decile1 requested more deciles (%d) than data rows (%d)\n',no_deciles,size(data,1));
  decdata = data;
  decdata_std = zeros(size(data));
  cutpoint = decdata;
  member = [1:size(data,1)]';
  return;
end;

 
if size(key,2)>1,
  fprintf('Key must be only one dimension. If categorical data, you could use recode\n');
  fprintf('to recode the categories to a single category\n');
  fprintf('If numeric, you could use pca to recode the many key dimensions\n');
  fprintf('into a single principal component dimension, which will represent the movement\n');
  fprintf('in all of those key dimensions\n');
  return;
end;
 

% Allow matlab to create equally spaced bins
% -------------------------------------------

clear proto protoy;
[count,proto] = hist(data(:,key),no_deciles);
proto=proto';
% The prototypes are the equally spaced histogram bins
figure(1);clf;hist(data(:,key),no_deciles); % plot them graphically
title(['Frequency for ' deblank(nodestr(key,:))]);
xlabel([deblank(nodestr(key,:))]);
ylabel('frequency');

% Find out the proto assignment in the training data
% --------------------------------------------------

[yest,id] = k_nearest_neighbour(data(:,key),proto,proto,1);
% Find out what each row of the data belongs to (in the 
% prototypes) 


% Compute protoy
% --------------

protoy = zeros(no_deciles,size(aggregate_type,2));
% these are the other values in the data
% These other values are being keyed off the key dimension, eg. 
% the key dimension prototype is x, the 

for dim=1:size(aggregate_type,2),
  curfun = char(aggregate_type(dim));
  for i=1:no_deciles, 
    this = find(id==i);
    if isempty(this),
      protoy(i,dim) = nan;
      decdata_std(i,dim) = nan;
    else
      protoy(i,dim) = feval(char(aggregate_type(1,dim)),data(this,dim));
      decdata_std(i,dim) = std(data(this,dim));
    end;
  end;
  figure(2);clf;plot(proto,protoy(:,dim),'ro-');hold on;
  eplot([proto zeros(size(proto,1))] ,[protoy(:,dim) decdata_std(:,dim).*0.1],'r-');
  
  xlabel([deblank(nodestr(key,:)) ' bin']);
  ylabel(['resulting ' curfun ' for ' deblank(nodestr(dim,:))]);
  title([deblank(nodestr(dim,:)) ' ' curfun ' for each bin']);
end;

decdata = protoy;
cutpoint = proto;
member = id;

