function [decdata,decdata_std,cutpoint,member,declower,decupper,group] = decile1(data,key,no_deciles,aggregate_type,showgraph,caseweights,dozerospread);
%
% [decdata,decdata_std,cutpoint,member,declower,decupper,group] = decile1(data,key,no_deciles,aggregate_type,showgraph,caseweights,dozerospread)
%
% aggregate_type is a *Horizontal array* the same size across as data
% 0==sum, 1==mean, 2==stdev, 3==min, 4==max
% You can now use a function instead of one of the options above, see 
% example below 
% key == horizontal array with the column ids of data to use as keys
% 
% EXAMPLE
%
% decile1(data,[1],10,zeros(1,size(data,2))); % sum all cols
% decile1(rand(100,3),[1],99,{'sum','sum','U0.9'});
%
% %{'0.9'} means take the lower 90th percentile of the distribution
% 

if nargin<7,
  dozerospread=0;
end;
if nargin<6,
  caseweights = ones(size(data,1),1);
end;
if nargin<5, 
  showgraph=0;
end;

% Check for more deciles than data
% --------------------------------

if no_deciles>=size(data,1),
  fprintf('decile1 requested more deciles (%d) than data rows (%d)\n',no_deciles,size(data,1));
  orig_size = size(data,1);
  data = [data ; ones(   (no_deciles-size(data,1)),1) * data(1,:) ];
  decdata = data;
  decdata_std = zeros(size(data));
  cutpoint = decdata;
  member = [1:size(data,1)]'; 
  % The old way of doing it. Problem was, this claimed that data at position 5 had member 5. But 
  % data at position 5 was invented - and does not really exist outside. Therefore, we have to force the 
  % made up data not to have a member that will be matched on
  member(orig_size+1:length(member)) = nan.*ones(length(member)-orig_size,1);
  return;
end;

  
% Create the declower and decupper for each decile
% ------------------------------------------------

days_each_range = size(data,1)./no_deciles;
for i=1:no_deciles,
   declower(i,1) = floor((i-1).*days_each_range)+1;
   decupper(i,1) = ceil(i.*days_each_range);

   if declower(i,1)<1,            declower(i,1)=1;            end;
   if decupper(i,1)>size(data,1), decupper(i,1)=size(data,1); end;

   % By the above, declower..decupper should always span at 
   % least one row
end;

if nargout>=7,
  if round(days_each_range)~=days_each_range,
    fprintf('number of bins does not divide evenly into data, group will lose some information\n');
    group = zeros(days_each_range-1,no_deciles); % Will store ids of each group
  else
    group = zeros(days_each_range,no_deciles);
  end;
end;


% Sort key
% --------

keydata = data(:,key);
[keysorted,sortid] = sortrows(keydata);
sortid    = flipud(sortid); 
keysorted = flipud(keysorted); 

% flip to have 1 equals first decile
% Each decile can be referenced as follows:
% pres = sortid(declower(i,1):decupper(i,1),1); % gets the ith decile


% Calc aggregates for each decile
% -------------------------------

for i=1:no_deciles, 

   % pick out the correct ids from sortid for decile i
   pres           = sortid([declower(i,1):decupper(i,1)]',1);
   member(pres,1) = ones(size(pres,1),1).*i;
   if nargout>=7,
     group(:,i)     = pres(1:size(group,1)); % The ids of rows belonging to member i
   end;
   
   cutpoint(i,:) = data(pres(1),key);

   % Aggregate each column of data (including the key columns)
   for ii=1:size(data,2),
     if ~isnumeric(aggregate_type(1,ii)), 
       % Assume this is a function call
       
       n = str2num(char(aggregate_type(1,ii)));
       if ~isempty(n),
         if n>0 & n<1, 
           if dozerospread==1, 
             decdata(i,ii) = percentile(zerospread(data(pres,ii)),n);
           else
             decdata(i,ii) = percentile(data(pres,ii),n);
           end;
         else 
           fprintf('Argument %d of decile functions was %s.\n',ii,char(aggregate_type(1,ii)));
           fprintf('This was parsed as a request to run percentile on the decile at percentile level %f\n',n);
           fprintf('The percentile level must be >0 and <1.\n');
           fprintf('Changing this to the mean. PLEASE CORRECT THIS PROBLEM.');
           keyboard;
         end;
       %func = char(aggregate_type(1,ii));
       %firstchar = upper(func(1));
       %func = func(2:length(func));
       %n = str2num(func);
       %if (firstchar=='L' | firstchar=='U') & ~isempty(n) & n<=1 & n>0, 
       %  if firstchar=='L' & ~isempty(n) & n<=1 & n>0,
       %    decdata(i,ii) = errorbars(data(pres,ii),n); 
       %  end;
       %  if firstchar=='U' & ~isempty(n) & n<=1 & n>0,
       %    [nuh,decdata(i,ii)] = errorbars(data(pres,ii),n);
       %  end;

       else  
         if dozerospread, 
           decdata(i,ii) = feval(char(aggregate_type(1,ii)),zerospread(data(pres,ii)));
         else
           decdata(i,ii) = feval(char(aggregate_type(1,ii)),data(pres,ii));
         end;
       end;
     end;
     if ~iscell(aggregate_type(1,ii)),
     if aggregate_type(1,ii)==1,
       if dozerospread, decdata(i,ii) = mean(zerospread(data(pres,ii))); else decdata(i,ii) = mean(data(pres,ii));end;
     end;
     if aggregate_type(1,ii)==0,
       if dozerospread, decdata(i,ii) = sum(zerospread(data(pres,ii))); else decdata(i,ii) = sum(data(pres,ii));end;
     end;
     if aggregate_type(1,ii)==2,
        if dozerospread, decdata(i,ii) = std(zerospread(data(pres,ii))); else decdata(i,ii) = std(data(pres,ii));end;
     end;
     if aggregate_type(1,ii)==3,
        if dozerospread, decdata(i,ii) = min(zerospread(data(pres,ii))); else decdata(i,ii) = min(data(pres,ii));end;
     end;
     if aggregate_type(1,ii)==4,
        if dozerospread, decdata(i,ii) = max(zerospread(data(pres,ii))); else decdata(i,ii) = max(data(pres,ii));end;
     end;
     end;
     if dozerospread, decdata_std(i,ii) = std(zerospread(data(pres,ii))); else decdata_std(i,ii) = std(data(pres,ii));end;
   end;

end; % foreach decile
cutpoint(i+1,:) = data(pres(size(pres,1)),key);

if showgraph,
  figure(1);clf;
  bar(decdata(:,1));
  set(gca,'xticklabel',num2str(decdata(:,key)));
end;
