function [lowerval,upperval,stderr,stderr_resamp] = errorbars(data,conflevel);
%
% [l,u,stderr,stderr_resamp] = errorbars(data,conflevel);
%
% Finds the upper and lower values, which the data is conflevel%
% of the time going to be within. For example, [l,u]=errorbars(data,0.8)
% finds upper and lower values which the distribution is 80% of the time
% within. Or, for any point, there is an 80% chance that its value is
% within those lower and upper bounds.
%
% [l,u] are calculated using the empirical distribution. Ie. the data
% is sorted, and then bounds taken such that 80% of the distribution
% is within those limits.
%
% [ls,us] are calculated using the standard error, which is an approximate
% method for calculating upper and lower bounds, and uses the
% Central Limit Theorem.
%
% [] are calculated using resampling stats
%
% SEE ALSO
%
% [l,u] = errorbars(data,conflevel);
% 

if isempty(data) | size(data,1)==0,
   lowerval=NaN;
   upperval=NaN;
   stderr=NaN;
   fprintf('errorbars: no data, returning empty upper and lower bounds');
   return
end;

samp_percent = 0.3;
number_of_samples = 10;




% Empirical error bars
% --------------------

[datasorted,i] = sort(data);

endfraction = (1-conflevel)./2; 
% this is the bit on the left and right sides of the series

maxlower = floor(endfraction.*size(data,1));
if maxlower>size(data,1), maxlower=size(data,1);end;
if maxlower<=0, maxlower=1;end;
lowerval = datasorted(maxlower,:);

minupper = size(data,1)-ceil(endfraction.*size(data,1));
if minupper<1, minupper=1;end;
if minupper>=size(data,1),minupper=size(data,1);end;
upperval = datasorted(minupper,:);


% Standard error
% --------------

stderr = std(data)./sqrt(size(data,1));

% we need to find out what the distribution is to get the 95% 
% interval

if 0,
% Resampling method
% -----------------

% Select 30 samples of 10% size, and calc standard error of the mean

samplesize = round(samp_percent.*size(data,1));
for i=1:number_of_samples,
  
  % Shuffle data in-place for random sampling
  r = rand(size(data,1),1);
  [a,ii] = sort(r);
  data = data(ii,:);

  % Sample 10%
  samp = data(1:samplesize,1);

  m(i,1) = mean(samp);
end;

stderr_resamp = std(m); 
% standard deviation of the mean
% is the standard error


end;
