function [err,dist, estimate, estvect, regmean, ...
          stddev,stderr,num]   = accuracy_estimate_mean2(...
                                              data,             ...
                                              iterations,       ...
                                              kernel_size,      ...
                                              dims_to_estimate, ...
                                              missing_val);
%
%function [err,dist,estimate,estvect, regmean, ...
%          stddev,stderr,num] = accuracy_estimate_mean2( ...
%                      data,                    ...
%                      iterations,              ...
%                      kernel_size,             ...
%                      dims_to_estimate,        ...
%                      missing_val);
%

present_dims  = setdiff(1:size(data,2),dims_to_estimate);

regdata = replace_nulls_with_mean(data,missing_val);

it=1;
for dim=present_dims,

  % for each present_dim, test it at its min and max range
  % then test the estimate method moving from training set
  % to that min and max range

  % find max and min range for this dimension
  % -----------------------------------------

  jj = find(data(:,dim)~=missing_val);
  minval = min(data(jj,dim));
  maxval = 10; %maxval = max(data(jj,dim));

  % find all datapoints within 0.5 stdev from the max and min
  % ---------------------------------------------------------

  %stdd   = std(data(jj,dim));
  %highvals = find( abs(maxval - data(jj,dim)) < stdd .* 2);
  %lowvals  = find( abs(minval - data(jj,dim)) < stdd .* 4);
  highvals = find(data(jj,dim)>maxval-2);
  lowvals = find(data(jj,dim)<minval+2);

  allminushigh = setdiff(1:size(jj,1),highvals)';
  allminuslow  = setdiff(1:size(jj,1),lowvals)';

  [size(highvals,1) , size(allminushigh,1)],

  %estvect(it,:) = ones(1,size(data,2)) .* -9;
  estvect(it,:) = mean_missing(data(jj(highvals),:),missing_val);

  % this is the position that we will test in our training
  % set. (Simply the mean of the test set which has been 
  % randomized for us). We happen to know what the missing dims 
  % are for this pos in our test set. (its equal to
  % mean_missing(data(te,dims_to_est))
  
  % estimate the mean of the dims_to_estimate using the training
  % set, relocated at position estvect

    [estimate(it,:),stddev(it,:),stderr(it,:),num(it,:)] = ...
    estimate_mean_newregion(data(jj(allminushigh),:), ...
    estvect(it,:),kernel_size,dims_to_estimate,missing_val);
  
  % calc distance between training and test set means (ie.
  % the distance between the training set mean and the point
  % that was just estimated)

  meantrain       = mean_missing(data(jj(allminushigh),:),missing_val);
  dist(it,1)      = distance(meantrain(1,present_dims), ...
                             estvect(it,present_dims));

% perform regression from data(jj(allminushigh)) all dims onto 
% same data, revenue (8). We can then test generalization ability of 
% this model by testing it on the points jj(highvals), and checking
% the resulting mean.

  w=polyfitf(regdata(jj(allminushigh),present_dims),...
             regdata(jj(allminushigh),dims_to_estimate),1);
  yest = polyff(w,1,data(jj(highvals),present_dims));
  % will actually estimate the rev value for every person. 
  % But what we're interested in is the mean for the group. How
  % accurate is the mean. So we mean these estimates and compare
  % against the actual mean.
  regmean(it,1) = mean(yest);

it=it+1;

  %estvect(it,:) = ones(1,size(data,2)) .* -9;
  estvect(it,:) = mean_missing(data(jj(lowvals),:),missing_val);

    [estimate(it,:),stddev(it,:),stderr(it,:),num(it,:)] = ...
    estimate_mean_newregion(data(jj(allminuslow),:), ...
    estvect(it,:),kernel_size,dims_to_estimate,missing_val);
  
  meantrain       = mean_missing(data(jj(allminuslow),:),missing_val);
  dist(it,1)      = distance(meantrain(1,present_dims), ...
                             estvect(it,present_dims));

  w=polyfitf(regdata(jj(allminuslow),present_dims),...
             regdata(jj(allminuslow),dims_to_estimate),1);
  yest = polyff(w,1,data(jj(lowvals),present_dims));
  % will actually estimate the rev value for every person. 
  % But what we're interested in is the mean for the group. How
  % accurate is the mean. So we mean these estimates and compare
  % against the actual mean.
  regmean(it,1) = mean(yest);

it=it+1;

end;

err = abs(estimate(:,dims_to_estimate)-estvect(:,dims_to_estimate));

% we can plot dist,err 
% also can plot estimate(:,8),estvect(:,8)
%

%estimate = estimate(:,dims_to_estimate);
%estvect  = estvect(:,dims_to_estimate);






% ---------------------------------------------------------------
% notes

  % randomize data (randomizes train and test sets)
  %[a,i] = sort(rand(size(data,1),1));
  %data = data(i,:);

% split into train, test
if 0,
maxtrain = floor(size(data,1).*trainpercent)+1;
tr=1:maxtrain; 
te=maxtrain+1:size(data,1);
end;

