function [dist, estimate, regmean, regadjust, correct, ...
          stddev,stderr,num]   = accuracy_estimate_mean3(...
                                              traindata,        ...
                                              testdata,         ...
                                              iterations,       ...
                                              kernel_size,      ...
                                              dims_to_estimate, ...
                                              missing_val);
%
%function [err,dist,estimate,estvect, regmean, ...
%          stddev,stderr,num] = accuracy_estimate_mean3( ...
%                      data,                    ...
%                      iterations,              ...
%                      kernel_size,             ...
%                      dims_to_estimate,        ...
%                      missing_val);
%
% Example
%
% % see how avg dist from true mean (stderr of mean)
% % decreases as you approach full sample size. The origin of the
% % std/sqrt(n).
% % When sample population size = full pop, error is 0.
% % When sample population size = 1, average expected error = stddev
%
% for i=1:30, % number of trials                                                                 
%   % reshuffle the data each pull (isn't really necessary, omit
%   % for speed)
%   % r=rand(size(data,1),1);[a,id]=sort(r);data=data(id,:);
%   testrange = 1:round(size(data,1)./20):size(data,1);
%   for ii=testrange,
%     % pick ii samples from the data and find out mean-truemean
%     m(i,ceil(ii/10))=distance(mean(data(1:ii,:)),truemean);
%   end;
% end;
% mm = mean(m);
% figure(1);plot(1:11,mm','-');
%

% regression assumes that we've created a model from attitudes 
% to revenue, and are then testing that model's mean when the
% population shifts to new values

disp('set difference');
present_dims  = setdiff(1:size(traindata,2),dims_to_estimate);

disp('ensuring no missing values');
testdata = replace_nulls_with_mean(testdata,missing_val);
traindata = replace_nulls_with_mean(traindata,missing_val);

mean_traindata = mean(traindata);
% used to get distance to test subpop

test_kernel = 5; 
% size of the test kernel. Should be reasonably
% large so that it picks up enough points. To large and it becomes
% equal to the group.

% precalculate regression
degree = 1; 
w=polyfitf(traindata(:,present_dims), ...
           traindata(:,dims_to_estimate), ...
           degree);
  
for it=1:iterations,

  % select point at random from the test set. This will be the centroid
  % for the subpopulation we will be testing

  cent=floor(rand(1,1).*size(testdata,1)-eps)+1;
  
  % find all points within test_kernel of this point

  subpop = [];mult = test_kernel;
  while size(subpop,1)<20,
    dist_to_cent = ...
    distance(ones(size(testdata,1),1)*testdata(cent,:),testdata);
    subpop = find(dist_to_cent < mult);
    size(subpop,1),
    mult = mult.^1.1; % 1.1 works well
  end;
 
  % if we can get a large spread of large and small groups, then
  % we can see how as dist from mean of training pop to mean of
  % test goes to zero (in case of very large sample), how error
  % increases. We can then get an expected error for distance
  % error is also dependent on size

  % now send the mean for this subpop to estimate_mean_newregion
  % this will find elements in the training set close to the desired
  % point, and will use the mean value of these points mean to estimate
  % the mean for the test point

  mean_testdata = mean(testdata(subpop,:));

  % training data estimate subpop
  % -----------------------------

  [estimate(it,:),stddev(it,:),stderr(it,:),num(it,:)] = ...
   estimate_mean_newregion( ...
                           traindata, ...     % training pool
                           mean_testdata, ... % point to estimate
                           kernel_size, ...   % size of generalization win
                           dims_to_estimate, ...
                           missing_val);
  
  % regression model
  % (i) prop the mean of the location we want to get a value for
  regmean(it,:) = polyff(w,degree,mean_testdata(1,present_dims));

  % regression model
  % (ii) modify the training set (eg. increasing variables to 
  % the amount needed to get a mean of testsetmean). Then run
  % each of those adjusted training records through to get the 
  % values for each. Then take the mean.

  diff = mean_testdata-mean_traindata;
  yest = polyff(w,degree,traindata(:,present_dims) + ...
               ones(size(traindata,1),1)*diff(1,present_dims) ./ ...
               size(traindata,1) );
  regadjust(it,:) = mean(yest);

  % correct answer
  correct(it,:) = mean_testdata;
  % correct answer is correct(it,dims_to_estimate)

  % distance from training mean to subpop mean
  dist(it,1)      = distance(mean_testdata(1,present_dims), ...
                             mean_traindata(1,present_dims));

end;

estimate = estimate(:,dims_to_estimate);
correct  = correct(:,dims_to_estimate);

