function [rankcorr,corr,meanabs,medabs,meanerr,pcorrect1,pcorrect_overall, ... 
          lift1, cumm1, ...
          rankcorr1,corr1,meanabs1,medabs1,meanerr1, ...
          mat,no_actual,no_pred,r] = ...
          accuracy_of_model(actual,pred,no_bins,useactual,usepred);
%       
% [rankcorr,corr,meanabs,medabs,meanerr,pcorrect,lift1, cumm1, rankcorr1,corr1, ...
%  meanabs1,medabs1,meanerr1,pcorrect1,pcorrect_overall,mat,no_actual,no_pred]=accuracy_of_model(actual,pred,no_bins,useactual,usepred);
%
% Calculates error / relationship statistics for two variables, actual and pred
% Also pools actual and pred into deciles (1..no_bins), and compares predictions.
% This allows a confusion matrix to be generated, and average number of deciles 
% error to be calculated (so its like a non-parametric version of the accuracy, 
% where we're interested in how well it does in predicting ranked levels
%
% no_bins > 0 means discretize the real values into artificial categories 
% for analysis
% no_bins == 0 means use the actual values in the actual and pre

pred=full(pred(:,1));
actual=full(actual(:,1));

if nargin<=2,
   no_bins=10;
end;
use_actual_vals = 0;
if no_bins==0,
  fprintf('no_bins (%d) must be greater than 0\n',no_bins);
  return;
end;
if nargin<4, 
  useactual=0;
  usepred=0;
end;
if nargin<5, 
  usepred=0;
end;

fprintf('Model Fit Diagnositics -----------------------\n');

% Rankcorr on ordinary actual and pred
fprintf('Performance of raw predictions against raw actuals\n');
rankcorr = rank_correlation(actual,pred);                  fprintf(' rankcorr \tSpearman''s nonparametric\t%f\n',rankcorr);
%mn = min([actual;pred]);
%loglogcorr  = correlation(log(actual-mn+1),log(pred-mn+1));fprintf(' loglogcorr\t%f\n',rankcorr);
% allows slop in the high values, but must do well in the bulk of the low ranges
corr     = correlation(actual,pred);                       fprintf(' corr     \tCorrelation affected by outliers\t%f\n',corr);
meanabs  = mean(abs(actual-pred));                         fprintf(' meanabs  \tAverage difference between forecast and actual\t%f\n',meanabs);
pmeanabs  = meanabs./(mean(abs(actual-mean(actual)))+eps); fprintf(' meanabs%% \tAverage error as a percent of actual\t%6.2f%%\n',100.*pmeanabs);
medabs   = median(abs(actual-pred));                       fprintf(' medabs   \tMedian difference between forecast and actual\t%f\n',medabs);
warning off;
pmedabs   = medabs./(median(abs(actual-mean(actual)))+eps); fprintf(' medabs%%  \tMedian error as a percent of actual\t%6.2f%%\n',100.*pmedabs);
warning on;
meanerr  = mean(actual-pred);                           fprintf(' meanerr  \tForecast bias (accumulating high or low)\t%f\n',meanerr);
maxover  = max(pred-actual);                            fprintf(' maxover  \tWorst over-forecast\t%f\n',maxover);
maxunder = min(pred-actual);                            fprintf(' maxunder \tWorst under-forecast\t%f\n',maxunder);


% Convert to deciles
% ------------------

figure(1); % reserved for decile1
%iv=find(actual~=0); % omit cases where actual==0 & pred==0
iv=[1:size(actual,1)]';
actual1 = actual(iv,1);
pred1   = pred(iv,1);
if useactual==0, [a1,a2,a3,actual1] = decile1(-1.*actual(iv,1),1,no_bins,0); end;
if usepred==0,   [p1,p2,p3,pred1]   = decile1(-1.*pred(iv,1),  1,no_bins,0); end;
% The -1 is because decile1 sorts from highest to lowest

% Histogram of residuals
[x,a]=hist(actual-pred,100);
figure(2);clf;subplot(2,1,1);bar(a,x);title('Prediction error (under/over)');
xlabel('error');
ylabel('number of cases');hold on;
decdata = decile1(actual-pred,[1],100,{'sum'});
figure(2);subplot(2,1,2);bar(decdata(:,1));
xlabel('error');ylabel('number of cases');title('Deciled prediction error');

%keyboard
%i=find(actual1<1);actual1(i,1)=ones(size(i,1),1);
%i=find(pred1<1);pred1(i,1)=ones(size(i,1),1);
% We shouldn't have to do this. CHECK DECILE1 CODE
%actual1 = a1(actual1);
%pred1   = p1(pred1);

% collapse common values. 
% eg. actual1 is 1..10
% but a1 has many redundant
% merge each.
   

% Rankcorr on relative deciles
fprintf('Performance in predicting decile groups\n');
rankcorr1 = rank_correlation(actual1,pred1); fprintf(' rankcorr1 \t%f\n',rankcorr1);
corr1     = correlation(actual1,pred1);      fprintf(' corr1     \t%f\n',corr1);
meanabs1  = mean(abs(actual1-pred1));        fprintf(' meanabs1  \t%f\n',meanabs1);
pmeanabs1  = meanabs1./(mean(abs(actual1-mean(actual1)))+eps);    fprintf(' meanabs%% \t%6.2f%%\n',100.*pmeanabs1);
medabs1   = median(abs(actual1-pred1));      fprintf(' medabs1   \t%f\n',medabs1);
warning off;
pmedabs1   = medabs1./(median(abs(actual1-mean(actual1)))+eps); fprintf(' medabs%%  \t%6.2f%%\n',100.*pmedabs1);
warning on;
meanerr1  = mean(actual1-pred1);             fprintf(' meanerr1  \t%f\n',meanerr1);

% Histogram of residuals
[a,x]=hist(actual1-pred1,100);
try,
figure(3);clf;bar(a,x);title('Rank group prediction error (under/over)');
catch,
fprintf('histogram failed\n');
end;
xlabel('error');
ylabel('number of cases');

if ~useactual & ~usepred, 
  [mat,no_actual,no_pred,vals,pcorrect1,number_correct] = ...
     confusion_matrix(actual1,pred1,4,[1:no_bins]');
else 
  [mat,no_actual,no_pred,vals,pcorrect1,number_correct] = ...
     confusion_matrix2(actual1,pred1,4);
end;

r = [];
if useactual & ~usepred,
  ii=find(actual1~=0 & actual1~=1); 
  if isempty(ii), 
    r = point_biserial(actual1,pred1);
    fprintf('point biserial (only applicable if actual was 1,0 and pred was continuous) %f\n',r);
  end;
end;

%[mat,no_actual,no_pred,vals,pcorrect1,number_correct] = ...
%   confusion_matrix(actual1,pred1,4);

xlabel('rank bin');ylabel('correct predicton %');
title('Performance in predicting each decile');

pcorrect_overall = number_correct ./ size(actual,1);
fprintf(' pcorrect_overall %f\n',pcorrect_overall);
fprintf('------------------------------------\n\n\n');

% Lift curves
lift_bins = 10;
fprintf('Lift\n');
[valseries,liftval]=lift2(actual,pred,lift_bins,9);
fprintf(' decile\t');fprintf('%d\t',[1:lift_bins]);fprintf('\n');
fprintf(' lift  \t');fprintf('%f\t',liftval);fprintf('\n');

figure;clf;hold on;
decdata = decile1([pred actual],[1],10,{'mean','mean'});
figure(1);clf;hold on;plot(decdata(:,1),decdata(:,2),'o-');xlabel('predicted');ylabel('actual');axis('equal');axis('square');
v=axis;mv = max(v);axis([0 mv 0 mv]);
hold on;plot([0 mv],[0 mv],'-');
grid on;title('predicted deciles versus actual deciles');
