function y = binary_eval(x, c, centroidlist, cy);
%
% y = binary_eval(x, c, centroidlist, cy);
%
% EXAMPLE
%
% % (1) accuracy improves as you go down the cluster tree 
% datapoints=30; p=3; iterations=10; lc=0.3;
% x = rand(datapoints,2); y = x(:,1).^2; % training data
% 
%   number_of_clusters=round(2.*datapoints./p),
%   [c,centroidlist,cover,cy,level,member] = ...     
%     binary_cluster(x,number_of_clusters,lc,iterations,y);
%   figure(1);clf;
%   for lev=2:2+p-1,
%     levcent = level(1:lev,:);
%     levcent = levcent(:);
%     yest = binary_eval(x,c,levcent,cy);
%     subplot(p,1,lev-1);plot(yest,'b-');hold on;plot(y,'r-');
%     drawnow;
%     fprintf('level %d of binary tree / correlation %f\n', ...
%             lev-1,correlation(yest,y));
%   end;
%
% figure(2);graph_btree_wrapper(centroidlist,1,20,.1);
% 
% SEE ALSO
% [cterminal,c,level,centroidlist,cover,member] = ...
% binary_cluster(x,number_of_clusters,lc,iterations,y,missing_val,terminal_layer);
%


  % Check inputs
  % ------------

  if ~ismember(1,centroidlist),
    disp('something wrong with centroidlist. No 1 node');
    return
  end;

  y = zeros(size(x,1),size(cy,2));
  
  % Loop over rows
  % --------------

  for row=1:size(x,1),

    % Perform branch traversal
    % ------------------------

    id=1; % start at top
    while 1,
      % search down the branches

      lchild = id.*2;
      rchild = id.*2+1;

      if ~ismember(lchild,centroidlist),
        % there are now children underneath this node. 
        % therefore, this is the last node, return its y value
        y(row,:) = cy(id,:);
        break;
      end;

      if distance(c(lchild,:),x(row,:)) < distance(c(rchild,:),x(row,:)), 
        % choose the left branch
        id = lchild;
      else % right branch
        id = rchild;
      end;

    end; % while 1
  end; % for rows


return
