function [retainvars, collapsed_variables] = ...
   collapse_colinear_vars(corr,threshold,method);
%
% [retainvars,collapsed_vars] = ...
% collapse_colinear_vars(corr,threshold,method);
%
% method==1, eliminate variables with high correlations
% method==2, find groups of correlated variables and replace with average
%
% EXAMPLE
% corr=corrcoef(x,x);
% collapse_colinear_vars(corr-triu(corr),0.5,1);
%


%corr = corrcoef(data);
% to calculate this, we can use outer product, of 
% 1/n .* data * data'
% then pointwize divide this matrix by 
% sqrt(c(i,i) * c(j,j))
%

%corr = corr-triu(corr); % subtracts out the 1s along the diagonal,
% and the correlation in the upper triangular part. In the lower part,
% we have corr-0, so it remains the same

% now find the max correlations

collapse = [];
collapsed_variables = [];
while 1,
   a=max(max(corr));
   b=min(min(corr));
   if abs(a)<abs(b),a=b;end;
   %[down across a],
   if abs(a)>threshold, 
     [down,across] = find(corr==a);
      
      
      % Collpase one of these variables!
      % ---------------------------------
      
%      collapse = [collapse;down across];
%   else
%      break;
%   end;
%      
%end;

% find the variable which is most numerous in the collapse variables. 
% Collapse this first

% collapse is in order from most severe to least

% choose the variable to keep which is not correlated with many others. ie.
% the variable with the fewest correlations wins. and destroys the competition

% calculate correlation degree
% down and across are each variables.

  downcorr   = sum(corr(down,:)'.^2) + sum(corr(:,down).^2);
  acrosscorr = sum(corr(across,:)'.^2) + sum(corr(:,across).^2);

%[downcorr,acrosscorr],
% whichever is smaller is the winning variable

  if downcorr<acrosscorr,
     collapsedim = across;
  else
     collapsedim = down;
  end;

  corr(collapsedim,:) = zeros(1,size(corr,1));
  corr(:,collapsedim) = zeros(size(corr,1),1);
  collapsed_variables = [collapsedim collapsed_variables];

% look for more correlations>threshold and zero them

else % none found > threshold
   break;
end;

end; % while

retainvars = [];
for i=1:size(corr,1),
   if isempty(find(collapsed_variables==i)),
      retainvars = [retainvars i];
   end;
end;


return