function testresult = compare_two_groups2(x,y,topN,nodestr);
%
% testresult = compare_two_groups2(x,y,topN,nodestr);
%
% DESCRIPTION
% 
% Runs a number of non-parametric and parametric, and comparison tests simultaneously
% Assumes that  you have a single data set, eg. x, and 
% this is comprised of many variables. You have two groups or sets of observations 
% that you want to compare as being different. Do this by passing in 
% x, a, b where a and b are vectors with the observation numbers that belong to 
% group a and group b respectively
%
% Think of a as the experimental group, and b as the control group
% Interpret the stats as "100 var1 means that a is +100 over b
%
% REFERENCES
%
% Blalock, H. (1972), Social Statistics, McGraw-Hill (for information on t-test)
% 
% SEE ALSO
% 
% compare_two_groups(dat,a,b,topN,nodestr);
%


if size(x,2)~=size(y,2), 
  fprintf('x and y have different number of columns\n');
  return;
end;

dat = [x;y];
nx = size(x,1);
ny = size(y,1);
clear x y;
testresult = compare_two_groups(dat,[1:nx]',[nx+1:size(dat,1)]',topN,nodestr);


