function [valtree,no_vals,valtree2,xrecoded]=construct_valtree(x,missing_val);
% [valtree,no_vals,valtree2,xrecoded]=construct_valtree(x,missing_val);
%
% A valtree is just a vertical column of unique values which existed in 
% the x data. Because each dimension can have a different number of unique
% values, the valtree is terminated with a special value called missing_val
% valtree is used in dtree and nb to keep track of what unique values
% that algorithm has to consider. For example, in a decision tree it looks for
% splits on these unique values.
%
% valtree2 and xrecoded provides an alternative automatically recoded valtree
% and x set. The recoded valtree and xrecoded can be faster to execute in a 
% naive bayes algorithm

fprintf('constructing valtree');
valtree = sparse([]);

if nargout>=3, xrecoded = x; valtree2=valtree; end;

for d=1:size(x,2),
% foreach dimension
fprintf('.');
if isnan(missing_val),
   ii=find(~isnan(x(:,d)));
else
   ii=find(x(:,d)~=missing_val);
end;
   
  col = unique(full(x(ii,d)));
  %col = setdiff(col,missing_val)'; 
  % remove missing_val if it appeared in the set of unique values
  
  valtree(1:size(col,1),d) = col; 
  % put onto valtree
  
  no_vals(d,1)=size(col,1);
  
  valtree(size(col,1)+1,d) = missing_val; 
  % add a missing value terminal after the last val
  
  if nargout>=3,
     valtree2(1:size(col,1),d) = [1:size(col,1)]';
     valtree2(size(col,1)+1,d) = missing_val;
     % valtree2 is a valtree where we've recoded all values to 
     % numbers 1..size(col,1). This will enable faster lookup, however 
     % we need to create a recoded x
     for i=1:size(col,1),
       jj=find(x(:,d)==valtree(i,d));            % find(data==vals(i,:));
       xrecoded(jj,d)=ones(size(jj,1),1).*i;
       count(i,1) = size(jj,1);
    end;
 end; % if 
end;
fprintf('\n');

return