
% BTOOLKIT: NUMERICAL ROUTINES FOR MATLAB
% 
% written by Brendan Kitts 1994-2002. Freely available for all scientific uses, 
% for the betterment of mankind. Please email enhancements to bkitts@excite.com
% 
% Please cite as: 
% Kitts, B. (2017), BToolkit: Numerical routines for Matlab,
% http://www.appliedaisystems.com/code/BToolkit.zip
% 
% ESTIMATORS
% 
%  adaline_train       - train an adaline model (identical to a regression model)
%  adaline_eval        - evaluate an adaline model (identical to a regression model)
%  bayesian_train      - train a bayesian model
%  bayesian_eval       - evaluate a bayesian model
%  bboost              - experimental boosting algorithm
%  bboost_eval         - experimental boosting algorithm
%  dtree_train         - train a decision tree
%  dtree_eval          - evaluate a decisio ntree
%  k_nearest_neighbour - means for looking up solutions
%  knn                 - means for looking up solutions
%  linear_reg_train    - linear regression train
%  linear_reg_eval     - linear regression eval
%  loess_adaline       - loess smoother
%  nb_train            - train a naive bayes model
%  nb_eval             - evaluate a naive bayes model
%  polyfitf            - fit a polynomial model
%  polyff              - evaluate a polynomial model
%  spline_eval         - evaluate a spline model
%  spline_train        - train a spline model
%  stepwize            - stepwize polynomial regression
%  vam_train           - train a vam model
%  vam_eval            - evaluate a vam model 
% 
% COMMON CURVES
%
%  gutenberg_error
%  exp_error
%  pow_error
%  polyfitf
%  
% CLUSTERING
%
%  kmeans_mean         - please use this first
%  binary_cluster
%  binary_eval
%  kmeans
%  kmeans2
%  kmeans_mean
%  kmeans_batch
%  kmeans_batchc2
%  agglomerative_cluster
%  macqueen
%  vam
%  vam_online
% 
% MODEL FIT DIAGNOSTICS
% 
%  accuracy_of_model             - runs a battery of tests to determine how closely one variable tracks another
%  confusion_matrix2             - generates a matrix binning the actual versus predicted values, and their distribution
%  lift2                         - the level above random that a predictor vector matches a dependent
%  point_biserial                - fit between a dichotomous and continuous variable
%  correlation                   - correlation between two variables
%  corrmat2                      - correlation matrix between two sets of variables
%  NEED tau, OTHER DICHOTOMOUS AND CONTINUOUS VARS
%  
% TRANSFORMATIONS
%
%  dummyvarf                  - create dummy variables for categories 
%  discretize_continuous_vals - discretizes continuous variables
%  data2zscore                - converts raw data into zscores
%  zscore2data                - converts zscores to data
%  pca                        - Principal components analysis
%  bin                        - bins continuous data by measuring equally sized bins
%  decile1                    - bins continuous data by putting the same number of cases into each bin
%  ranks                      - converts raw values to ranks - taken from XXXX's library
%
% VARIABLE SELECTION
%
%  stepwize                        - can be used to determine most significant predictors
%  dtree_calc_significance_wrapper - analyzes a computed decision tree model to determine variable contributions
%  entropic_significance           - information value of variable values on a dependent
%  entropy                         - entropy of a distribution
%  cross_entropy                   - cross-entropy for two distributions
%  correlation                     - Pearson correlation between two vectors
%  rank_correlation                - Spearman rank correlation between two vectors
%  distance                        - Euclidean distance between two vectors
%  dist                            - Euclidean distance matrix between two matrices
%  corrmat2                        - Pearson correlation matrix between two matrices
%  corrmat                         - Pearson correlation matrix formed from one matrix (analogous to corrcoef in Matlab language)
%  CONTINUOUS_SIGNIFICANCE - TIDY UP
%  DISCRETE_SIGNIFICANCE - TIDY UP
%
% IMPUTATION
% 
%  impute_missing_vals   - performs four types of imputation
%  timeseries_imputation - imputate timeseries data
%  sum_missing           - same as sum except avoids missing values  
%  mean_missing          - same as mean except avoids missing values
%  min_missing           - same as min except avoids missing values
%  max_missing           - same as max except avoids missing values
%  correlation_missing   - same as correlation except avoids missing values
%  distance_missing      - same as distance except avoids missing values
%  data2zscore_missing   - same as data2zscore except avoids missing values
%
% HYPOTHESIS TESTING
% 
%  ttest_indep         - T-test for independent samples
%  ttest_paired        - T-test for paired samples
%  f_statf             - F statistic
%  compare_two_groups  - runs statistical tests on two vectors of different size.
%  compare_two_groups2 - runs statistical tests on two vectors of different size. 
%  nchyp               - non-central hypergeometric distribution
% 
% OTHER
% 
%  showresults                   - takes a 2 column vector (x1 x2) and a dependent (y) and creates a 3Dimensional surface
%  mean_nomemory                 - compute the mean so as not to blow memory on large matrices
%  correlation_nomemory          - compute correlation so as not to blow memory
%  string_load                   - load in a column vector of strings from a file
%  
% SEE ALSO
% 
%  _mds                          - multidimensional scaling
%  _timeseries                   - timeseries routines
%  _profitopt                    - optimization routines
%  _boyd                         - deciling routines
%  _waldo                        - image processing routines
% 
% BTOOLKIT: NUMERICAL ROUTINES FOR MATLAB
% 
% written by Brendan Kitts 1994-2002. Freely available for all scientific uses, 
% for the betterment of mankind. Please email enhancements to bkitts@excite.com
% 
% Please cite as: 
% Kitts, B. (2017), BToolkit: Numerical routines for Matlab,
% http://www.appliedaisystems.com/code/BToolkit.zip
 
accuracy_estimate_mean2.m          impute_missing_vals.m              
accuracy_estimate_mean3.m          ind_missing_predict.m              
accuracy_of_model.m                initialize_centroids.m             
adaline_eval.m                     isnum.m                            
adaline_train.m                    k_nearest_neighbour.m              
agglomerative_cluster.m            kmeans.m                           
bayesian_train.m                   kmeans2.m                          
bboost.m                           kmeans_batch.m                     
bboost_eval.m                      kmeans_batch2.m                    
binary_cluster.m                   kmeans_batchc2.m                   
binary_eval.m                      kmeans_mean.m                      
bintargs.m                         knn.m                              
bp.m                               lift.m                             
btree_width.m                      lift2.m                            
chisquare_incomplete.m             linear_reg_eval.m                  
collapse_colinear_vars.m           linear_reg_train.m                 
compare_two_groups.m               loess_adaline.m                    
compare_two_groups2.m              logistic_error.m                   
confusion_matrix.m                 logistic_eval.m                    
confusion_matrix2.m                logistic_train.m                   
construct_valtree.m                logit_error.m                      
contents.m                         macqueen.m                         
convert_to_ranks.m                 mat2im.m                           
corr_with_dep.m                    max_missing.m                      
correlation.m                      maxval.m                           
correlation_missing.m              mean_missing.m                     
corrmat2_nomemory.m                mean_nomemory.m                    
create_vander.m                    memlookup_train.m                  
create_vanderf.m                   min_missing.m                      
cross_entropy.m                    mlp_error.m                        
cumm.m                             mlp_eval.m                         
data2im.m                          mlp_train.m                        
data2valtree_index.m               multistart_kmeans.m                
data2zscore.m                      nb_eval.m                          
data2zscore_missing.m              nb_train.m                         
decodebin.m                        nchyp.m                            
discretize_continuous_vars.m       opt_adaline.m                      
display_accuracy_multiple_vars.m   pca.m                              
display_intensitymat.m             percent_error.m                    
distance.m                         percentile.m                       
distance_missing.m                 point_biserial.m                   
distmat.m                          polyf.m                            
dscale.m                           polyff.m                           
dtree_calc_significance.m          polyfitf.m                         
dtree_calc_significance_wrapper.m  polyfitm.m                         
dtree_eval.m                       post_cluster_mean.m                
dtree_train.bak.m                  pow_error.m                        
dtree_train.m                      ppoly_eval.m                       
dtree_train2.m                     ppoly_train.m                      
dtree_traverse.m                   rank_correlation.m                 
dummyvar.m                         rankcorr.m                         
dummyvarf.m                        ranks.m                            
encode_data_using_valtree.m        rbf_eval.m                         
entropic_significance.m            rbf_train.m                        
entropy.m                          remove_duplicates.m                
estimate_mean_newregion.m          remove_ids.m                       
exp_error.m                        replace_nulls_with_mean.m          
expfit.m                           samplerows.m                       
exponential_smooth_error.m         show_splines.m                     
exponential_smooth_eval.m          showresults.m                      
f_stat.m                           sigmoid.m                          
f_statf.m                          spline_activ.m                     
false_positives.m                  spline_eval.m                      
find_closest.m                     spline_train.m                     
find_closest2.m                    split_node.m                       
find_closest_missing.m             std_missing.m                      
find_minimal_present_set.m         stepwize.m                         
find_same_mv_pattern.m             string_load.m                      
find_sig_cluster_attributes.m      sum_missing.m                      
find_sig_corrmat_els.m             threshold.m                        
force_mean_std.m                   timeseries_imputation.m            
gaussianD.m                        triangle.m                         
gaussianF.m                        ttest_indep.m                      
gaussiandata.m                     ttest_paired.m                     
graph_btree.m                      ttestb.m                           
graph_btree_wrapper.m              ttestc.m                           
graph_unordered_btree.m            valtree_index2data.m               
graph_unordered_btree2.m           vanderid_to_varid.m                
graph_unordered_btree_wrapper.m    var.m                              
gutenberg_error.m                  zscore2data.m                      
hypergeometric_graphs.m            
