function [corr,supp] = correlation_missing(x,y,missing_val,metric,proj);
% 
% [corr,supp] = correlation_missing(x,y,missing_val,metric,proj);
% 
% metric==0 (or not supplied) correlation
% metric==1 distance
% metric==2 meanabs distance
% proj special matrix multiplier to apply when comparing each case. 
% proj is the same size as x down, and provides a dimension multiplier to apply
% to each dimenson, for a case, 
%
% r=rand(4,5);d=correlation_missing(xplot(1:4,1:5)',xplot(1:4,1:5)',nan,2,r');
% is equivalent to
% mean(r(1,:).*abs(xplot(1,1:5)-xplot(2,1:5)))
%
% EXAMPLE
%
% r1=rand(10,2);r2=rand(10,2);correlation_missing(r1,r2,-9), % gives the correlation
% 
% % To calculate distance_missing between the two, we assume we want to calc
% % distance between each of the cases of r1 and r2 (there are 10 cases)
% % Therefore, we call the following:
%
% correlation_missing(r1',r2',-9,1);
% % recall, r1' 
% 

% Input parameters
% ----------------

if nargin<5, proj=ones(size(x));end;
if nargin<4, metric=0;end;
if metric<0 | metric>2,
   fprintf('metric is 0, 1, or 2 (correlation, distance, meanabs distance)\n');
end;

if metric==0,
xmean = mean_missing(x,missing_val);
ymean = mean_missing(y,missing_val);
end;

corr = zeros(size(x,2),size(y,2));
supp = corr;

x_present = zeros(size(x));
for i1=1:size(x,2),
   if isnan(missing_val), xpresent = find(~isnan(x(:,i1)));
   else xpresent = find(x(:,i1)~=missing_val);end;
   
   if metric==0,
      % correlation, subtract the mean from this xdimension
      xx = x(xpresent,i1)-ones(size(xpresent,1),1)*xmean(1,i1);
   else 
      xx = x(xpresent,i1);
   end;
   
  for i2=1:size(y,2),
    if isnan(missing_val), ypresent = find(~isnan(y(xpresent,i2))); 
    else ypresent = find(y(xpresent,i2)~=missing_val); end;
    
    if isempty(ypresent) | isempty(xpresent),
       if metric==0,
         % correlation
         corr(i1,i2)=0;
         supp(i1,i2)=0;
       else
          % any distance metric
          corr(i1,i2)=nan;
          supp(i1,i2)=0;
       end;
    else
      xf = xx(ypresent);
      if metric==0,
        % correlation
        yf = y(xpresent(ypresent),i2) - ones(size(ypresent,1),1)*ymean(1,i2);   
        corr(i1,i2) = correlation(xf,yf);
        if isnan(corr(i1,i2)), corr(i1,i2)=0;end;
      end;
      if metric==1, 
        % L2 distance
        yf = y(xpresent(ypresent),i2);
        corr(i1,i2) = distance(xf',yf');
      end;
      if metric==2,
         %keyboard
         % meanabs distance
         yf = y(xpresent(ypresent),i2);
         corr(i1,i2) = mean_missing(proj(xpresent(ypresent),i2).*abs(xf-yf),nan);
      end;
      supp(i1,i2) = size(ypresent,1);
    end;
  %keyboard
end;
  fprintf('%d ',i1);
end;

if 0,

x_present = zeros(size(x));
for i=1:size(x,2),
  if isnan(missing_val),
    present = find(~isnan(x(:,i)));
  else
    present = find(x(:,i)~=missing_val);
  end;
  x_present(present,i) = ones(size(present,1),1);

  if isnan(missing_val),
    present = find(~isnan(y(:,i)));
  else
    present = find(y(:,i)~=missing_val);
  end;
  y_present(present,i) = ones(size(present,1),1);
end;

end;
