function corr = corrmat2_nomemory(x,y,chunksize,missingVal);
%
% SYNOPSIS
% 
% corr = corrmat2_nomemory(x,y,chunksize,missinVal);
%
% DESCRIPTION
% 
% same as corrmat2, except use when running low on memory
% a full x matrix is correlated against y(:,1:chunksize)
% Therefore, the y matrix must be the larger
% Its the best I could do at short notice
% 
% SEE ALSO
%
% corrmat2:            Fast matrix correlation
% correlation_missing: Matrix correlation including missing values
% corrcoef:            Matlab language's correlation matrix
% correlation:         vector correlation
% rank_correlation:    rank correlation
% dist2:               Fast matrix distance
% distance:            vector distance
% 

if nargin<3, chunksize = 100; end;
corr = zeros(size(x,2),size(y,2));
N = size(y,2);
it=1;
no_iterations = ceil(N./chunksize);
while 1,

  % Figure out lower and upper range of chunk
  % ------------------------------------------
  
  lower  = (it-1).*chunksize+1;
  upper  = it.*chunksize;
  if upper>=N, upper=N; end;
  range1 = [lower:upper]';

  if nargin>3, 
    % missing value version
    corr_range = corrmat2(x,y(:,range1),missingVal); 
  else   
    corr_range = corrmat2(x,y(:,range1));
  end;

  % append the correlation submatrix to the complete matrix
  corr(:,range1) = corr_range;
      
  if rem(it,5)==0, fprintf('%d/%d ',it,no_iterations); end;
  it=it+1;
  if upper>=N, break; end;
end;

