function [xrecoded,count,no_vals] = data2valtree_index(x,valtree,missing_val);
% [xrecoded,count,no_vals] = data2valtree_index(x,valtree,missing_val);
%
% let's say a valtree has been constructed previously (this is often the case)
% We're now given a new set of x data. In some algorithms like naive bayes,
% its desirable to recode the data from its real values, to become an *index*
% into valtree. Therefore, xrecoded(10,5)=3 means the value is equal to valtree(2,5)
% x(row,dim)=valtree(xrecoded(row,dim),dim)

count = zeros(size(valtree));
for d=1:size(valtree,2),
   
   % find number of values in this valtree dimension
   if isnan(missing_val),
      nuh = min(find(isnan(valtree(:,d))))-1;
      % id position down of the missing_val
   else
      nuh = min(find(valtree(:,d)==missing_val))-1;
   end;
   
   if isempty(nuh), 
      fprintf('Cannot find missing_val %f in valtree dimension %d\n',missing_val,d);
      fprintf('it is likely that the missing_val has been mis-specified\n');
   else
     no_vals(d) = nuh;
   end;
  
   remaining = [1:size(x,1)]';
   for i=1:no_vals(d)+1, % includes the missing_val
      % for each value in this dimension
      if i==no_vals(d)+1 & isnan(missing_val),
         jj=find(isnan(x(remaining,d)));
      else
        jj=find(x(remaining,d)==valtree(i,d));            % find(data==vals(i,:));      
        % find out which x values in this dim match this value
      end;
      xrecoded(remaining(jj),d)=ones(size(jj,1),1).*i;
      % recode these xs to the index number into the valtree
      
      count(i,d) = size(jj,1);
      % and we can report the number of xs which have this value
      
      remaining = remove_ids(remaining,jj);
   end;
   % we have now searched through all values in this dim, including missing_val
   % if there are any remaining indices, then these are mistakes - values which
   % exist in the new x data, but which are not in the valtree.
   % What to do with these? Issue a warning and code their value as missing. 
   % Ideally the user should re-construct his/her valtree, and then re-perform
   % this data2valtree_index mapping
   if ~isempty(remaining),
      fprintf('Found data values in dimension %d which do not exist in the valtree dimension %d.\n',d,d);
      fprintf(' This means that the valtree was not generated using all of the data. Suggest\n');
      fprintf(' re-generating valtree on all of the data, to at least cover all possible values.\n');
      fprintf(' For now outstanding values which need to be recoded to valtree indices will be set to\n');
      fprintf(' missing_val. This will be problematic, since these real cases and missing may\n');
      fprintf(' signify different things.\n');
  end;
end;
