function [zscore,datamean,datastd] = data2zscore(data);
%
% function [zscore,datamean,datastd] = data2zscore(data);
%
% given a set of data, this normalizes the data to 0 mean, unit
% variance columns. datamean and datastd should be retained, so that
% any results can be retransformed back from zscores into the original
% metric space representation.
%
% SEE ALSO zscore2data

zscore = zeros(size(data));
datamean = mean(data);
datastd  = std(data);
i = find(datastd>0);
j = find(datastd==0);
k = find(isnan(datastd));

% preserved so that zscores can be transformed back into original data

if ~isempty(i),
% the formula is, mean-x / std
  zscore(:,i) = ( data(:,i) - ones(size(data,1),1) * datamean(:,i) ) ./ ...
     ( ones(size(data,1),1) * datastd(:,i)  );
end;
if ~isempty(j),
  zscore(:,j) = data(:,j);
end;
