function display_accuracy_multiple_vars(series,series_str,measure_str,axistype,legpos);
% display_accuracy_multiple_vars(series,series_str,measure_str,axistype,legpos);
%
% series is series_size down, by numberofvars across, by measures across.
% For each additional measure, a new plot is opened up sideways.
% On this plot, all series are compared against each other.
% series_str is numberofvars down, a character array with names of the series
% measure_str is nomeasures down, a character array with names of measures
%
% axistype is optional, a vector of 1,2,or 3 to indicate how to generate the axes.
% 1 means correlation plot (0..1), 2 means meanabs plot, 3 means correlation plot (-1..1)
%
% EXAMPLE
% figure(1);dat(1:10,1:2,1)=[[1:10]' [6:15]'];dat(1:10,1:2,2)=[[11:20]' [1:10]'];display_accuracy_multiple_vars(dat,['aa';'bb'],['meas1';'meas2']);
%

stylestr = [ ...
'- '; ...
'x-'; ...
'o-'; ...
'*-'; ...
'+-'; ...
's-'; ...
'>-'; ...
'^-'; ...
'd:'; ...
': '; ...
'-.'];

if nargin<5, legpos = 4; end;
if nargin<4, axistype=-9.*ones(1,size(series,2));end;
   
nomeasures = size(series,3);
novars = size(series,2);
series_size = size(series,1);

if novars>size(stylestr,1),
   fprintf('too many vars to plot neatly, try it with fewer vars\n');
   return;
end;

clf;
for meas=1:nomeasures,
   subplot(1,nomeasures,meas);hold on;
   legstr = [];
   for v=1:novars,
      plot(1:series_size,squeeze(series(:,v,meas)),stylestr(v,:));
      if meas==1,legstr = [legstr sprintf(' ''%s'', ',series_str(v,:))];end;
   end;
   
   % Change axes
   % ------------
   
      if axistype(1,meas)==1, % correlation plot
         axis([1 series_size 0 1]);
      end;
      if axistype(1,meas)==2, % meanabs plot
         vv=axis;axis([1 series_size 0 vv(4).*1.5]);
      end;
      if axistype(1,meas)==3, % correlation plot
         axis([1 series_size -1 1]);
      end;
      if axistype(1,meas)~=-9,
         axis([1 series_size 0 axistype(1,meas)]);
      end;
      
      % Add legend
      % -----------
      
   if meas==1,
     legstr = legstr(1,1:size(legstr,2)-2); % gets rid of the last comma
     eval(['h=legend(' legstr ',legpos);']);
     set(h,'fontsize',8);
     %set(h,'font','timeroman');
   end;
   ylabel(measure_str(meas,:));
   grid on;   
   
end;
