function [dout,novals,strout,valtree] = dummyvarf(dat,nodestr,valtree,missingVal);
% 
% [dout,novals,strout,valtree] = dummyvarf(dat,nodestr,valtree,missingVal);
%
% Convert categorical variable into a series of indicator 1,0 variables, where 
% 1 means == category, and 0 means not 
%
% If valtree is provided, it creates dummy categories based on the values in valtree
% If valtree is not provided, it creates dummy categories from the data, dat
% 
% 

norows=size(dat,1);
nodims=size(dat,2);


dout = [];
strout=[];

if nargin>1,
strout = [];
maxstr = size(nodestr,2)+100;
end;

if nargin<4,
  missingVal = -9999;
end;

if nargin>2,
  % valtree supplied
  % Check the valtree is correct
  if size(dat,2)~=size(valtree,2), 
    fprintf('valtree and dat do not have the same number of columns');
    return;
  end;
  for dim=1:size(valtree,2),
    novals(dim) = min(find(valtree(:,dim)==missingVal))-1;
  end;
else
  % valtree not supplied, make one
  [valtree,novals] = construct_valtree(dat,missingVal);
end;
  

for dim=1:nodims,
  %u = unique(dat(:,dim)); % find all unique values
  %novals(dim) = size(u,1);
  u = valtree(1:novals(dim),dim);
  code = zeros(norows,novals(dim)); % initialize the dummy code
  % This will have novals across, one for each category, and norows down
  remaining = [1:norows]';
  for val=1:novals(dim),
  %if dim==7 & val==48, keyboard; end;
    if isempty(remaining), end;
    % This kind of error is very possible. It means that we created a valtree once, and 
    % now are coding a new set of data, and the new set of data doesn't have all of the values
    % that were seen in the initial training.
    i=find(dat(remaining,dim)==u(val));
    code(remaining(i),val) = ones(size(i));
    remaining = remove_ids(remaining,i);
  end;
  if ~isempty(remaining), fprintf('internal error 2 detected. ');fprintf('internal error 1 detected. Some of the data row values did not match a value in the valtree. The valtree was probably constructed on a different set of data to that which is now being turned into dummy variables. This is a serious error. For now, those additional rows are being dropped from the dummy codings');end;

  % Append the new dummycode
  dout    = [dout sparse(code)];

  if nargin>1,
    newvars = char(ones(novals(dim),maxstr).*32);
    newvars(1:novals(dim),1:size(nodestr,2)) = ones(size(u,1),1)*nodestr(dim,:);
    new = num2str(u);
    newvars(1:novals(dim),size(nodestr,2)+1:size(nodestr,2)+size(new,2)) = new;
    strout = [strout;newvars(:,1:maxstr)];
  end;
  
  fprintf('.');
end;

% Resize the strout
% -----------------

%keyboard
if nargin>1,
  i=find(sum(double(strout)-32)~=0);
  maxlen = max(i);
  strout = strout(:,1:maxlen);
end;   

figure(1);clf;barh(novals);
%ylabel('Dimension');
ylabel('Dummy categories');
if nargin>1,
  %set(gca,'yticklabel',nodestr);
  for i=1:size(nodestr,1),
    text(novals(i),i,deblank(nodestr(i,:)));
  end;
end;

figure(2);clf;spy(dout);
drawnow;