function result = encode_data_using_valtree(data,valtree,missing_val,method);
% 
% encoded_data = dataencode_data_using_valtree(data,valtree,missing_val,method);
% method==1 use nearest neighbour which is fast but could outstrip memory if a lot of data is being encoded
% method==2 use slow for loop which puts no strain on memory
%

if nargin<3, fprintf('Not enough parameters\n');help encode_data_using_valtree;return;end;
if nargin<4, method==1; end;

fprintf('Encoding data using valtree\n');
fprintf(' missing_val %f\n',missing_val);
fprintf(' method %d ('); 
if method==1, fprintf('nearest neighbour, memory intensive)\n');end;
if method==2, fprintf('for loop, slow)\n');end;
fprintf('\n');

% Determine where the end of each valtree is
no_of_vals = zeros(1,size(valtree,2));
for dim=1:size(valtree,2),
  no_of_vals(1,dim)=min(find(valtree(:,dim)==missing_val)) - 1;
  if isempty(no_of_vals(dim)), 
    fprintf('Could not find missing_val %f in dimension %d. valtree may be defective\n',missing_val,dim);
    return;
  end;
end;
  
if method==1,
  tic,
  for dim=1:size(data,2),
    if no_of_vals(dim)==0,
      result(:,dim) = zeros(size(data,1),1);
      fprintf('** Error: no values besides missing val (%f) detected for dimension %d, valtree may be defective?',missing_val,dim);
    else
      if no_of_vals(dim)==1,
        result(:,dim) = ones(size(data,1),1)*valtree(1,dim);
      else 
        % a number of no_of_vals(dim) greater than 1
        nuh=k_nearest_neighbour([data(:,dim) data(:,dim)],valtree(1:no_of_vals(dim),dim)*ones(1,2),valtree(1:no_of_vals(dim),dim)*ones(1,2),1);
        result(:,dim) = nuh(:,1);
      end;
    end; 
    fprintf('%d/%d ',dim,size(data,2));
  end;
  fprintf('Elapsed time %d\n',toc);
end;

if method==2,
for i=1:size(data,1),
  for dim=1:size(data,2),
    [a,i]=min(abs(data(i,dim) - valtree(1:no_of_vals(1,dim),dim)));
    result(i,dim) = valtree(i,dim);
  end;
  if rem(i,1000)==0, fprintf('%d/%d ',i,size(data,1));end;
end;
end;
