function entropic_significance(x,y,missing_val,xdim_focus,experiments,ygivenx,nodestr); % valtree,cattree,experiments,nodestr); %,valtree,cattree,prob_xval_given_yval);
% entropic_significance(x,y,missing_val,xdim_focus,experiments,ygivenx,nodestr);
%
% Quantifies the significance of each xdimension and dimension value
% for predicting each yvalue. All algorithms assume that x and y have discrete
% values.
% experiments==1 means normal, experiments==2 means skip KL distance
%
% EXAMPLE
% no_cases=100;x=round(rand(no_cases,4).*5);y = x(:,1)+round(0.2.*x(:,3));missing_val=-9;
% entropic_significance(x,y,-9,[1 2 3 4]);
%
% SEE ALSO
% discretize_continuous_vars(x,bins,method,missing_val)
% entropy(x)
% cross_entropy(x,y,missing_val,absolutediff,protectzero)
% accuracy_of_model(x(:,1),y);
% entropic_significance(x,y,missing_val,xdim_focus,experiments,ygivenx,nodestr)
% % experiments=0, ygivenx=1

% Input parameters
% -----------------

   caseweights      = ones(size(x,1),1);
   missing_handling = 1;

   if nargin<3,
      missing_val=-9;
   end;
   
   if nargin<4,
      xdim_focus = [1];
   end;
   
   % Create cattree
   % --------------
   
%   if nargin<6, % no cattree
     cattree = construct_valtree(y,missing_val);
 %  end;
   
   % Create prob_xval_given_yval
   % ---------------------------
   
  % if nargin<5,
     % no valtree - construct valtree and cattree
      [yest,prob,probdist,prob_xval_given_yval,yprior,valtree,cattree] = ...
         nb_train(x,y,missing_val,missing_handling,caseweights);
  % else
      % nargin >=5, valtree was supplied, cattree created above if necessary, 
      % valtree and cattree should now exist
  %    [yest,prob,probdist,prob_xval_given_yval,yprior] = ...
  %       nb_train(x,y,missing_val,missing_handling,caseweights,valtree,cattree);
  % end;
   
   % Get no_vals and no_catvals
   % ---------------------------

     maxvals = -1;   
     for d=1:size(valtree,2),
       if isnan(missing_val),
          no_vals(d,1)=min(find(isnan(valtree(:,d))))-1;
       else
          no_vals(d,1)=min(find(valtree(:,d)==missing_val))-1;
       end;
       if (no_vals(d,1)>maxvals), maxvals = no_vals(d,1); end;
     end;
   
   no_catvals = size(cattree,1)-1;


  % xprior
  % ------
  
  %fprintf('Calculating xprior and prob_yval_given_xval\n');
  % TODO: Find a faster way of doing the below
  
  xprior = zeros(maxvals,size(valtree,2));
  for d=1:size(valtree,2),
    for xval=1:no_vals(d,1)+1,
      xprior(xval,d) = size(find(x(:,d)==valtree(xval,d)),1) ./ size(x,1);
    end;
  end;
  
  if (nargin<6), ygivenx = 1; end;
  if (nargin>=6), ygivenx = (ygivenx>0); end;

  if (ygivenx==1),   
  %yout = zeros(size(valtree,1),size(x,2),no_catvals);
  %prob_yval_given_xval = zeros(size(yout));
  prob_yval_given_xval = zeros(size(x,2),size(valtree,1),no_catvals);
  for d=1:size(valtree,2),
    for xval=1:no_vals(d,1)+1,
      %yout(xval,d,1:no_catvals) = squeeze(prob_xval_given_yval(d,xval,1:no_catvals));
      %prob_yval_given_xval(xval,d,:) = (squeeze(yout(xval,d,1:2)) .* yprior(:,1)) ./ size(x,1) ./ xprior(xval,d);
      % worked
      prob_yval_given_xval(d,xval,:) = ((squeeze(prob_xval_given_yval(d,xval,1:no_catvals)) .* yprior(:,1)) ./ size(x,1) ./ xprior(xval,d));
    end;
  end;
  prob_xval_given_yval = prob_yval_given_xval;
  end; 
      
  if nargin<5,
    experiments=1;
  else 
    experiments=(experiments>0);
  end;

  nodestr_supplied=0;  
  if nargin>=7, % nodestr supplied
    if size(nodestr,1)>=size(x,2),
      nodestr_supplied=1;
    end;
  end;
  
     
   % -----------------------------------------------------------------------
   
   
   
   % Main procedures
   % ===============
   
   % Cross-entropy == KL-distance 
   % =============================
   % Method from 
   % Koller, D. and Mehran, S. (ref), Toward Optimal Feature Selection, Ref
   %
   % Kullback and Leibler, (1951), On Information and Sufficiency, 
   % Annals of Mathematical Statistics, Vol. 22, pp. 76-86.
   % D(correctdist,approxdist) = sum over all x datapoints (correctdist .* 
   % log(correctdist./approxdist)
   % Problem with this is if approx is [0 0.3 0.2 0.1]. log([0.1 0.2 0.2 0.1]/approx) 
   % gives log(inf)
   % -------------
   % Cross_entropy(Pr(y|x),Pr(y|x-xdim))
   % Percents expressed in ./ Cross_entropy(Pr(y|x)-Pr(y))
   % get prob dist for each xpattern, using full 
   % get prob dist for each xpattern_new (using bayesian method), using only x
   % xpatterndist, xpattern_newdist
   % multiply by no_xpatterns .* kldist(xpatterndist,xpatternnew_dist)
   % get prob dist for each row, taking away x
   % kldistance is roworigdist rownewdist 
   % implicitly multiplied by the number of cases which have that pattern
   % Prove kldistance(xpatterndist,xpatternnew_dist).*no_patterns 
   % equal to sum(kldistance(xpatterndist,xpatternnew_dist) foreach row)
experiments=0;  
  if experiments==1, 
   [xpattern,xpatterndist,support,estdist,yest] = bayesian_train(x,y,missing_val);
   
   for xdim=1:size(x,2),
      % calculate case-weighted cross-entropy between xpatterndist original, and
      % xpatterndist_new where this uses only x
      [xpattern_new,xpatterndist_new,support_new,estdist_new,yest_new] = ...
         bayesian_train(x(:,xdim),y,missing_val,valtree(:,xdim),cattree);
      
      % calculate cross-entropy (implicitly weighted by the number of each pattern)
      xalone(xdim) = cross_entropy(estdist,estdist_new,[],1,1)./size(estdist,1);
      % entropic distance from y|x(:,xdim) to y|x(:,:), smaller the better x is 
      
      allexceptxdim = remove_ids([1:size(x,2)]',xdim);
      [xpattern_new,xpatterndist_new,support_new,estdist_new,yest_new] = ...
         bayesian_train(x(:,allexceptxdim),y,missing_val,valtree(:,allexceptxdim),cattree);
      
      xremove(xdim) = cross_entropy(estdist,estdist_new,[],1,1)./size(estdist,1);
      % entropic distance from y|x(:,allexceptxdim) to y|x(:,:),
      % larger the better x is (it contributes more, and removing x results in a
      % violence to the y distribution). If there is only a small diff, it means
      % you can remove x and loose almost no info - the ydist will be almost the same
   end;
   
   % Assume prior is the simplest possible model
   % This is the same as the very worst possible prob(y|x1) model, above
   % Then the KL-distance between yprior (for all cases) and y|everything
   % gives us the range we need to span
   
   kl_full_minus_yprior = cross_entropy(ones(size(estdist,1),1)*(yprior'./sum(yprior)),estdist,[],1,1) ./size(estdist,1);
   % larger means larger spread
   
   % All xalone can now be seen as positive contributions away from ideal (at
   % dist=kl(full-yprior) towards 0.
   
   fprintf('\n');
   fprintf('Cross-entropy entropic discrepancy from removal of each dimension\n');
   fprintf('Dimension\t KL(Pr(y|xfull)-Pr(y|xallexceptxdim))\tKL(Pr(y|full)\n');
   fprintf('         \t/KL(Pr(y|xfull)-yprior)              \t - Pr(y|xallexceptxdim))\n');
   fprintf('---------------------------------------------------------------------------\n');
   for xdim=1:size(x,2),
      fprintf('%d    \t                     %5.2f%%        \t%f\n',xdim,100.*xremove(xdim)./kl_full_minus_yprior,xremove(xdim));
   end;
   fprintf('---------------------------------------------------------------------------\n');
   fprintf('\n');   
   end; % experiments~=1
   
   % Entropy drop
   % ============
   % it is possible to calculate the entropy drop on each dimension-value
   % nb_train gave us the prob_xval_given_yval (or equivalently),
   % prob(yval|xval)
   % CHECK THIS, THIS IS WEIRD
   
   ent_original = entropy(yprior./sum(yprior));   
      
   ent          = zeros(size(valtree,1),size(x,2));
   ent_overall  = zeros(1,size(x,2));
   no_cases     = zeros(size(valtree,1),size(x,2));
   chi          = zeros(size(valtree,1),size(x,2));
   chi_overall  = zeros(1,size(x,2));
   gain_overall = zeros(1,size(x,2));
   yout         = zeros(size(valtree,1),size(x,2),no_catvals);
   
   for xdim=1:size(x,2),
      remaining = [1:size(x,1)]';
      for xval=1:no_vals(xdim)+1, % includes missing_val
         
         % Entropy
         % we now have a prob(yval|xval),
         % get the entropy of all of these
         ent(xval,xdim) = entropy(squeeze(prob_xval_given_yval(xdim,xval,1:no_catvals)));
         
         % need number of cases matching xval
         i = find(x(remaining,xdim)==valtree(xval,xdim));
         no_cases(xval,xdim) = length(i);       
         % Chi-square == sum(prob(a,c)^2 / p(a).p(c) )      
         if isempty(i),
            ent(xval,xdim)=nan;
            chi(xval,xdim)=nan;
            yout(xval,xdim)=nan;
         else
            chi(xval,xdim) = sum((squeeze(prob_xval_given_yval(xdim,xval,1:no_catvals))'.^2) ...
            ./ ((yprior'./sum(yprior)) .* no_cases(xval,xdim)./size(x,1)) );
         
            yout(xval,xdim,1:no_catvals) = squeeze(prob_xval_given_yval(xdim,xval,1:no_catvals));
         end;
         remaining=remove_ids(remaining,i);
      end;
      % combine all entropies
      ent_overall(1,xdim) = sum_missing((no_cases(1:no_vals(xdim)+1,xdim)./size(x,1)).*ent(1:no_vals(xdim)+1,xdim),nan);
      chi_overall(1,xdim) = sum_missing(chi(1:no_vals(xdim)+1,xdim),nan);
      gain_overall(1,xdim)= (ent_original-ent_overall(xdim)) ./ entropy(no_cases(1:no_vals(xdim)+1,xdim)./size(x,1));
      % sum all chi scores in this xdim
      fprintf('.');
   end;
   
   % Initialize figures
   for dim=1:max(size(xdim_focus)),
     figure(dim);
   end;
   
   
   % Graph Entropy Drop
   % Need a nice simple flat shaded pixel diagram
   figure(max(size(xdim_focus))+1);
   clf;
   entdrop = 100.*(ent_original-ent)./ent_original;
   ii=find(entdrop==100);
   entdrop(ii)=nan.*ones(size(ii,1),1);
   bar3(entdrop);
   xdim=1;
   set(gca,'ytick',[1:size(ent,1)]');
   set(gca,'yticklabel',strcat('val',num2str([1:size(ent,1)]'))); %num2str(valtree(1:no_vals(xdim)+1,xdim)));
   ylabel('value');
   set(gca,'xtick',[1:size(ent,2)]);
   xlabel('dimension');
   if nodestr_supplied,
    set(gca,'xticklabel',char(nodestr(1:size(ent,2),:)));
   else
    set(gca,'xticklabel',num2str([1:size(ent,2)]'));
   end;
   zlabel('entropy drop');
   title('Entropy drop');

   % Graph Chi
   figure(max(size(xdim_focus))+2);
   clf;
   bar3(chi);
   xdim=1;
   set(gca,'ytick',[1:size(chi,1)]');
   set(gca,'yticklabel',strcat('val',num2str([1:size(chi,1)]'))); 
   ylabel('value');
   set(gca,'xtick',[1:size(chi,2)]);
   xlabel('dimension');
   if nodestr_supplied,
    set(gca,'xticklabel',char(nodestr(1:size(chi,2),:)));
   else
    set(gca,'xticklabel',num2str([1:size(chi,2)]'));
   end;
   zlabel('Chi-square');
   title('Chi-square independent contribution');


      
   fprintf('\n\n');
   fprintf('Entropy criterion (entropy of y distribution %f)\n',ent_original./size(x,1));
   for xdim=1:size(x,2),
      fprintf('Dimension==value\t');
   end;
   fprintf('\n');
   for xval=1:size(valtree,1),
      for xdim=1:size(valtree,2),
         if xval<=no_vals(xdim)+1,
            fprintf('%8f==%5.2f%%\t',valtree(xval,xdim),100.*(ent_original-ent(xval,xdim))./ent_original);
         else
            fprintf('               \t');
         end;
      end;
      fprintf('\n');
   end;
   
   fprintf('------------------------------------------------------------\n');
   fprintf('ent ');
   for xdim=1:size(x,2),
      fprintf('          %5.2f%%\t',100.*(ent_original-ent_overall(xdim))./ent_original);
   end;
   fprintf('\n');
   fprintf('gain');
   for xdim=1:size(x,2),
      fprintf('          %5.2f%%\t',100.*gain_overall(xdim));
   end;
   fprintf('\n');
   fprintf('------------------------------------------------------------\n');   
   
   fprintf('\n\nChi-square criterion\n');
   for xdim=1:size(x,2),
      fprintf('Dimension==value\t');
   end;
   fprintf('\n');
   for xval=1:size(valtree,1),
      for xdim=1:size(valtree,2),
         if xval<=no_vals(xdim)+1,
            fprintf('%8f==%f\t',valtree(xval,xdim),chi(xval,xdim));
         else
            fprintf('               \t');
         end;
      end;
      fprintf('\n');
   end;
   fprintf('------------------------------------------------------------\n');  
   fprintf('chi');
   for xdim=1:size(x,2),
      fprintf('          %f\t',chi_overall(xdim));
   end;
   fprintf('\n');
   fprintf('------------------------------------------------------------\n');   
   
   % This should definitely include support, number of yval and xval
   fprintf('\nE(Y|xdim=xval)\n');
   for xdim=1:size(x,2),
      fprintf('DimensionXval==Yvalue\t');
   end;
   fprintf('\n');
   for xval=1:size(valtree,1),
      for xdim=1:size(valtree,2),
         if xval<=no_vals(xdim)+1,
            [m,mid]=max(yout(xval,xdim,1:no_catvals));
            fprintf('%8f==%f (%5.2f%% / %d)\t',valtree(xval,xdim),cattree(mid),100.*m,no_cases(xval,xdim));
         else
            fprintf('                            \t');
         end;
      end;
      fprintf('\n');
   end;
   
fig=1;
for xdim=xdim_focus;
   if 1,   % used to be 0
   fprintf('\nProb(Y=yval|x1=xval) for xdim=%d\n',xdim);
   for yval=1:no_catvals,
      fprintf('          %f\t',cattree(yval));
   end;
   fprintf('\n');
   for xval=1:no_vals(xdim)+1,
      for yval=1:no_catvals,
        fprintf('%8f\t%5.2f\t',valtree(xval,xdim),100.*yout(xval,xdim,yval));
      end;
      fprintf('\n');
   end;
   end;

  figure(fig);fig=fig+1;
  clf;bar3(squeeze(yout(1:size(valtree,1)-1,xdim,1:no_catvals)));
  set(gca,'ytick',1:no_vals(xdim)+1);
  set(gca,'yticklabel',num2str(valtree(1:no_vals(xdim)+1,xdim)));
  if nodestr_supplied,
    ylabel(strcat('Dimension: ',char(nodestr(xdim,:))));
  else
    ylabel(['x (dimension ' num2str(xdim,'%2d') ')']);
  end;
  set(gca,'xtick',1:no_catvals);
  set(gca,'xticklabel',num2str(cattree(1:no_catvals,1)));
  xlabel('y');
  zlabel('probability');
  title('Probability(y=yval|x(:,xdim)=xval) Shows any relationship');
end;

%save entropic_state.mat
keyboard

   % Symmetric entropy drop
   % Gini?
   % Correlation heuristic
   % 
   if 0,
   fprintf('Dimension\tEntropyDrop\n');
   for xdim=1:size(x,2),
      fprintf('%d\t%f\n',ent_original-ent_overall(xdim));
   end;
   end;
   % Entropy gain ratio
   
