function [err,yest] = exp_piecewise_linear_error(a,x,y,a3,w);
% 
% [err,yest] = exp_error(a,x,y,a3,w);
%
% a is a 2x1 vector of parameters, x is a column vector, and y is a column vector
% w is a vector of weights (optional)
% a3 is a fixed additive term that is passed in
% Computes the root sum of squared error (ie. the Euclidean distance) 
% between an exponential fit of a demand curve, g(x) and 
% the known demand y. This error can be used to optimize the exponential model.
% The exponential model can also be optimized using analytic derivatives
%
% EXAMPLE
% 
% a = fminsearch('exp_piecewise_linear_error',[0.5;0.4],[],x(:,7),y,1); % 1 is the beginning of the linear region
% yest = exp(a(2).*x).*a(1) + 1;
%
% figure(1);clf;[err,yest]=exp_piecewise_linear_error([1 -0.2],[1:100],[1:100],.2);figure(1);clf;plot([1:100],yest)
%  
% SEE ALSO
%
% exp_error, pow_error, gutenberg_error, adaline_error, polyfitf, 
% 

if nargin<5, w=1; end;

% Exponential region
yest = exp(a(2).*x).*a(1);  

% Linear region
i = find(yest<a3);
if ~isempty(i), yest(i)=a3.*ones(size(i));end;

% Calculate error
if nargin<5,
  err = norm(yest-y);
else
  err = sum(  (w .* (yest-y)).^2);
end;
