function f = f_stat(x,y,GPOLYD)
%
% f = f_stat(x,y,GPOLYD)
% x is the independent variable(s), y is the dependent
% GPOLYD is the degree of the polynomial to test for. Ordinarily use
% degree 1, but can check for higher order relationships too
%
% function will calculate r^2. This can be converted into an f statistic
% to give a measure of how much some variable accounts for the variance of
% some dependent (there are tables in most stat books which use the number
% of samples and the r value to give an f value.
%
% Data format:
%
% x = [1 2 3 4 5 ;     y = [5;
%      3 4 5 6 7 ;          6;
%      1 4 5 5 6 ];         7];
%
% x has 5 dimensions (corresponding to 5 variables which were recorded),
% and three points for each x there are three resulting y points
%
% Simple tests would be:
%  x = [1;2;3] y=[5;6;7] and GPOLYD=1. This would test linear correlation
%

disp(' ')
disp('f_stat.m')
disp(' ')

[downx,acrossx] = size(x);
[downy,acrossy] = size(y);
k = acrossx;
n = downx;
AVG_VALUE = ones(1,acrossx);
% this takes care of the polyf assumes there is an AVG_VALUE


disp('calculating polyfit');
GPOLY=polyfitm(x,y,GPOLYD);
disp('polynomial fitted');

for i=1:downx;
  Regression_error(i,:) = (y(i,:)-polyf(GPOLY,GPOLYD,x(i,:))).^2;
end
TotalRegressionVariance = sum(Regression_error)

[down,across] = size(y);

Meany = ones([down,across]) .* mean(y);
TotalVariance = sum( (y-Meany).^2 )

disp(['numerator:' num2str(k)])
disp(['denominator:' num2str(n-k-1)])
f = ((TotalVariance - TotalRegressionVariance) ./ k)  ...
    ./ (TotalVariance ./ (n-k-1));

% p = fpdf(

disp('to find significance, look up the f distribution table')
disp('(if I knew the function for f I could calculate significance automatically!)')
