function [f,r,meanerr,num,den,p] = f_statf(x,y,GPOLYD, ...
                                         criterion,xtrain,ytrain)
%
% [f,r,meanerr,num,denom,p] = f_statf(x,y,degree, ...
%                                   criterion,xtrain,ytrain)
%
% x is the independent variable(s), y is the dependent
% degree is the degree of the polynomial to test for. Ordinarily use
% degree 1, but can check for higher order relationships too
% 
% Optional parameters
%
% criterion, xtrain and ytrain are optional. Criterion is used
% to short-circuit the f calculation and return a different criterion:
% criterion==1, calculate f
% criterion==2, calculate r^2 and return
% criterion==3, calcualte mean abs error for fit and return
%
% xtrain and ytrain are used if we want to perform a fit on a
% training set, and then reserve x and y for testing the fit.
% x and y can be the same (if we want to train and test on the
% same data as in regular f), or different if we want to use
% the hold-out estimation.
% 
% Example
% 
% x = [1;2;3]; y=[5;6;7];
% f_statf(x,y,1);
%

% clean this up

[downx,acrossx] = size(x);
[downy,acrossy] = size(y);
k = acrossx;
n = downx;

if nargin<=4, 
  % no xtrain or ytrain specified, 
  % train and test on the same data
  xtrain = x;
  ytrain = y;
end;

% perform the fit
% ---------------

GPOLY = polyfitf(xtrain,ytrain,GPOLYD); % fit on the train
est   = polyff(GPOLY,GPOLYD,x); % estimate off the test


% return r or mean abs error
% --------------------------

if nargin>=4,
  if (criterion==2), % rsq only
    yy=y-mean(y);dd=est - ones(size(y,1),1)*mean(est);
    f = ((yy'*dd) / (norm(yy)*norm(dd)));
    % this isn't really f, this is an rsq
    % have to fix this corr so it works with multi dims
    % actually, that's canonical which is involved
    return
  end;
  if (criterion==3), % mean abs error only
    f= -1.* mean(mean(abs(y-est))); 
    % negative because larger F is better, here all errors are negative
    % and the higher they go (the closer to zero) the better
    % double mean in case y has more than one dim, in which
    % case we get a n-vec out and get the mean of it
    return
  end;
  if (criterion<1) | (criterion>3)
    disp(['unrecognized criterion ' num2str(criterion)]);
    disp('defaulting to F ');
    criterion = 1;
  end;
end;


% calculate f
% -----------

Regression_error = (y-est).^2;

%for i=1:downx;
%  Regression_error(i,:) = (y(i,:)-polyf(GPOLY,GPOLYD,x(i,:))).^2;
%end

TotalRegressionVariance = sum(Regression_error);

[down,across] = size(y);

Meany = ones([down,across]) .* mean(y);
TotalVariance = sum( (y-Meany).^2 );

%disp(['numerator:' num2str(k)])
%disp(['denominator:' num2str(n-k-1)])

f = ((TotalVariance - TotalRegressionVariance) ./ k)  ...
    ./ (TotalVariance ./ (n-k-1));

if nargout>=6,
  p = fpdf(f,k,(n-k-1));
  if p==0, 
    fprintf('WARNING: Outstripped numerical precision');
  end;
end;


% return values
% -------------

num     = k;
den     = n-k-1;
r       = (TotalVariance - TotalRegressionVariance) ./ TotalVariance;
meanerr = mean(sqrt(Regression_error));
