function [candidate_var,row]=find_minimal_present_set(data,n,missing_val);
% [var,row,candidate_history,n_history] = find_minimal_present_set(data,n,missing_val);
% 
% Tries to find the largest set of variables such that there are 
% at least n records which are full of real values.
% Does this by eliminating variables until a set with the n full records remains
% 
% candidate_var is the list of candidates
% candidate_history provides the list of candidates for each n
% candidate_history(i,1:i), % gives i columns
% n_history(i,1), % gives the number of rows this provides 
%
% EXAMPLE
%
% i = find(dep~=missing_val);
% trainvect = vect(i,:);traindep = dep(i,:);
% [candidate,row] = find_minimal_present_set(trainvect,10,-9);
% stepwize(trainvect(row,candidate),traindep(row,1),2,0.01,3);

candidate_var = [1:size(data,2)];
candidate_history = zeros(size(data,2),size(data,2));
n_history = [];

if isnan(missing_val),
  present  = ~isnan(data);
  missing  =  isnan(data); 
else
  present = data~=missing_val;
  missing = data==missing_val;
end;

% present is a matrix with 1s where there are non-missing vals
% to find out if there are any missing values, simply do
% sign(sum(missing')'), there will be a 1 in the location of missing
% vals


% This is very inefficient! 
% The conjunctive property exists on all columns,
% a ^ b ^ c ^ d
% Let's say we think about removing d,
% You only need to know all the missing vals on d, and 
% match against any 
% I think there is some way to do this efficiently

% determine the number of full rows
current_missing_rows = sum(sign(sum(missing')'));

while size(candidate_var,2)>0,
   
  if size(data,1)-current_missing_rows>n, 
    break;
    % candidate_vars are the answer
  else
    % find a dimension to remove
    % drop the dim with the max missing_vals

    best_var = 1;min_missing_rows = inf;
    for i=1:size(candidate_var,2),
      % count number of present_rows 
      remaining = setdiff(candidate_var,candidate_var(i));
      new_missing_rows = sum(sign(sum(missing(:,remaining)')'));
      %fprintf('%d ',new_missing_rows);
      if new_missing_rows <= min_missing_rows,
        min_missing_rows = new_missing_rows;
        best_var = i;
      end;
    end; % try to remove each variable to see the result
    % I believe this could be speeded up immensely by storing the 
    % mat in memory and understanding which 
    fprintf('Dropping %d won back %d rows\n',candidate_var(best_var),current_missing_rows - min_missing_rows);
    candidate_var = setdiff(candidate_var,candidate_var(best_var));
    current_missing_rows = min_missing_rows;
    %candidate_history(i,1:i) = candidate_var;
    %n_history(i,1) = size(data,1)-min_missing_rows;
  end; % if
end; % while1


row = find(sign(sum(missing(:,candidate_var)')')==0);
  
