function [corrlist] = find_sig_corrmat_els(c,num,s,sthresh);
%
% [corrlist] = find_sig_corrmat_els(c,num,s,sthresh);
%
% Given a correlation matrix, c, returns a corrlist 
% [down, across, corr]
% which is the num strongest positive and negative correlations
% in that correlation matrix
%
% To prevent this from returning elements on the
% diagonal (for example, the elements on the diagonal of
% corrmat2(a,a) will always be 1.0), call this as
% "find_sig_corrmat_els(c-tril(c));" This will replace the tri-lower
% diagonal with zeros, which should not (hopefully!) effect the
% correlation)
%

% Sort all correlations strongest to weakest corrs
% ------------------------------------------------

c=full(c);
if nargin>=3,
   s=full(s);
   % zero out all correlations lower than min support
   i=find(s<sthresh);
   c(i) = zeros(size(i,1),1);
end;

[null,id] = sort(-1.*abs(c(:))); 

% the -1 inverts the corrs, so higher the absolute corr, (or
% strength), the lower it is. When we sort, we get the strongest
% corrs first on the list


% Determine the down,across position of these correlations
% --------------------------------------------------------

  down = rem(id(1:num,1),size(c,1));
  z=find(down==0);
  if ~isempty(z),
    down(z,1) = ones(size(z,1),1).*size(c,1); 
  end;
  across = ceil(id(1:num,1)./(size(c,1)));


% Return (down,across,corr)
% -------------------------

if nargin>=3,
  corrlist(:,1:4) = [down across c(id(1:num)) s(id(1:num))];
else
   corrlist(:,1:3) = [down across c(id(1:num))];
end;
%for i=1:num,
%  corrlist(i,1:3) = [down(i,1) across(i,1) c(down(i,1),across(i,1))];
%end;





% =============================================================
%if 0,
%[a,b]=find(c2>=thresh);size(a,1),
%for i=1:size(a,1),
%  fprintf('%d\t%d\t%f\n',a(i),b(i),c2(a(i),b(i)));
%  highcorr(i,1:3) = [a(i) b(i) c2(a(i),b(i))];
%end;
%end;
