function data = force_mean_std(data,new_mean,new_std);
%
% function force_mean_std(data,new_mean,new_std);
%
%% let's say you've got a set of estimates on the test set which
%% are giving a good correlation, but you think the scale might
%% be off. Here's what to do. Get the mean and std for the training
%% set:
%
%m=mean(training_set);
%s=std(training_set);
%
%% and call force_mean_std...
%
%newdata = force_mean_std(data,m,s);
%
%% this forces newdata to have the same mean and std as the m and s. This just
%% linearly rescales and shifts the distribution, so doesn't harm it. (identical to
%% changing the intercept in regression). The newdata
%% will have the same correlation as before, but just be shifted according to 
%% the training data
%

dstd = std(data);
data = data - ones(size(data,1),1) * mean(data);
data = ones(size(data,1),1)*new_mean + ...
       ones(size(data,1),1)*new_std .* ...
       data ./ (ones(size(data,1),1)*dstd);

return
