function [err,yest] = gutenberg_error(a,x,y);
% 
% [err,yest] = gutenberg_error(a,x,y);
%
% a is a 4x1 vector of parameters, x is a column vector, and y is a column vector
% Computes the root sum of squared error (ie. the Euclidean distance) 
% between a gutenberg fit of a demand curve, g(x) and 
% the known demand y. This error can be used to optimize the gutenberg model.
% The gutenberg model can also be optimized using analytic derivatives
%
% EXAMPLE
%  
% a = fminsearch('gutenberg_error',rand(5,1),[],x(:,7),y);
% yest = a(1)-a(4).*x-a(2).*sinh(a(3).*x);
% 
% SEE ALSO
%
% exp_error, pow_error, gutenberg_error, adaline_error, polyfitf, 
% 


  yest = a(1)-a(4).*x-a(2).*sinh(a(3).*x);
  err = norm(yest-y);
  