function [newdata_nearest,    ...
          newdata_mean,       ... 
          newdata_regression, ...
          newdata_ordinarymean] = impute_missing_vals( ...
                                  data,                ...
                                  testdata,            ...
                                  missing_val,         ...
                                  min_training_pool,   ...
                                  dims_to_replace,     ...
                                  degree);
%
% [new1,new2,new3,new4] = impute_missing_vals(traindata,    ...
%                                        testdata,          ...
%                                        missing_val,       ...
%                                        min_training_pool, ...
%                                        dims_to_replace,   ...
%                                        degree);
%
% new1 == nearest neighbbour finds all vectors matching the current's missing val pattern,
% (but which have a val in the missing location), and then takes
% the nearest of these vectors, and uses its val.
% new2 == conditional mean finds the set of vectors which match the current, and have a 
% value in the missing position, and then takes the mean of these missing
% vals to fill in the value for the current row.
% new3 == conditional regression performs regression (should use a larger pool than normal if
% using this result)
% new4 == available mean replaces all missing values with the mean for entire column.
% This is different from the above two: The above two find 
% 
% dims_to_replace = [1 4 5 10 11 12]; % dims which we want to estimate
% dims_to_replace = -1 == replace all missing dims
%

for row=1:size(testdata,1), 
   disp(num2str(row));
   present = find(testdata(row,:)~=missing_val);
   missing = find(testdata(row,:)==missing_val);
   % all of these missing elements must now be estimated

   if dims_to_replace==-1,
     % continue
   else
     missing=intersect(missing,dims_to_replace);
     % only replace the dims_to_replace, and only replace
     % if that dim is actually missing in this example
   end;

   newdata_nearest(row,:)    = testdata(row,:);
   newdata_mean(row,:)       = testdata(row,:);
   newdata_regression(row,:) = testdata(row,:);
   % assign all the existent values
   
   for dim=missing,
      desirable_row = testdata(row,:);
      desirable_row(1,dim) = 0; % -1.*missing_val;
      % this is the dimension we want to find a value 
      % for. We place a legal value in it, and find all
      % other rows which match the entire row + missing val dim
      
         [candidates,vars_present] =               ...
            find_same_mv_pattern(desirable_row,    ... % row matching 
                                 data,             ...
                                 missing_val,      ...
                                 min_training_pool,... 
                                 dim);                % drop tabu
                     
      % we now have the pool
      
      if size(candidates,1)<0, keyboard;
      %   jj=find(data(:,dim)~=missing_val);
      %   newdata(row,dim) = -9; %mean(data(jj,dim));
      else
         % mean
         % ----
         
        newdata_mean(row,dim) = mean(data(candidates,dim));
        % the mean of the candidates value in this dim
        
        % nearest neighbour
        % -----------------
        
        % selects the closest vector from candidates in terms
        % of similarity based on present dimensions
        % (but this vector also happens to have a value for 
        % the missing value)

        [a,id] = min(distance( data(candidates,present), ...
                               ones(size(candidates,1),1)* ...
                               testdata(row,present)));             
        newdata_nearest(row,dim) = data(candidates(id,1),dim);
        % the value this closest vector has
        % in the missing location, is our estimate
        % for the data vector
        
        % regression
        % ----------
        
        w = polyfitf(data(candidates,present),data(candidates,dim),degree);
        %[v,deg]=stepwize(data(candidates,present),data(candidates,dim),2,0.1,degree);
        %w = polyfitf(data(candidates,present(v)),data(candidates,dim),deg);
        % regression from candidates who have dim present and have the
        % same present dims as our row, from their vector
        % without dim, onto dim
        % Use this same regression to estimate missing value for the
        % particular case of desirable_row
        newdata_regression(row,dim) = polyff(w,degree,testdata(row,present));
        %newdata_regression(row,dim) = polyff(w,deg,testdata(row,present(v)));
        
     end; % if no matches
   end; % for all missing values in the row
end; % for all rows in the data

newdata_ordinarymean = replace_nulls_with_mean(data,missing_val);
return

   
if 0,
for row=1:size(data,1),
   [candidates,vars_present, ...
    candidates_for_dim,num_candidates] =      ...
   find_same_mv_pattern(data(row,:), traininp, ...
                        missing_val,        ...
                        1);
                     
   % this is the base set of candidates in the population
   % which have the same set of values present as data(row,:)
   % Now we are going to impute each missing val of data.
   % Do this by finding all candidates which have the extra val
   % present.
   % note: this picks up specific matches as possible, so that we
   % get a decrease matrix which we can use in dropping dimensions
   % next.
   % (note that this will also drop dimensions to make this set)
   % return the list of decreases.
   % now we'll use this to drop dimensions if we need
   % candidates is a list of rowids which match the present pattern
   % vars_present is the number of vars present in the match set
   % num_candidates is a horizontal vector giving the number of matching
   % rows, for each present dimension. As dimensions increase, the
   % number of matching decreases. 

   present = find(data(row,:)~=missing_val);
   missing = find(data(row,:)==missing_val);
   % all of these elements must be estimated
   
   for dim=missing,
      % for this mval (dim), find all candidates which have this present.
      % this final set will be used to estimate the val
      
      cindex = find(data(candidates,dim)~=missing_val);
      thisdim_candidates = candidates(cindex);
      
      if size(thisdim_candidates,1)<min_training_pool,
        % single pass
        % drop dimensions until we have candidates>min_pool
        % the current dim is sacred (we need a value for it).
        % therefore, we drop any dims on the num_candidates
        % list until we reach the desired number

        i = size(num_candidates,2); % index into the present
        % vals which we got up to

        decrease = [-inf ...
        (num_candidates(1,1:i-1)-num_candidates(1,2:i)) ./ ...
        num_candidates(1,1:i-1)];
  
        while size(thisdim_candidates,1)<min_training_pool,

          [m,dropdim] = max(decrease);

          % found the max decrease var, now drop it by mangling it
          % from the original input vector!
          
        %  decrease(
          testinp(row,nonmissing_dims(dropdim)) = missing_val;
        end; % while
      end; % if lower than min_training pool
   end; % for each missing dim in this row
end; % for each row

end;
