function [c,cx,cy,member] = ...
         kmeans_batch(x,number_of_clusters, lc,number_of_loops,y);
%
% function [c,cx,cy,member] = ...
%          kmeans_batch(x,number_of_clusters,lc,iterations,y);
%


  % initialize clusters to the k random vectors in the input set
  % ------------------------------------------------------------

  i = ceil(rand(number_of_clusters,1).*(size(x,1)-1));
  c = x(i,:);

  s=mean(distance(x, ones(size(x,1),1) * mean(x) ));
  % standard deviation

  % general initialization
  % ----------------------

  member = ones(size(x,1),1) .* -1;
  closest_inputd = ones(number_of_clusters,1).*inf;
  closest_inputi = ones(number_of_clusters,1).*-1;
  cy = [];
  cx = []; % these are only returned if y is supplied

  for count=1:number_of_loops,

    disp(['iteration ' num2str(count)]);

    % calc correlation matrix between centroids c and inputs x
    % each centroid vector row multiplies with each input
  
    corr = c * x';

    % this correlation isn't normalized, so create a normalization 
    % matrix
    % normmat = c1 c1 c1 c1 ... .* x1 x2 (pointwize multiply)
    %           c2 c2 c2 c2 ...    x1 x2
    %
    % this is equivalent to normc = [c1*c1';c2*c2';...]' * [x1*x1';]
    % equivalent to diag(c*c')' * diag(x*x');
    % 
    
    % create norm matrices
    for i=1:size(c,1),
      cnorm(i,ones(1,size(c,2))) = sum(c(i,:) .* c(i,:)).^0.5;
    end;
    for i=1:size(x,1),
      xnorm(i,ones(1,size(x,2))) = sum(x(i,:) .* x(i,:)).^0.5;
    end;

    % outer product of these two to give the normalization matrix
    normmat = cnorm*xnorm';

    % now pointwize normalize by this matrix to give correlation
    corr = corr ./normmat;

    % hopefully all numbers are -1..1 !!!!
    
    % corr = [c1x1 c1x2 c1x3 c1x4 c1.. = coefficients for c1
    % telling us how close c1 should be pulled by x1,x2, and so on
    % therefore, change this to a coefficient matrix alpha
    % alpha = 
    %
    % diffmat = direction vector between input and centroid, ie. the
    % direction centroid must move to move to the input. (or the direction
    % it is being pulled by the input)
    % diffmatc1 = x - c(ones(size(x,1),1),:)
    % c1 = c1 + alpha=corr(1,:) * diffmatc1
    %
    % remember that alphas are distances! so they are the inverse of a
    % correlation. Therefore, we should use 2-alpha to get a positive
    % coefficient

    for i=1:size(c,1),
      diffmat = x - ones(size(x,1),1) * c(i,:);
      %diffmat = triangle(diffmat,1) .* sign(diffmat);
      diffmat = threshold(diffmat,s);
      %if diffmat>
      totalpha = sum(1-corr(i,:));
      c(i,:) = c(i,:) + lc .* (1-corr(i,:))./totalpha * diffmat;
    end;

  end;


