function [c,cx,cy,mini,no_members,movement,disttravelled] = ...
         kmeans_batch2(x,number_of_clusters, lc,number_of_loops,y,c,stopifadjustmentlessthan);
%
% function [c,cx,cy,member,movement,disttravelled] = ...
%          kmeans_batch2(x,number_of_clusters,lc,iterations,y,stopifadjustmentlessthan);
%
% 
% EXAMPLE
% [c,cx,cy,member,movement,disttravelled] = kmeans_batch2(x,5,0.1,10,y,
% figure(1);clf;plot(movement);
% % if you include all outputs, the routine will plot a graph, etc


  % initialize clusters to the k random vectors in the input set
  % ------------------------------------------------------------

  if nargin>=6,
    % c were supplied, so don't overwrite
  else
    % select random vectors from x to be cs
    % i = ceil(rand(number_of_clusters,1).*(size(x,1)-1));
    % c = x(i,:);
    [c,cy] = memlookup_train(x,y,number_of_clusters);
  end;

  if nargin>=7,
  else
    stopifadjustmentlessthan = -inf;
  end;


  mini   = [];
  cy       = [];
  cx       = []; % these are only returned if y is supplied
  movement = zeros(number_of_loops,1);
  disttravelled = zeros(number_of_clusters,1);
  no_members = [];


  % kmeans algorithm
  % ----------------

  for count=1:number_of_loops,
    %disp(num2str(count));
   
    % find vector of closest centroids to each x
    % mini = find_closest2(x,c);
    [a,mini] = min(dist2(x,c)'); mini=mini';

    % adjust closest centroids
    adjustment = (x-c(mini,:));
    cold = c;
    c(mini,:) = c(mini,:) + lc .* adjustment;
    disttravelled = disttravelled + distance(c,cold);
    movement(count,1) = mean(mean(abs(adjustment)));
    fprintf('%d(%f) ',count,movement(count));
    if movement(count)<stopifadjustmentlessthan, count=number_of_loops+1;end;
  end; % iterations


  % select cys for each discovered cluster
  % --------------------------------------

  if nargout>1,
    % find the closest input to each centroid, and use that input's
    % y value for the cy for the centroid
    mininput = find_closest2(c,x);
    cx = x(mininput,:);
    cy = y(mininput,:);
  end;

  if nargout>=6,
    fprintf('Cluster Statistics:\n\n');
    fprintf('Cluster\tNumber of members\tpercent membership\tDistance travelled\n');
    for i=1:number_of_clusters,
      no_members(i,1) = size(find(mini==i),1);
      fprintf('%d\t%d\t%6.3f\t%f\n',i,no_members(i,1),100.*no_members(i,1)./size(x,1),disttravelled(i,1));
    end;
    fprintf('look for clusters which had low movement, and have few members, these were probably ');
    fprintf('stranded\n');
    figure(1);plot(movement);xlabel('iteration');ylabel('max avg dimensional adjustment');
  end;
return
