function [c,cx,cy,mini, ...
          std_prop, coeffvar_prop, popmean_dist, ...
          number_belonging_to_cluster] = ...
          kmeans_batchc2(x,number_of_clusters, lc, ...
          number_of_loops,y, ...
         cluster_method, c);
%
% function [c,cx,cy,member, ...
%          std_prop, coeffvar_prop, popmean_dist, ...
%          number_belonging_to_cluster] = ...
%          kmeans_batch2(x,number_of_clusters,lc,iterations,y,
%          cluster_method,c);
% 
% cluster_method==0, kmeans
% cluster_method==1, conscience use conscience penalty to shrink distances 
% cluster_method==2, mcqueen, use learning rate braking based on assignment size
% cluster_method==3, powel, remove cluster if has less than 1 element and 
% assign at same pos as centroid with max proportion of points. (ie. it
% is assigned to same position and splits that centroid, competing with
% it)
%
% EXAMPLES
%
% cluster_method=3; powel
% for it=1:20,
% x=rand(100,2);y=rand(100,1);c=rand(4,2);x(:,1)=(x(:,1)+2).^4;
% [c,cx,cy,member] = kmeans_batchc2(x,4,0.2,10,y,cluster_method,c);
% a=find(member==1);b=find(member==2);
% cc=find(member==3);d=find(member==4);
% figure(1);clf;plot(x(a,1),x(a,2),'ro');hold on;
% if ~isempty(a),plot(c(1,1),c(1,2),'rx');end;
% if ~isempty(b),plot(x(b,1),x(b,2),'bo');plot(c(2,1),c(2,2),'bx');end;
% if ~isempty(cc),plot(x(cc,1),x(cc,2),'go');plot(c(3,1),c(3,2),'gx');end;
% if ~isempty(d),plot(x(d,1),x(d,2),'mo');plot(c(4,1),c(4,2),'mx');end;
% pause;end;
%
% %test each method for stdev of prop, mean dist from centroid
% 
% for i=1:200,
%   x=rand(20,2);y=rand(20,1);c=rand(4,2);x(:,1)=(x(:,1)+2).^4;
%   for method=0:3, 
%    [c,cx,cy,mini,st(method+1,i),co(method+1,i),dist(method+1,i)] = ...
%    kmeans_batchc2(x,4,0.2,20,y,method,c);
%   end;
% end;
% i=find(isnan(dist));dist(i)=zeros(size(i));
%   [ mean(st')' mean(co')' mean(dist')'],
%
% SEE ALSO 
% kmeans2

MIN_CLUST_SIZE = 100;
  KMEANS     = 0;
  CONS       = 1;
  MCQUEEN    = 2;
  POWEL      = 3;

  if cluster_method==CONS,    disp('conscience'); end;
  if cluster_method==MCQUEEN, disp('mcqueen');    end;
  if cluster_method==POWEL,   disp('powel');      end;

  % initialize clusters to the k random vectors in the input set
  % ------------------------------------------------------------

  if nargin==8,
     % c were supplied, so don't overwrite
  else
    % select random vectors from x to be cs
    i = ceil(rand(number_of_clusters,1).*(size(x,1)-1));
    c = x(i,:);
  end;

  member = [];
  cy     = [];
  cx     = []; % these are only returned if y is supplied


  % kmeans algorithm
  % ----------------

  conscience = zeros(number_of_clusters,1);
  % start off with no penalties on any cluster
  % conscience = conscience + prop each iteration until
  % props equal out and each prop==mean
  
  lc = lc * ones(number_of_clusters,1);
  % init learning rate. mcqueen will change these for each cluster
  
  p    = zeros(number_of_clusters,1);
  % cumulative number of datapoints that won at this centroid
  
  prop = zeros(number_of_clusters,1);
  % proportion of datapoints owned by the centroid, current iteration
  % for mcqueen, its the raw number of datapoints owned by the centroid


  % kmeans main loop
  % ----------------

  for count=1:number_of_loops,
    %disp(num2str(count));
   
    % find vector of closest centroids to each x
    [mini,dist] = find_closest2(x,c);

    % modify distance (if using CONSCIENCE)
    % -------------------------------------

    if cluster_method==CONS,
      dist = dist + (conscience * ones(1,size(x,1))) .* dist;
      % modify dist by conscience
      [m,mini] = min(dist);mini=mini';
      % recalc the new min closest centroids
    end;
   
    % calc the proportions of ownership for each centroid
    % ---------------------------------------------------

    for cid=1:number_of_clusters,
      prop(cid,1) = size(find(mini==cid),1);
    end;

   % update conscience (distance multiplier)
   % ---------------------------------------

   if cluster_method==CONS,
     % normalize prop, and subtract mean to get pos and neg
     prop = prop ./ size(x,1); % total number which could be won
     prop = prop - mean(prop); % prop is around the mean
     
     conscience = conscience + prop.*number_of_clusters; % continues to increase this
      % multiplier until the dist calc leads to 

      % consc prop to amount above mean prop is. 
      % consc continues to grow until dist impaired and begins to 
      % loose, resulting in prop dropping to 0 (mean) and conscience
      % stabilizing

      if 0,
      if rand(1,1)<0.2,
         conscience(ceil(rand(1,1)*(number_of_clusters-1))+1,1) = 0;
        % if rand<0.1, randomly eliminate conscience in one or more
        % clusters
        % prob, clusts way out still won't make it in, due to
        % they still can't beat the other clusts. However, it does
        % allow clusts to invade the space of a clust 
      end;
      end;
   end;

   
   % MCQUEEN
   % -------

   if cluster_method==MCQUEEN,
      % change the learning constant based on the number of
      % inputs assigned to that centroid
      p=p+prop;
      %p=prop;
      lc = 1./(p(:,1)+1);
   end;
   

   % POWEL
   % -----

   if cluster_method==POWEL,
     % should test if number of any clust less than a min, then run this
     % on that clust
      recalc=0;
      % if a centroid has no members, reinitialize it to a successful centroid
      %m=remove_duplicates(mini);
      for present=1:number_of_clusters,
         if prop(present,1)>=MIN_CLUST_SIZE, 
            % its on the in-use list
         else
            if rand(1,1)<0.5, % this only occurs 1 in 5 times
            % this cluster is no good! 
            % assign it to a present cluster.
            %c(present,:) = c(m( ceil(rand(1,1) * (size(m,1)-1) )+1),:);
            [mm,mi] = max(prop);
            %mm,mi,
            c(present,:) = c(mi,:)+(rand(1,size(c,2))-0.5).*eps;
            recalc=1;
            end;
         end;
      end;
      % this is similar to hillclimbing, to find dense regions
      % will always work as long as clusters < points

      if recalc,
        [mini,dist] = find_closest2(x,c);
      end;
   end;

   
   % adjust closest centroids
   % ------------------------

   c(mini,:) = c(mini,:) + (lc(mini,1) * ones(1,size(x,2))) .* (x-c(mini,:));

  end; % iterations


  % select cys for each discovered cluster
  % --------------------------------------

  if nargout>1,
    % find the closest input to each centroid, and use that input's
    % y value for the cy for the centroid
    mininput = find_closest2(c,x);
    cx = x(mininput,:);
    cy = y(mininput,:);
  end;

  [mini,dist] = find_closest2(x,c);

  % report the variance for each cluster, number of centroids in each
  % cluster
  % standard deviation of proportions / mean , ie. coefficient of
  % variation, meaning how the number of clusters varies from cluster
  % to cluster. The lower the better (more uniform)

  for cid=1:number_of_clusters,
    members_of_cid = find(mini==cid);
    number_belonging_to_cluster(cid,1) = size(members_of_cid,1);
    prop(cid,1) = size(members_of_cid,1);
    pointdist = distance(  x(members_of_cid,:) , ...
                      ones(size(members_of_cid,1),1) * c(cid,:) );
    popmean_dist(cid,1) = mean(pointdist);
    % popvar(cid,1) = x(members_of_cid,:)-c(;
  end;

  std_prop      = std(prop);
  coeffvar_prop = std_prop / mean(prop);
  popmean_dist  = mean(popmean_dist(:,1));

  disp('evenness of cluster numbers:');
  disp(['std(prop) ' num2str(std_prop) ' coeff of variation ' num2str(coeffvar_prop)]);
  disp('(ii) quality, or distance from cluster center to each point (should be minimal)');
  disp(['popmean_dist ' num2str( popmean_dist ) ]);
return
