function [c,cx,cy,mini,no_members,movement,disttravelled] = ...
         kmeans_mean(x,number_of_clusters,number_of_loops,y,c,stopifadjustmentlessthan,lc);
%
% function [c,cx,cy,member,no_members, movement,disttravelled] = ...
%          kmeans_mean(x,number_of_clusters,iterations,y,c,stopifadjustmentlessthan,lc);
%
%
% Performs the kmeans algorithm where centroids are calculated as the mean of the
% datapoints (xs) which are members of them. This algorithm could be better than
% kmeans_batch2 because the centroids move quicker to actual cluster centers. 
% kmeans_batch2 may also have trouble with the batch update (multiple movements
% cancel out, resulting in no movement). I should check kmeans against it to see if
% kmeans works any better.
%
% This algorithm needs to be improved by performing better centroid initialization
%

  % initialize clusters to the k random vectors in the input set
  % ------------------------------------------------------------

  if nargin>=5,
    % c were supplied, so don't overwrite
  else
    % select random vectors from x to be cs
    % i = ceil(rand(number_of_clusters,1).*(size(x,1)-1));
    % c = x(i,:);
    [c,cy] = memlookup_train(x,y,number_of_clusters);
  end;

  if nargin>=6,
  else
    stopifadjustmentlessthan = -inf;
  end;

  if nargin>=7,
  else
    lc = nan;
  end;

  mini          = [];
  cy            = [];
  cx            = []; % these are only returned if y is supplied
  movement      = zeros(number_of_loops,1);
  disttravelled = zeros(number_of_clusters,1);
  no_members    = [];


  % kmeans algorithm
  % ----------------

  for count=1:number_of_loops,
    %disp(num2str(count));
   
    % find vector of closest centroids to each x
    % mini = find_closest2(x,c);
    [a,mini] = min(dist2(x,c)'); mini=mini';
    
    cold = c;
    % adjust closest centroids
    id_to_replace =[];
    for cent=1:size(c,1),
      member = find(mini==cent);
      c(cent,:) = mean(x(member,:));
      if ~isnan(lc),
        c(cent,:) = cold(cent,:) + lc .* (c(cent,:) - cold(cent,:));
      end;
      no_members(cent,1) = size(member,1);
      if no_members(cent,1)==0, id_to_replace = [id_to_replace;cent];end;
    end;
      
    if ~isempty(id_to_replace),
       % find the centroids which account for the largest number of points
       [a,ii] = sort(-1.*no_members);
       c(id_to_replace,:) = c(ii(1:size(id_to_replace,1)),:)+0.000001.*(rand(size(id_to_replace,1),size(c,2))-0.5);
       fprintf('reinserted %d centroids\n',size(id_to_replace,1));
    end;
    
    disttravelled = disttravelled + distance(c,cold);
    movement(count,1) = mean(mean(abs(c-cold)));
    fprintf('%d(%f) ',count,movement(count));
    if movement(count)<stopifadjustmentlessthan, count=number_of_loops+1;end;
  end; % iterations


  % select cys for each discovered cluster
  % --------------------------------------

    % find the closest input to each centroid, and use that input's
    % y value for the cy for the centroid
    mininput = find_closest2(c,x);
    cx = x(mininput,:);
    cy = y(mininput,:);

  if nargout>=6,
    fprintf('Cluster Statistics:\n\n');
    fprintf('Cluster\tNumber of members\tpercent membership\tDistance travelled\n');
    for i=1:number_of_clusters,
      no_members(i,1) = size(find(mini==i),1);
      fprintf('%d\t%d\t%6.3f\t%f\n',i,no_members(i,1),100.*no_members(i,1)./size(x,1),disttravelled(i,1));
    end;
    fprintf('look for clusters which had low movement, and have few members, these were probably ');
    fprintf('stranded\n');
    figure(1);clf;plot(movement);xlabel('iteration');ylabel('max avg dimensional adjustment');
  end;
return
