function [out,cummulative] = lift(forecast,target,groups)
% [out,cummulative] = lift(forecast,target,groups)
%
% This is only designed to work with targets which are 1,0
% and real valued forecasts.
% 
% Appropriate for retention models. Not appropriate for rev forecasts
% unless predicting 0 rev or some rev is the nature of the problem.
% But then this is really predicting a visit
% 
% forecast is a column vector of signed confidence forecasts
% target is the binary target classification (0/1)
% groups is the number of bins in the lift analysis

target = target > 0;
% forces them to positive, ignores real value
forecast = forecast * -1;

% start with a random sort
[s i] = sort(rand(length(target),1));
forecast = forecast(i);
target = target(i);
% shuffles forecast and target in random order

% sort on forecast confindences
[s i] = sort(forecast);
forecast = forecast(i);
target = target(i);
% forecast and target are now in forecast order, from lowest to highest

% measure the incidence of positive targets
rows = length(target);
incidence = sum(target)/length(target);
% incidence is prob of seeing a one in the target

% for each forecast score, from max to min (neg so does high early),
% count the number of positive 1s (ie. target existed as a 1)
% This is divided by the baseline probability of having a target 1

% So lift is only designed to work with a target which is 0 or 1
% (or -1,1)
% If target is a real value, this is transformed into a 1.
% It still normalizes by baseline probability, so this is still picking up
% number of times over random rate of having target 1
% However, expect accuracy isn't as good as normalizing by decile membership

% calculate lift for each group
out = [];
for i = 0 : rows/groups : rows-(rows/groups)
   startindex = round(i + 1);
   endindex = round(startindex + (rows/groups) - 1);
   rowsInGroup = endindex - startindex + 1;
   
   safeend = min([size(target,1) endindex]);
   response = sum(target(startindex:safeend))/rowsInGroup;
   lift = response/incidence;
   
   out = [out; [lift]];
end
cummulative = out./([1:groups]');
