function [valseries,liftval,rankseries,liftrank, ...
          cumvalseries,cumliftval,cumrankseries,cumliftrank,nuh,nuh2,auc1,auc2] = ...
  lift2(actual,pred,no_groups,fig);
% [valseries,liftval,rankseries,liftrank, ...
%  cumvalseries,cumliftval,cumrankseries,cumliftrank,cumforecast,cumbest] = ...
%   lift2(actual,pred,no_groups,fig);
%
% if fig is not empty, then plot it in these figures
%
% EXAMPLE
% 
% actual = round(rand(1000,1).*10);pred = actual+randn(size(actual,1),1).*10;rank_correlation(actual,pred,4),
% lift2(actual,pred,10,1);

if nargin<3, 
   fprintf('error\n');
end;
if nargin==3, 
   fig=[];
end;


[d1,d2,d3,actual1] = decile1(actual,1,no_groups,0);


% First create 
[a,i]=sort(-1.*pred);nuh=cumsum(actual(i));
[a,i]=sort(-1.*actual);nuh2=cumsum(actual(i));
if ~isempty(fig),
   figure(fig);clf;
   plot(1:size(nuh,1),nuh,'-');hold on;
   plot([1 size(actual,1)],[0 sum(actual)],'--');
   plot(1:size(nuh,1),nuh2,':');
   legend('forecast','random','max possible',0);
end;

%keyboard;
auc1 = mean(nuh./max(nuh) - [max(actual)./length(nuh):max(actual)./length(nuh):max(actual)]'./max(actual)) + 0.5;
auc2 = mean(nuh2./max(nuh2) - [max(actual)./length(nuh2):max(actual)./length(nuh2):max(actual)]'./max(actual)) + 0.5;

fprintf('auc\tforecast\tmaxpossible\tpct\n');
fprintf('auc\t%f\t%f\t%f\n',auc1,auc2,auc1./auc2);

% i is the order in which we will traverse the forecast,actual pairs

%[d1,d2,d3,actual1] = decile1(actual,1,no_groups,0);
actual1 = no_groups-actual1+1; % puts it into reverse order (top decile 10)
[a,i] = sort(-1.*pred);
actual = actual(i,1);
actual1 = actual1(i,1);
% puts actual into order of forecast

% Now, we will traverse forecasts in order from highest to lowest
% For each group, we will count the number of actuals which are in this same group
% This was the number of correct rank predictions. 
% Then we will normalize this number correct by baseline probability of being correct.

group_size = size(actual,1)./no_groups;
mean_actual = mean(actual);

for i=1:no_groups,
   r = [round((i-1).*group_size)+1 : round(i.*group_size)];
   valseries(i,1) = mean(actual(r,1)); 
   % interp: if my forecast is top decile, my custs, actual tends to be this
   rankseries(i,1) = mean(actual1(r,1));
   % interp: if my forecast is top decile, my custs tend to be in this decile (average)
   liftval(i,1) = valseries(i,1)./mean_actual;
   % interp: number of times over random this actual value is 
   liftrank(i,1) = rankseries(i,1)./(no_groups./2);
   % interp: number of times over random, the correspondence of forecast to actual decile
end;

cumvalseries = cumsum(valseries)./[1:no_groups]';
cumrankseries = cumsum(rankseries)./[1:no_groups]';
cumliftval    = cumsum(liftval)./[1:no_groups]';
cumliftrank   = cumsum(liftrank)./[1:no_groups]';

if ~isempty(fig),
   
   percent_of_pop = 100.*([1:no_groups]./no_groups);
   
   figure(fig+1);clf;
   bar(percent_of_pop,valseries);hold on;
   plot(percent_of_pop,cumvalseries,'o-');
   v=axis;
   plot([v(1) v(2)],[mean(valseries) mean(valseries)],':');
   legend('cumvalseries','random','valseries',0);
   title('Isolation of value in actuals by ordering predictions');
   xlabel('percent of population');
   ylabel('value isolated by using forecasts');
   
   figure(fig+2);clf;
   bar(percent_of_pop,rankseries);hold on;
   plot(percent_of_pop,cumrankseries,'o-');
   v=axis;
   plot([v(1) v(2)],[mean(rankseries) mean(rankseries)],':');
   legend('cumrankseries','random','rankseries',0);
   title('Rank prediction: Isolation of value in actuals by ordering rank predictions');
   xlabel('percent of population');
   ylabel('value isolated by using forecasts');
   
   figure(fig+3);clf;
   plot(percent_of_pop,cumliftval,':');hold on;
   plot(percent_of_pop,cumliftrank,'-');
   legend('cumliftval','cumliftrank',0);
   title('Cummulative isolation of value in actuals (value found over random)');
   xlabel('percent of population');
   ylabel('lift (value over random value)');
   
   figure(fig+4);clf;
   bar(percent_of_pop,[liftval liftrank],'grouped');hold on;
   v=axis;
   plot([v(1) v(2)],[1 1],':');
   legend('random','liftval','liftrank',0);
   title('Lift isolation of value in actuals, (value found over random)');
   xlabel('percent of population');
   ylabel('lift (value over random value)');
   
   figure(fig+5);clf;
   bar(percent_of_pop,liftval,1);hold on;
   v=axis;
   plot([v(1) v(2)],[1 1],':');
   legend('liftval',0);
   title('Lift isolation of value in actuals, (value found over random)');
   xlabel('percent of population');
   ylabel('lift (value over random value)');
   
      figure(fig+6);clf;
   plot(percent_of_pop,cumliftval,':');hold on;
   legend('cumliftval',0);
   title('Cummulative isolation of value in actuals (value found over random)');
   xlabel('percent of population');
   ylabel('lift (value over random value)');
   

end;
