function [w,yest] = linear_reg_train(x,y);
%
% function w = linear_reg_train(x,y);
%
% w is in order from dim1,dim2,dim3,...,+c
%

% question, can we extend this to the multidim dependent case.
% if y has more than one dimension, then 
%
% I assume that if we had a multidimensional dependent case, then
% each y dim can be dependent on any combination of the inputs. 
% therefore, for each ydim we calculate a regression equation as is
% done here.
% ie. try to approximate each dim of y separately. This makes sense,
% since the dimensions in y are being treated separately.
% This means that the w vector becomes several deep, one deep for 
% each dim of y which is being approximated
%
% also, we are calculating a correlation in each dimension (or
% the step in y we have to take for each step in x, or (y-x)/x_dist
% and 
% summing. Why don't we have to normalize by the number of dimensions,
% eg. if x is 5 dimensions, divide all coefficients by 5?
%

meanx = mean(x);
meany = mean(y);

x = x - (ones(size(x,1),1) * meanx);
y = y - (ones(size(y,1),1) * meany);

% normalizing factor (doesn't normalize against y, which is how
% high the line goes)
x_squared = sum(x.^2);

for i=1:size(x,2),
  w(1,i) = sum(x(:,i) .* y) ./ x_squared(:,i);
  % (x(:,i)' * y) / (x(:,i)' * x(:,i))
  % almost exactly the same as the correlation equation, except no
  % normalization by y.    
end;

w(1,size(x,2)+1) = meany - sum((w .* meanx)');
% all variables have subtracted their mean. Therefore, at the mean,
% 

yest = linear_reg_eval(x,w);

