function im = mat2im(y,x,z,max_down,max_across);
%
% im = mat2im(y,x,z,max_down,max_across);
%
% takes data [y,x,z] and translates it into a 2D image matrix.
% Size of the matrix is max_down+1,max_across+1
% 
% EXAMPLE
% 
% im = mat2im(cout2(:,1),cout2(:,2),cout2(:,3),20,20);
% [x,y] = meshgrid(1:size(im,1),1:size(im,2));
% [xi,yi] = meshgrid(1:0.2:size(im,1),1:0.2:size(im,2));
% zi=interp2(x,y,im,xi,yi,'cubic');
% surf(zi);shading faceted;
% 

CONST = -1000; % this signifies a no-entry
% usually this is zero, but here we want to see the real zeros
% 


x = x-min(x);
y = y-min(y);
maxy = max(y);
maxx = max(x);
res_down   = max_down   / maxy;
res_across = max_across / maxx;

im     = ones(max_down+1,max_across+1).* CONST;
normim = zeros(size(im));

for i=1:size(y,1),
  down   = floor(y(i,1).*res_down)+1;
  across = floor(x(i,1).*res_across)+1;
  if im(down,across) == CONST, 
    im(down,across) = 0;
  end;
  im(down,across)     = im(down,across) + z(i,1);
  normim(down,across) = normim(down,across) + 1;
end;

% divide by normalization matrix
i=find(normim>0);
im(i) = im(i) ./ normim(i);

