function m = mean_missing(data,missing_val);
% m = mean_missing(data,missing_val);
%
% Returns the mean applied to non-missing values only
% The data must consist of column vectors. (row vectors will be treated as a 1 row column vector)
% If a dimension has all missing values, then the result of mean_missing 
% will be to return a nan on that dimension. A warning message will also be displayed
% 
% SEE ALSO
%
% sum_missing, min_missing, max_missing
% 

[m,count] = sum_missing(data,missing_val);

if size(m,2)==1 & count==0,
  m = nan;
  return;
end;

i = find(count~=0); 
j = remove_ids([1:size(count,2)],i);
m(i) = m(i) ./ count(i);

if ~isempty(j),
  m(j) = ones(1,length(j)).*nan;
  fprintf('WARNING: mean_missing, some dimensions were all missing. They were returned as nans. You may want to code them as zeros or something\n');
  return;
end;

return;


% OLD CODE

m = zeros(1,size(data,2));

for i=1:size(data,2),
  if isnan(missing_val),
    jj=find(~isnan(data(:,i)));
  else
    jj=find(data(:,i)~=missing_val);
  end;
  if ~isempty(jj),
    m(1,i) = mean(data(jj,i));
  else
    m(1,i) = nan;
  end;
end;
