function [whid,wout,yest,testerr] = ...
         mlp_train(x,y,no_hidden_units,holdoutpercent);
%
% function [whid,wout,yest,testerr] = ...
%           mlp_train(x,y,no_hidden_units,holdoutpercent);
%

  MIN_ERROR_CHANGE = -0.01;     
  % if error change is less than this figure, optimization will stop.
  % Change this if needed (eg. make it smaller if error is not small 
  % enough)

  whid = (rand(size(x,2),no_hidden_units)-0.5)./2; % range -0.5..0.5
  wout = (rand(no_hidden_units,size(y,2))-0.5)./2;

  % initialize training and holdout set 
  % -----------------------------------

  maxtrain = ceil(holdoutpercent.*(size(x,1)-1));
  xtrain = x(1:maxtrain,:);
  if maxtrain>0,
    xtrain = x(1:maxtrain,:);
    ytrain = y(1:maxtrain,:);
    xtest  = x(maxtrain+1:size(x,1),:); 
    ytest  = y(maxtrain+1:size(x,1),:);
  else 
   xtrain = x; ytrain = y;
   xtest  = x;  ytest = y;
   disp('testerr results are for training set');
  end;
  disp(['training elements ' num2str(maxtrain) ' test elements ' ...
         num2str(size(xtest,1))]);
  disp('press any key to begin training');
  pause

  % Initialize
  % ----------

  % fmins settings - 1 update and then exit
  options(1)=0; % display
  options(2)=0.0001; % x tolerance before exit. Reduce this to keep in
  options(3)=0.0001; % err tolerance before exit. (zero - termination by no. of steps)
  options(10)=100; % no. of function steps. 
  options(14)=100; % no. of

  delta_testerr = -inf;
  testerr(1,:) = inf;
  it = 1;

  ww = [whid ; wout']; 
  % packages the weights so that they can be optimized by fmins (fmins
  % expects a single matrix or parameters)

  while delta_testerr< MIN_ERROR_CHANGE,

    % Optimize weights
    % ----------------

    ww = fmins('mlp_error', ww, options,[], ...
                            xtrain,ytrain,no_hidden_units);

 %   ww = gradient_optimize('mlp_error',ww, [],[], ...
  %    0.000001, 10, inf, inf, 0.001, 1, 0, ...
   %   xtrain,ytrain,no_hidden_units);
   % Need to make the mlp_error function 
   % return us derivatives as the second argument 
      
    whid = ww(1:size(x,2),:);
    wout = ww(size(x,2)+1:size(x,2)+size(y,2),:)';
    ww = [whid ; wout'];

    % Evaluate on test set
    % --------------------

    it = it+1;
    testerr(it,1) = mlp_error(ww,xtest,ytest,no_hidden_units);
    delta_testerr = testerr(it,1) - testerr(it-1,1);

  end;

  % Eval once to get yest from the training set
  % -------------------------------------------
   
  yest = mlp_eval(x,whid,wout);

return
