function [cliquemean,cliquedist,number_belonging_to_clique, ...
          clique,clusterlist] = ...
          multistart_kmeans(x, ...
          number_of_clusters, ...
          lc, ...
          number_of_iterations, ... 
          y, ...
          cluster_method, ...
          runs, ...
          multistart_method);

%[c,cdist,cnumber_els,clique,centroidlist] = ...
%   multistart_kmeans(x,number_of_clusters,lc, ...
%   number_of_iterations, y, ...
%   cluster_method, ...
%   runs, multistart_method);
%
% multistart_method == 1, use clique formation
%                   == 2, cluster the clusters
%
%EXAMPLE
%
%%show the cliques produced from the multistart runs
%%this hopefully shows that the cluster groups produced are tightly
%%packed, meaning that the clusters are robust
%
%multistart_method=2;cluster_method=0;
%x=rand(20,2);y=rand(20,1);c=rand(4,2);x(:,1) = (x(:,1)+2).^4
%[c,cdist,cnumber_els,clique,clusterlist] = ...
% multistart_kmeans(x,4,0.2,10,y,cluster_method,20,multistart_method);
%figure(1);clf; hold on;
%if multistart_method==1,
%a=clusterlist(clique(1,:)',:);plot(a(:,1),a(:,2),'g+');hold on;
%a=clusterlist(clique(2,:)',:);plot(a(:,1),a(:,2),'mo');
%a=clusterlist(clique(3,:)',:);plot(a(:,1),a(:,2),'bs');
%a=clusterlist(clique(4,:)',:);plot(a(:,1),a(:,2),'r>');
%else
%m = find(clique(:,1)==1);
%a=clusterlist(m,:);plot(a(:,1),a(:,2),'g+');hold on;
%m = find(clique(:,1)==2);
%a=clusterlist(m,:);plot(a(:,1),a(:,2),'mo');
%m = find(clique(:,1)==3);
%a=clusterlist(m,:);plot(a(:,1),a(:,2),'bs');
%m = find(clique(:,1)==4);
%a=clusterlist(m,:);plot(a(:,1),a(:,2),'r>');
%end;
%plot(c(1,1),c(1,2),'g+','linewidth',1,'markersize',20);
%plot(c(2,1),c(2,2),'mo','linewidth',1,'markersize',20);
%plot(c(3,1),c(3,2),'bs','linewidth',1,'markersize',20);
%plot(c(4,1),c(4,2),'r>','linewidth',1,'markersize',20);
%

if multistart_method==2, % cluster the clusters
   for run=1:runs,
      clusterlist((run-1)*number_of_clusters+1 : ...
                     run * number_of_clusters,:) = ...
        kmeans_batchc2(x,number_of_clusters,lc,...
                       number_of_iterations,y,cluster_method);
   end;
   
   % run the clustering on the clusters
   [cliquemean,d,d2,clique] = kmeans_batchc2(clusterlist,number_of_clusters,lc, ...
      number_of_iterations,clusterlist(:,1),cluster_method);
   
   % measure the stability of these clusters
   % for each cluster, distance
   
   for cliqueno=1:number_of_clusters,
      members_of_clique = find(clique==cliqueno);
      number_belonging_to_clique(cliqueno,1) = size(members_of_clique,1);
      cliquedist(cliqueno,1) = ...
         mean(distance( ones(number_belonging_to_clique(cliqueno,1),1) * cliquemean(cliqueno,:), ...
         clusterlist(members_of_clique,:) ));
   end;
                               
   return
end;
                 
% otherwise, multistart_method==1, form mean cliques
for run=1:runs,
   
   c = kmeans_batchc2(x,number_of_clusters,lc,...
      number_of_iterations,y,cluster_method);
   
   if run==1,
      clique = [1:number_of_clusters]'; % run 1 is vertical
      % these are ids which point into clusterlist
      clusterlist = c;
   else
      
     clusterlist = [clusterlist;c];
      
     % find cliquemean of cliqueno (clique nodes 1..run-1)
     for cliqueno=1:number_of_clusters,
        cliquemean = ...
           mean(clusterlist(clique(cliqueno,1:run-1)',:));
     
       % find closest c to the mean of the current clique      
       dist = distance(ones(number_of_clusters,1)* ...
                       cliquemean, c);
        [m,cliqueadd] = min(dist);
        clique(cliqueno,run) = cliqueadd + ...
           (run-1) * number_of_clusters;
        
        %cliqueno,
     %keyboard   
     end; % foreach cliqueno
  end; % if
end; % run

% now we have a list of cliques, 
% find the mean once more. This is the final cluster estimate
% find the mean once more. This is the final cluster estimate

number_belonging_to_clique = runs;
for cliqueno=1:number_of_clusters,
   clique_members = clusterlist(clique(cliqueno,1:runs)',:);
   cliquemean(cliqueno,:) = mean(clique_members);
   cliquedist(cliqueno,1) = mean(distance( ones(runs,1) * cliquemean(cliqueno,:), ...
                                     clique_members));
end;