function [c,yest] = polyfitf(x,y,degree)
% [c,yest]=polyfitf(x,y,degree)
% 
% estimate a polynomial mapping from x to y of degree, degree.
% degree is either a scalar (eg. 1,2), or a vector, eg.
% [1 3 1 2 1]. if degree is a vector, then each degree column is
% applied to the corresponding dim of the data matrix, ie. that
% degree is the degree of that dim.
% If degree is a scalar, then that degree is applied to each 
% of the input dimensions.
% 
% Example
% 
% x = rand(100,4); y=x(:,1).^2 + x(:,2).^3 - 5.*x(:,3);
% figure(1);clf;plot(x(:,1),y,'x');
% degree = [2 3 1 1]; 
% w=polyfitf(x,y,degree);
% yest=polyff(w,degree,x);
% figure(1);hold on;plot(x(:,1),yest,'o');
%
% % (estimates a mapping from 4d input space, and plots 
% % a 1d slice of x(:,1) versus y).
%

A=create_vanderf(x,degree);
c=A\y; % performs least means squared on matrix to give poly fit.
yest = A*c;
