function [w,c,yest] = ppoly_train(x, y, degree, number_of_clusters, ...
                                  lc, iterations, KMEANS, ...
                                  cluster_input);
% 
% [w,c,yest] = ppoly_train(x,y,degree,number_of_clusters, ...
%                              lc,iterations, KMEANS, ...
%                              cluster_input);
%
% y must be a one-column vector
% w returns a set of columns, one column for each cluster
% each column is a GPOLY set of coefficients
% KMEANS==1,  use kmeans
%       ==0,  use random inputs
%       ==-1, the single cluster_input column is a list of integer bins
%
% cluster_input can be identical to x, and is a data vector 
% related to x (eg. a subset) which ppoly will cluster / break up
% its models on. If cluster_input is a single column of integers,
% then we assume we're clustering on integer classifications.
%

dim = size(x,2);
w   = zeros(dim*degree+1,number_of_clusters);

disp('clustering the input space');

tmp = zeros(size(x,1),1); % tmp used for cy
if KMEANS==1,
  [c,cx,cy,member] = kmeans_batch2(cluster_input,number_of_clusters,lc,iterations,tmp);
  %[c,cx,cy,member] = kmeans_batchc2(cluster_input,number_of_clusters,lc,iterations,tmp,3);  
end;
if KMEANS==0,
  [c,cy,member] = memlookup_train(cluster_input,tmp,number_of_clusters);
end;
if KMEANS<0,
   if size(cluster_input,2)>1, 
      disp('special option. can only break up on one column');
      return;   
   end;
   c=[0:max(cluster_input)]';
   number_of_clusters = size(c,1);
   %c = [c zeros(size(c))]; % this is needed to prevent degenerate case
   % only works for the case where we are breaking up on a  
   % single dimension which has been pre-binned 0..n
   % each entry in the x(:,cluster_on) column is the id
   % of its cluster
   member = cluster_input+1;
   %cluster_input = [cluster_input zeros(size(cluster_input))];   
end;
   
   

% calc weights mappings for each input group

for cid=1:number_of_clusters,
% we have to build an x..y vector. Therefore, we have
% to group all of the inputs belonging to cluster cid
% in the same training vector

  disp(['training mapping for cluster ' num2str(cid)]);
  members_of_cid = find(member==cid);
  train_input = x(members_of_cid,:);
  train_output = y(members_of_cid,:);
  
  if size(train_input,1)<=size(x,2),
     w(:,cid) = zeros(size(w,1),1);
     disp(['warning: cluster ' num2str(cid) ' has rows<dims']);
  else
    % perform polyfitm, fit weights from input to output
    w(:,cid) = polyfitf(train_input,train_output,degree);
    % ll other 
 end;
 
  % next clustered input group please
end;

%keyboard

disp('evaluating ppoly model at training points');
% perform an eval to get yest for training data
yest = ppoly_eval(x,w,degree,cluster_input,c);
