
function [w,err,yest] = regress_train(x,y,caseweights,missingval,its);
% [w,err,yest] = regress_train(x,y,caseweights,missingval,its);
%
% missingval == Only include missingval if there are indeed missing values in the data,
% as code runs slower with them. missingval=nan means no missing values. 
% Missing value mode effectively converts weights to 0..1 (w/sum(w))
% 
% its == iterations (default 10)
% 
% EXAMPLE
%
% % Weighted Adaline fit
% nuh(:,1) = rand(100,1);nuh(:,2) = nuh(:,1).^2;nuh(:,3)=ones(size(nuh,1),1);
% ww=adaline_train([ones(size(nuh,1),1) nuh(:,1)],nuh(:,2),nuh(:,3));
% figure(2);clf;scatter(nuh(:,1),nuh(:,2),nuh(:,3).*20);hold on;plot(nuh(:,1),polyff(ww',1,nuh(:,1)),'x-');
%
% % regresspositive fit
% [w,err,yest] = regresspositive_train([ones(size(nuh,1),1) nuh(:,1)],nuh(:,2),nuh(:,3));
% % figure(3);clf;scatter(nuh(:,1),nuh(:,2),nuh(:,3).*20);hold on;plot(nuh(:,1),polyff(ww',1,nuh(:,1)),'x-');
% figure(3);clf;scatter(nuh(:,1),nuh(:,2),nuh(:,3).*20);hold on;plot(nuh(:,1),yest,'x-'); 
%
% % Multiple dimensions
% x = rand(100,5);
% y = x(:,1).^2+5.*x(:,2)+2.*x(:,3).^2;
% [w,err,yest] = regresspositive_train(x,y,ones(size(x,1),1));w,figure(3);clf;scatter(y,yest,'o');fprintf('corr %f\terr %f\n',correlation(y,yest),err);


if nargin<5, its = 10; end; 
if nargin<4, missingval=nan; end;
if nargin<3, caseweights = ones(size(x,1),1);end;

minerr = inf;

for it=1:its,

winit=rand(size(x,2),1);

%warning off;
options=optimset('fminsearch');options.Display='on';%'off'; % Stops it from complaining if it runs out of iterations
w = fminsearch(@(w) regress_err(w,x,y,caseweights,missingval),[winit],options);
%w = fminsearch(@(w) adaline_eval(w,x,y,caseweights),rand(1,size(x,2)),options);
%warning on;

[err,yest] = regress_err(w,x,y,caseweights,missingval);

if err<minerr | it==1, minerr=err; yestsave = yest; wsave = w; end;

% i = find(w<0);
% if ~isempty(i), w(i) = zeros(length(i),1); fprintf('%d dims < 0\n',length(i));end;
% i = find(w>1); 
% if ~isempty(i), w(i) = ones(length(i),1);fprintf('%d dims > 1\n',length(i));end;
%w = w./sum(w); % renormalize to all between 0 and 1

winit = w;


end;

w = wsave;
err = minerr;
yest = yestsave;

return;
