
function [err,yest] = regresspositive_err(w,x,y,caseweights,missingval);
% [err,yest] = regresspositive_err(w,x,y,caseweights,missingval);
%
% Missingval is a specific kind of processing that will convert the weights
% to 0..1 (w/sum(w)) and also supports missing values in each dimension
% 

if nargin<5, missingval=nan;end;

if isnan(missingval), 
  % No missingvalues
  yest = x * w;
else
  yest = zeros(size(x,1),1);
  west = zeros(size(x,1),1);
  for dim=1:size(x,2),
      i = find(x(:,dim)~=missingval);
      if ~isempty(i), 
        yest(i) = yest(i) + x(i,dim).*w(dim); 
        west(i) = west(i) + w(dim);
      end;
  end;
  %warning off;
  yest = yest./west;  
  %warning on; 
 
  % This shouldn't happen
  %i1 = find(west==0);
  %if ~isempty(i1),
  %  yest(i1)=ones(length(i1),1).*missingval;
  %end;
  
  
%   .1;1.0  
%   5 8 == .1*5 + 1.0*8 = .5+8 = 8.5 / 1.1 = 7.72
%   2 -99 == .1*2 + null = .2 / .1 = 2
end;


err     = sum((yest-y).^2.*caseweights);
meanerr = sum((mean(yest)-y).^2.*caseweights);

i = find(w>1 | w<0);
%if ~isempty(i), err = err.*length(i); end; 
if ~isempty(i), err = err + meanerr.*length(i); end; 
%if ~isempty(i), err = inf;  end; 
% Scale up error by number of violations

