function list = remove_duplicates(x);
% function list = remove_duplicates(x);
% given an integer vector, x, returns a new list with the duplicates
% in x removed. x is a vertical vector
%
% Does this by creating a large linear hash table, 
% sending all list values to this hash table, allowing collisions to 
% occur, then re-finding the values to build the list again

% hash key is storage_loc = list_val-minx+1, which is pretty simple

maxx = max(x);
minx = min(x);

nullval = minx-1; 
% this value doesn't appear in the list. Used for
% space filler. When we reconstruct the list we'll only find values
% larger than this filler value (so ignore the filler)
%keyboard
hash                = ones(maxx-minx+1,1) .* nullval;
hash(x-minx+1,1)    = x; 
% puts the value of x at x positions. If there are duplicates, these
% collapse to a single value (at the single location)

list = hash(find(hash>nullval),1);
% find the values of hash which are greater than nullval
% returns the actual values


