function scatter_numerical(x,y,roundfactor,scale,pweight,varargin);
% 
% scatter_numerical(x,y,roundfactor,scale,pweight,varargin);
%
% Plot a matlab scatter plot / bubble plot, where size and colour are used to 
% represent the number of cases occupying that region
% When plotting real-valued data, it is unlikely that any real point will exactly 
% equal any other, and so the size and colouring of the bubble plot is wasted. 
% The present routine discretizes the data enough to allow for this size information 
% to be displayed
% If roundfactor and scale aren't provided, defaults of 1000 and 1 are selected
%
% EXAMPLES
%
% x = rand(100,1);y=rand(100,1); 
% scatter_numerical(x,y);
% scatter_numerical(x,y,[],0.1);
% scatter_numerical(x,y,[],[]);
% 

if nargin<4 | isempty(scale),
  scale = 1;
end;
if nargin<3 | isempty(roundfactor), 
  roundfactor=1000;
end;
if nargin<5 | isempty(pweight),
  pweight = ones(size(x,1),1);
end;


[group,pgroup] = group_by_numerical(round(x.*roundfactor)./roundfactor,[round(y.*roundfactor)./roundfactor pweight],{'mean','sum'});
clf;scatter(group,pgroup(:,1),pgroup(:,2).*scale,pgroup(:,2),'s','filled');
colorbar;

