
function [meanopt,stdopt,meanmult,stdmult,probcurve,r] = show_two_normals(r,multopt_out,multiplier,logit,showprobwin);
%
% show_two_normals(x,dat1,dat2);
% 
% Takes two distributions, assumes that they're lognormal, and plots
% two normal distributions
% 

 if nargin<4, 
   logit=1;
 end;
 if nargin<5,
   showprobwin=0;
 end;
 if nargout<5,
   maxp=nan;
 end;
   
 figure;clf;hold on;
 
 rorig = r;
 if logit,
   r = log(r);
   multopt_out = log(multopt_out);
   multiplier  = log(multiplier);
 end;
 
 meanopt  = mean((multopt_out));
 meanmult = mean((multiplier ));
 stdopt   = std( (multopt_out));
 stdmult  = std( (multiplier ));

 normnew = gaussianF( (r-meanopt)./stdopt,1);
 normold = gaussianF( (r-meanmult)./stdmult,1);
 % normold = gaussianF(r-meanmult,stdmult);
 plot(r,normnew,'r-');
 plot(r,normold,'b-');

 % The following is related to trying to assess the probability of 
 % curve 1 being selected, and not curve 2 being selected. Note that 
 % if the two events are mutually exclusive, then this doesn't make 
 % sense - except as an approximation - we break the assumption of 
 % mutual exclusivity. Note that n
 
 title('Parametric win and loss distributions');
 %title('Comparison of two fitted normals (curves are parametric normals)');
 xlabel('Multiplier');
 ylabel('Pr');
 %legend('new','old');
 legend('win','loss',0);
 % find the regions where the mults drop below pr 0.001, these will be the left and right axes
 [a,oldmaxid] = max(normold);
 [a,newmaxid] = max(normnew);
 ratmaxold = r(oldmaxid);
 ratmaxnew = r(newmaxid); 
 leftedge  = (r(max(find(normnew<0.001 & normold<0.001 & r<ratmaxold & r<ratmaxnew))));
 rightedge = (r(min(find(normnew<0.001 & normold<0.001 & r>ratmaxold & r>ratmaxnew))));
 v=axis;
 
 rightedge = rightedge+eps;

 if (isempty(leftedge) | isempty(rightedge)),
   % don't do anything
 else
   axis([leftedge rightedge v(3) v(4)]);
 end;

 % Add tick labels
 tic = get(gca,'xtick');
 if logit,
   set(gca,'xticklabel',num2str(round(1000.*exp(tic'))./1000));
 end;


 % Following is if we need to show the showprobwin
 pwin  = normpdf((r-meanopt)./stdopt);
 ploss = normpdf((r-meanmult)./stdmult);
 probcurve = log(pwin./ploss); % pwin.*(1-ploss);
 % probcurve = probcurve ./ sum(probcurve);

 if showprobwin,

   % Show log odds
   figure;clf;
   plot(r,probcurve,'c-');
   xlabel('Multiplier');
   ylabel('Log odds of winning');
   title('Log odds of winning/losing, from fitted normal distributions'); 
   vv=axis;
   if (isempty(leftedge) | isempty(rightedge)), 
     % don't do anything
   else
     axis([leftedge rightedge vv(3) vv(4)]);
   end;
   set(gca,'ylimmode','auto');
   tic = get(gca,'xtick');
   if logit, 
     set(gca,'xticklabel',num2str(round(1000.*exp(tic'))./1000));
   end;
 
   % Show logistic fit to the log odds
   ii = find(~isnan(r) & ~isinf(r) & ~isnan(probcurve) & ~isinf(probcurve)); 
   w = polyfitf([r(ii) ones(length(ii),1)],probcurve(ii),1); % 2 if we want to try a quadratic fit
   figure;clf;
   plot(r(ii), exp(w(2).*r(ii)+w(1)) ./ (1+exp(w(2).*r(ii)+w(1))),'-');
   % for linear fit
   
   % plot(r(ii), exp(w(3).*r(ii).^2 + w(2).*r(ii)+w(1)) ./ (1+exp(w(3).*r(ii).^2 + w(2).*r(ii)+w(1))),'-');
   % for quadratic fit
   
   vv=axis;
   if (isempty(leftedge) | isempty(rightedge)), 
     % don't do anything
   else
     axis([leftedge rightedge vv(3) vv(4)]);
   end;
   set(gca,'ylimmode','auto');
   tic = get(gca,'xtick');
   if logit, 
     set(gca,'xticklabel',num2str(round(1000.*exp(tic'))./1000));
   end;
   xlabel('Multiplier');
   ylabel('Prob win');
   title('Win probability, given linear fit to log odds ratio (logistic regression of multiplier against win)');           

 end;

 if logit,
   r = exp(r);
   % so that we can return it back
 end;
 
return;

