function [minx,stepsize,zi,maxx]=showresults(x,y,fig);
% [minx,stepsize,zi,maxx] = showresults(x,y,fig);
% minx and stepsize can be used to 
% zi is the resulting matrix


% ------------------------------------------------------
% USER ADJUSTABLE PARAMETERS
%
points = 50; %100;             
% this is the "resolution" of the graph. Use more points
% to get more fine-grained graph, but this is slower

number_of_tickmarks = 7; 
% this is the number of tick marks on the axis. For instance, 7 means
% that there will be 7 labels. More tickmarks result in crowding

%
% End of user definable parameters
% ------------------------------------------------------------


if nargin<3,
   fig=[];
end;

minx = min(x);
maxx = max(x);
xrange = maxx-minx;

i = find(y~=0);
meanx = (mean(x(i,:))-minx)./maxx; % new

stepsize = xrange ./ points;

[xi,yi] = meshgrid(minx(:,1)+stepsize(:,1):stepsize(:,1):maxx(:,1),minx(:,2)+stepsize(:,2):stepsize(:,2):maxx(:,2));
if 0,
  zi = griddata(x(:,1),x(:,2),y(:,1),xi,yi,'linear'); % 'linear','cubic','nearest','v4'
else
  norml=0;width=0;lc=0;it=0;
  method=3; centroids=300; %650; %30 centroids=80;
  disp('training');
  [c,w,yest] = spline_train(x,y,centroids,it,lc,norml,method,width);
  disp('eval');
  yest = spline_eval([xi(:) yi(:)],c,w,norml,method,width);
  % reconstruct matrix
  disp('reconstructing');
  rows = size(xi,1);
  zi = zeros(rows,rows);
  for i=1:rows,
    zi(1:rows,i) = yest( (i-1).*rows+1: i.*rows, 1);
  end;
end;
  
disp('gridding completed');

if fig>0,
  figure(fig);
else
  figure;
end;
clf;

i = find(zi<0);if ~isempty(i),zi(i) = zeros(size(i));end;
% UNCOMMENT ANY OF THESE TO CREATE A DIFFERENT PLOT
%surfc(zi);shading faceted; colormap(pink);colorbar;%faceted;
%surfl(zi);shading interp;colormap(pink);colorbar;
%surfl(zi);shading faceted;colormap(pink);colorbar;%view(0,90);
%surfc(zi);shading interp;colormap(cool);colorbar;view(0,90);grid on;hold on;plot(rows./2,rows./2,'^','markersize',20,'markerfacecolor','auto');

% Plot the heatmap
cool2=cool;
cool2(1,1:3) = [1 1 1];
[c,h] = contourf(zi);colormap(cool2);%shading interp;

colorbar;
hold on;
% rows./2

% Plot the center, cross-hairs
v=axis;
mx = mean([v(1);v(2)]);
my = mean([v(3);v(4)]);

% Triangle in center
%plot(mx,my,'k^','markersize',20,'markerfacecolor','auto');%clabel(c,h);

% Cross-hairs
plot([v(1);v(2)],[my;my],'k-');
plot([mx;mx],[v(3);v(4)],'k-');

% Circles
if 1,
for i=50:50:300, % 100:150:300
    plot(mx,my,'ko','markersize',i);
end;
end;

% Square
plot(meanx(1).*rows,meanx(2).*rows,'kv','markersize',20,'markerfacecolor','auto'); % s p v

axis('square');
%grid on;

xlabel('kpi/targ');ylabel('pacing/targ');

% IF THE X1 AND X2 AXES ARE 1..10 and 0..1.0 THEN PERFORM THIS
x1tick = round([minx(:,1):xrange(:,1)./number_of_tickmarks:maxx(:,1)]'.*100)./100;
x2tick = round([minx(:,2):xrange(:,2)./number_of_tickmarks:maxx(:,2)]'.*100)./100;
% OTHERIWISE USE THIS! (THIS WILL WORK IN THE GENERAL CASE)
%x1tick = [minx(:,1):xrange(:,1)./number_of_tickmarks:maxx(:,1)]';
%x2tick = [minx(:,2):xrange(:,2)./number_of_tickmarks:maxx(:,2)]';

set(gca,'xtick',1:points./number_of_tickmarks:points);
set(gca,'xticklabel',num2str(x1tick));
set(gca,'ytick',1:points./number_of_tickmarks:points);
set(gca,'yticklabel',num2str(x2tick));

% This will capture a movie of the surface. You can save
% and execute it whenever

% for every 1 up to points, 
% this is minx:points:maxx
% so a point in real coords x (x1,x2)
% is equal to (x-minx)./stepsize it is points./number_of_tickmarks 

if 0,
%m=moviein(50); 
a=30;b=30;
for i=1:50,
   a=a+1;
   b=b+1;
   view(a,b);
   drawnow;
 %  m(:,i) = getframe;
end;
%movie(m);
clear
end;
