
function [alpha,scale,xshift,err,yest] = sigmoid_train01(x,y,caseweights,pos);
% [alpha,scale,err,yest] = sigmoid_train01(x,y,caseweights,pos);
%
% Sigmoid function is as follows:
% 
%if pos==0, 
%  y = scale .* (1+-1./(1+exp(alpha.*(x-xshift))));
%else
%  y = scale .* (1+-2./(1+exp(alpha.*(x-xshift))));
%end;
%
% CONSTRAINTS
% 
% scale e [0..1] (!)
% Forces the y-axis to vary from 0..1 only
% Still allows it to xshift etc
% 
% EXAMPLE
%
% % Weighted Sigmoid fit
% % nuh is a 3 column vector with [x,y,caseweightOrObservations]
% [alpha,scale,xshift,err,yest]=sigmoid_train01(nuh(:,1),nuh(:,2),nuh(:,3),0);
% figure(1);clf;scatter(nuh(:,1),nuh(:,2),nuh(:,3).*0.0001);hold on;plot(nuh(:,1),yest,'x-');
% 
% % Weighted Adaline fit
% ww=adaline_train([ones(size(nuh,1),1) nuh(:,1)],nuh(:,2),nuh(:,3));
% figure(2);clf;scatter(nuh(:,1),nuh(:,2),nuh(:,3).*0.0001);hold on;plot(nuh(:,1),polyff(ww',1,nuh(:,1)),'x-');
%


if nargin<4, pos=-1;end;

minerr = inf;

for it=1:10,

scaleinit = max(y).*rand(1,1);
xshiftinit = mean(x);;
alphainit = randn(1,1);

%warning off;
options=optimset('fminsearch');options.Display='on';%'off'; % Stops it from complaining if it runs out of iterations
w = fminsearch(@(w) sigmoid_err01(w,pos,x,y,caseweights),[alphainit,scaleinit,xshiftinit],options);
%w = fminsearch(@(w) adaline_eval(w,x,y,caseweights),rand(1,size(x,2)),options);
%warning on;

alpha = w(1);
scale = w(2);
xshift = w(3);

[err,yest] = sigmoid_err01(w,pos,x,y,caseweights);

if err<minerr | it==1, minerr=err; yestsave = yest; alphasave = alpha;scalesave = scale; xshiftsave = xshift; end;

end;

alpha = alphasave;
scale = scalesave;
xshift = xshiftsave;
err = minerr;
yest = yestsave;

return;

