function tree = split_node(nodeID,x,y,valtree,cattree, ...
                           missing_val,split_metric,min_cases,tree,entropy_xdim);
%
% tree = split_node(x,valtree,cattree,missing_val,split_metric,min_cases);
%
%

% We should implement chi-square, entropy gain

global nodestr;
global tabulist;
global experiment; % 0 means no experiment (usual mode)

if nargin<10, entropy_xdim = ones(1,size(x,2));end;

% we could also pass x, y, valtree, cattree, missing_val, split_metric,min_cases,tree
% as globals

  depth = floor(log2(nodeID));
  depth_limit = 20; % this is normally 32

%tree,

  % Stopping criteria
  % -----------------

  if (size(split_metric,2)>1), 
    % Assume this is a function
    func = char(split_metric);
    split_metric = 9999;
  else 
    func = split_metric;
  end;
    
  stop=0;
  if (split_metric==2 & tree(nodeID,5)==0) | ... % variance of 0
     (split_metric==1 & (tree(nodeID,7)==1 | tree(nodeID,7)==0)),      % probability of 1
    stop=1;
  end;
  if size(x,1)<min_cases,
    %tree = [];
    stop=1;
    %fprintf('mincases');
  end;
  if depth>depth_limit,
    indent(depth);fprintf('depth limit reached\n');
    stop=1;
  end;

  if stop, return; end;


  % Find a split
  % ------------

  % best_entropy = inf;
  % forces it to initialize it first time through
  original_entropy = tree(nodeID,5);
  best_entropy = original_entropy; % sets to variance, or entropy

  % test each dimension for best split
  % ----------------------------------

  for dim=remove_ids([1:size(x,2)],tabulist),

    % test each value on the dimension for best split
    v=1;

    while v<=size(valtree,1) & valtree(v,dim)~=missing_val,
  
      % Calc entropy in left and right partition
      % ----------------------------------------
      
      % to implement an == split, we only need to create an array, 
      % partition no_in_partition
      % for loop to find all in partition, and copy into partition array
      % then we can run this as for partit=1:
      
      l=find(x(:,dim) <  valtree(v,dim) & x(:,dim)~=missing_val);
      r=find(x(:,dim) >= valtree(v,dim) & x(:,dim)~=missing_val);
      
      if ~(size(l,1)==0 | size(r,1)==0), 
      if split_metric==1 | split_metric==3 | split_metric==4, % entropy, gain, chisquare
        % find number of cases each category, given this split
        % choose the most common value
        occurrences = zeros(size(cattree,1)-1,2);
        for i=1:size(cattree,1)-1,
	       occurrences(i,1) = size(find(y(l,:)==cattree(i,1)),1);
	       occurrences(i,2) = size(find(y(r,:)==cattree(i,1)),1);
        end;
        if split_metric==1, 
          % entropy
          entl = entropy(occurrences(:,1)'./size(y,1)) ./ size(occurrences,1);
          entr = entropy(occurrences(:,2)'./size(y,1)) ./ size(occurrences,1);
        end;
        if split_metric==3,
           % entropy gain ratio
           entl = entropy(occurrences(:,1)'./size(y,1)) ./ entropy_xdim;
           entr = entropy(occurrences(:,2)'./size(y,1)) ./ entropy_xdim;
           % this only really makes sense when we are not dealing with real numbers
           % (since every value only occurs once, entropy will be the same in each
           % dim). It also is easiest to calculate when we use == splits
        end;
        if split_metric==4,
           % chisquare
           entl = entropy(occurrences(:,1)'./size(y,1)) ./ entropy_xdim;
           entr = entropy(occurrences(:,2)'./size(y,1)) ./ entropy_xdim;
        end;           
      end;
      if split_metric==2, % variance
        entl = var(y(l,:));
        entr = var(y(r,:));
      end;        
      if split_metric==9999, % function
        if (~experiment),
          entl = feval(func,y(l,:));
          entr = feval(func,y(r,:));
        else
          % here only for purposes of an experiment
          entl = feval(func,x(l,:),y(l,:));
          entr = feval(func,x(r,:),y(r,:));
        end;
      end;

      % Check against best entropy so far
      % ---------------------------------

      ent = (size(l,1).*entl + size(r,1).*entr) ./ size(x,1);
%      fprintf('%f ',ent);
      % Total entropy across partitions

      if ent<best_entropy, 
        best_entropy=ent;
        best_split = [dim valtree(v,dim)];
        lpartition = l;
        rpartition = r;
        if split_metric==1 | split_metric==3 | split_metric==4,
          confl = entl; 
          confr = entr;
          [a,id] = max(occurrences(:,1));yl = cattree(id,1);
          probl = a./size(lpartition,1); % probability
          [a,id] = max(occurrences(:,2));yr = cattree(id,1);
          probr = a./size(rpartition,1); 
          sl = size(l,1);
          sr = size(r,1);
        end;
        if split_metric==2 | split_metric==9999,
           yl = mean(y(l,:));
           yr = mean(y(r,:));
           sl = size(l,1);
           sr = size(r,1);
           confl = entl;
           confr = entr;
           probr=confr;
           probl=confl;
        end;
      end;
      end; % the partition resulted in l and r with 
      % more than zero elements
	
      v=v+1; % look at the next value on this dim

    end; % for each v of this dim
  end; % foreach dim

  % We've now decided the best split
  % Create a node for the split.
  % Call split_node again on the children of this node

  % Record the split
  % ----------------

  if best_entropy==original_entropy,
    % this means that we couldn't get any parititons with greater
    % than 0 elements.
    %fprintf('best entropy isinf\n');
    return;
  end;

  if split_metric==1 | split_metric==3 | split_metric==4,
    method='prob';
  end;
  if split_metric==2,
    method='var';
  end;
  if split_metric==9999,
    method=func;
  end;

  % Display the nodeID and split criteria
  % -------------------------------------

  indent(depth);
  %if isempty(nodestr),
  fprintf('if %s < %f then %f n %d %s %f \n',...
          deblank(nodestr(full(best_split(1,1)),:)), full(best_split(1,2)), ...
          full(yl), full(sl), method, full(probl));
       
  % Call children
  % -------------

  % induce trees for left and right children
  lchild = nodeID.*2;
  rchild = nodeID.*2+1;
  tree(lchild,1:7) = sparse([nodeID best_split yl confl size(lpartition,1) probl]);

  if (experiment & split_metric==9999),
    %%%%%%%%%%% experimental only %%%%%%%%%%
    [err1,intercept,slope,mults,wins] = feval(func,x(lpartition,:),y(lpartition,:),1);
    tree(lchild,8) = intercept;
    tree(lchild,9) = slope;
    tree(lchild,10:9+size(mults,1)+size(wins,1)) = [mults' wins'];
  end;
  
  
  tree = sparse(split_node(lchild, x(lpartition,:), y(lpartition,:), ...
                     valtree, cattree, missing_val, func, ...
                     min_cases, tree,entropy_xdim));
  
  indent(depth);
  fprintf('if %s >= %f then %f n %d %s %f \n',...
          deblank(nodestr(full(best_split(1,1)),:)), full(best_split(1,2)), ...
          full(yr), full(sr), method, full(probr));


  if (experiment & split_metric==9999),
    %%%%%%%%%%%% temporary %%%%%%%%%
    [err1,intercept,slope,mults,wins] = feval(func,x(rpartition,:),y(rpartition,:),1);
    tree(rchild,8) = intercept;
    tree(rchild,9) = slope;
    tree(lchild,10:9+size(mults,1)+size(wins,1)) = [mults' wins'];
  end;



  tree(rchild,1:7) = sparse([nodeID best_split yr confr size(rpartition,1) probr]);
  tree = sparse(split_node(rchild, x(rpartition,:), y(rpartition,:), ...
                     valtree, cattree, missing_val, func, ...
                     min_cases, tree,entropy_xdim));
return


% ----------------------------------

function indent(depth);
  for i=1:depth,
    fprintf('  ');
  end;
return


