function [t,df] = ttest_indep(x,y);
% 
% t = ttest_indep(x,y);
%
% Returns the t ratio of the difference of the means between the two distributions 
% x and y. These are two "independently sampled distributions" that may come 
% from the same or a different population. x and y can have different length. 
% The t ratio is roughly the difference of the means in units of the standard 
% error of the difference distribution (so we can find out if the difference is 
% abnormally large given the variance of the two distributions)
% 
% REFERENCES
%
% Blalock, H. (1972), Social Statistics, McGraw-Hill (for information on t-test)
% 
% SEE ALSO
% 
% This function is exactly the same as ttestc, except it doesn't report a p value
% (that makes use of a function which only exists in the stats toolbox)
% 


meanx = mean(x);
meany = mean(y);
varx  = var(x);
vary  = var(y);

nocasesx = length(x);
nocasesy = length(y);

diff = meanx-meany;

stderr_of_diff = sqrt(varx/nocasesx + vary/nocasesy);

t = diff ./ stderr_of_diff;

zx = ((varx./nocasesx) ./ ((varx./nocasesx) + (vary./nocasesy))).^2./(nocasesx-1);
zy = ((vary./nocasesy) ./ ((varx./nocasesx) + (vary./nocasesy))).^2./(nocasesy-1);

df = 1./(zx + zy);

