function data = zscore2data(zscore,datamean,datastd);
%
% function data = zscore2data(zscore,datamean,datastd);
%
% takes a set of zscores, the mean and std calculated from the original
% data, and transforms those zscores back into their original data
% representation
% Method is to shift data to current mean. Then multiply +/- zscore offset
% by standard deviation. Add that to the mean.
%
% EXAMPLE
% 
% [xn,xmean,xstd] = data2zscore(x);
% [yn,ymean,ystd] = data2zscore(y);
% c = polyfitm(xn,yn,3);
% yestn = polyf(c,3,xn);
% yest = zscore2data(yestn,ymean,ystd);
%
% % this last step transforms the derived yn values, (which are
% % actually zscore values), back to data using the empirically
% % observed mean and variance of the training data set y values.
% % Of course, this won't always be valid.
%
% SEE ALSO data2zscore 
%

data = (ones(size(zscore,1),1) * datamean) + ...
       zscore .* (ones(size(zscore,1),1) * datastd);
