
                                  MDSKit
                        A Multi-dimensional scaling 
                            toolkit in Matlab

                               Brendan Kitts
                            10 Rockwell Street
                          Cambridge MA. 02139. USA
                           Email: bj@datasage.com
                   Web: http://www.cs.brandeis.edu/~brendy

This directory contains a range of functions related to MDS and
high-dimensional visualization. Many of these algorithms are copies of
algorithms used in my other toolkits, for instance, the estimation
toolkit. Therefore, try not to modify the estimators, as these changes
will be overwritten when I create new versions.

The most immediately useful algorithms are: 
torgerson.m                 Performs Torgerson's classic metric projection
direct_optimize_kruskal.m   Non-linear MDS optimization

Distribution and modifications
------------------------------

Please do not distribute this code without first contacting me. Further,
if you have a modification, please isolate this change by surrounding it
with comments. If its a good change, let me know and I'll update my
code.

Speeding up the code
--------------------

If you're using this on a shared system, you might have a C compiler. Try
"build direct_optimize_kruskal". If it works, you'll have really fast
code.

If this doesn't work, type "pcode *.m". This will pre-parse all .m files,
generating .p files, which should be faster. Please note that if you
modify the .m files, matlab will continue to read the .p files, so you
might want to be selective about the .m files you pcode.

Referencing
-----------

Please do not take or use any of these algorithms (including splines,
etc), without referencing. mapdata.m uses a special technique for
converting high-dimensional timeseries data into a trajectory across a
contoured landscape. If you use this, please reference:

Kitts, B., Beding, T., and Harris, C. (1998 submitted), Visualizing
Company Fitness in High dimensional parameter space, Decision Sciences
under review.

When referencing the other algorithms or this toolkits in particular,
please use the following format:

Kitts, B. (1998), MDSKit: A Multidimensional scaling toolkit in Matlab,
http://www.cs.brandeis.edu/~brendy

General
-------

If you have trouble, you can contact me at the email address above. I'm at
BU most evenings, please feel free to contact me, and I'll help you with
the algorithms. 

-----------------------------------------------------------------------

                           CONTENTS

MDS Algorithms
--------------

direct_optimize_kruskal.m       performs MDS on data using the Kruskal
                                metric. Minimizes stress by using Matlab's
                                built-in fmins function
torgerson.m                     Torgerson's metric MDS algorithm
monomds.m                       Monotonic MDS algorithm (not implemented)
stress.m                        stress between two coordinate matrices
stress_wrapper.m                takes in a vector of distances, converts
                                them into a distance matrix
stressd.m                       stress between two distance matrices
mapdata.m                       performs MDS and then interpolates surface
triulist2mat.m                  converts a vector to a triupper matrix
mat2triulist.m                  converts a triupper matrix to a vector
                                (both are used in monomds, which sorts
                                distances)
random_optimize_kruskal.m       attempts to optimize kruskal metric by
                                randomly putting points in the unit
                                hypersphere. Extremely slow. Theoretical
                                interest only
Test-data
---------

build_simplex.m                 builds an n-dimensional simplex
plot_fully_connected.m          Plots a simplex
sk.txt                          Data - Skandia UK Life group
sk2.txt                         Data - American Skandia
sk3.txt                         Data - SkandiaBanken
sk4.txt                         Data - Skandia Real Estate
skproj.m                        Data load - Skandia project loading info

Function estimation algorithms
------------------------------

polyfitf.m                      Polynomial fit
polyff.m                        Polynomial eval
create_vanderf.m                Creates a vandermonde matrix (used by polyfit)
spline_train.m                  Train spline model to fit data
spline_eval.m                   Evaluate spline model at point x
show_splines.m                  Show the spline options available
spline_activ.m                  calculate spline activations
grnn.m                          Generalized regression neural network
memlookup_train.m               Memorization of random set of points
general_model_train.m           trains one of the above estimators
general_model_eval.m            evaluates one of the above estimators
crossval.m                      estimates cross-validation error
vam.m                           Vector associative map
vam_online.m                    Vector associative map

Kohonen algorithm
-----------------

kmeans.m                        kmeans algorithm
myplot_kohonen.m                used to plot the kohonen grid
id2grid.m                       linear id to kohonen grid coords
neighbour.m                     Returns the centroids which neighbour c
                                (neighbours on the mesh grid)
kohonen.m                       runs the kohonen algorithm for a set
                                number of iterations
kohonen_anneal.m                Calls the kohonen algorithm & anneals its
                                neighbourhood. Also plots the kohonen
                                input and output grids
kohonen_initialize_centroids.m  initializes grid so that it spans the
                                input data nicely

Misc functions
--------------

data2zscore.m                   Convert data 2 zscores
data2zscore_missing.m           Convert data 2 zscores with missing vals
zscore2data.m                   Convert zscores 2 data
dist2.m                         distance between two matrices
distance.m                      distance between vectors
distance_activ_function.m       part of spline
distance_missing.m         
eval_simplex_coords.m      
find_closest.m
find_closest2.m                 for each data row in x, and centroid row
                                in c, returns centroid ID which is closest 
find_closest_missing.m
find_reverse_index.m       
basic3Dplot.m                   3D scatter plot of data, with a wire down
                                from each point to the z axis floor to
                                help the user see the points

