function n2 = dist2(x, c, Mean, Square)
%
%	Description
%	D = DIST2(X, C) takes two matrices of vectors and calculates the
%	squared Euclidean distance between them.  Both matrices must be of
%	the same column dimension.  
%
% Originally from Bishop, I modified by Brendan Kitts for use in 
% my functions
% 

if nargin<3, Mean=0; end; % by default, distance is the root sum of squares
if nargin<4, Square=1; end; % by default, distance is the root of the square of differences

[ndata, dimx] = size(x);
[ncentres, dimc] = size(c);
if dimx ~= dimc
	error('Data dimension does not match dimension of centres')
end

n2 = ((ones(ncentres, 1) * sum((x.^2)', 1))' + ...
  		ones(ndata, 1) * sum((c.^2)',1) - ...
        2.*(x*(c')) );
        
if Mean==1,   n2 = n2 ./ dimx; end;
if Square==1, n2 = n2.^0.5;    end;
     

% To do missing value handling properly, we would have to 
% change the sum(x^2) to a sum only those terms where the corresponding c is present
% 