function [c,cy,member] = ...
         kohonen(x,number_of_clusters, lc, nlc, number_of_loops,y,c,cy,neighbsize);
%
% c is the position of the clusters.
% y is: labels supplied for each x (each data point)
% lc is: learning constant
% nlc is no longer used
% neighbsize is the size of the grid neighbourhood
%
% Performs kohonen clustering with centroids constrained by a square grid
% of size neighbsize. Does this for number_of_loops iterations, and at
% learning rate lc.
%
% Note that to anneal the neighbourhood (which is generally required
% if you want the net to come out untangled), use kohonen_anneal. kohonen_anneal
% puts an annealing routine around kohonen, and continuously calls kohonen
% with progressively smaller neighbourhoods
%
% [c,cy,member] = kohonen(x,number_of_clusters, lc, nlc, number_of_loops,y,c,cy,neighbsize);
%
% SEE ALSO
% kohonen_anneal


  % initialize clusters to the k random vectors in the input set
  % ------------------------------------------------------------

  if nargin<7, 
    i = ceil(rand(number_of_clusters,1).*(size(x,1)-1 ));
    c = x(i,:);
    cy = y(i,:);
  end;
    
  % general initialization
  % ----------------------

  member = ones(size(x,1),1) .* (-1);
  closest_inputd = ones(number_of_clusters,1).*inf;
  closest_inputi = ones(number_of_clusters,1).*(-1);
  cx = []; % these are only returned if y is supplied

  % check centroids form a square grid
  % ----------------------------------
  % assume 2D grid
  % c, cy each centroid connected to another
  % Let the topographic space be a 2D matrix, physically
  % implemented
  
  grid_side = sqrt(size(c,1));
  
  if grid_side - round(grid_side) ~=0,
    disp('the grid should be square, so try to make ');
	disp('the number of centroids equal to 25, 36, 49, ');
   disp('and so on');
   keyboard
	return
  end;
  

  fprintf('\n');
  for count=1:number_of_loops,
     
     % randomize order % (this should slow it down a lot, uncomment if possible)
     %r=rand(size(x,1),1);
     %[a,i] = sort(r);
     %x = x(i,:);
     
    fprintf('.'); % (['iteration ' num2str(count)]);
    for i=1:size(x,1),

    % grid_to_id
    % down*grid_side + across  + 1 for the id offset
  
    % id_to_grid tells us what the grid coord is
	% subtract 1 from the id, then run conversion, then 
	% add 1 to down and across to put them onto the 
    % down = floor(id/grid_side), across = rem(id,grid_side)
	
	% final is 
	% grid_to_id
	% (down-1)*grid_side + across-1 + 1
	%
	% id_to_grid
	% down = floor((id-1)/grid_side), 
	% across = rem((id-1)/grid_side),
	%
	%
	% To initialize c to grid coords
	% c=[floor([1:36]./grid_side)'+2 rem([1:36],grid_side)'+2];
    % cy=c;
    
    
    
      % find closest centroid for this datapoint
      % ----------------------------------------
      
      mindist = norm(c(1,:)-x(i,:));
      mini    = 1;
      for cid=2:number_of_clusters,
        d = norm(c(cid,:)-x(i,:));
        if d<mindist,
          mindist = d;
          mini = cid;
        end;
     end;
       
     
     % adjust centroid and its neighbours
     % ----------------------------------
	       
     % Find the neighbours of the winning centroid
     [neighbs,dist] = neighbour(mini,neighbsize,1,size(c,1));
 
     % calculate lc multiplier based on neighbourhood distance
     nlc = (  distance_activ_function(dist,neighbsize) .* ...
        (ones(size(neighbs,1),1)*lc)  );
     nlcx = nlc * ones(1,size(x,2));
     nlcy = nlc * ones(1,size(y,2));

     % adjust neighbours
     c(neighbs,:)  =  c(neighbs,:) + nlcx .* (ones(size(neighbs,1),1)*x(i,:)-c(neighbs,:));
     cy(neighbs,:) = cy(neighbs,:) + nlcy .* (ones(size(neighbs,1),1)*y(i,:)-cy(neighbs,:));
     
     % adjust winning centroid a full step
     c(mini,:) = c(mini,:) + lc .* (x(i,:)-c(mini,:));
     cy(mini,:) = cy(mini,:) + lc .* (y(i,:)-cy(mini,:));
	     
     member(i,1) = mini; % input i belongs to centroid mini
     % is this the closest input to this centroid

    end; % loop over all datapoints  
  end; % number_of_loops

fprintf('\n');
return % return those centroids







% ====================================================================
% notes

	  % find grid position of winning centroid
	  % posy = floor((mini-1)/grid_side)+1; posx = rem((mini-1),grid_side)+1;

if 0,
	  for nx = -1:2:1,
	    for ny = -1:2:1,
		
		  % determine id of neighbouring neuron at (posy+ny, posx+nx)
		  neighb = (posy+ny-1)*grid_side + posx+nx-1 + 1;
		  
		  if neighb>1 & neighb<size(c,1),
		  % only adjust if the id is valid (ignore centroids)
		  % on the border of the grid
            c(neighb,:) = c(neighb,:) + nlc .* lc .* (x(i,:)-c(neighb,:));
            cy(neighb,:) = cy(neighb,:) + nlc .* lc .* (y(i,:)-cy(neighb,:));
   	      end;
		  
		end;
	  end;
  end;
  
  
        if nargin==5,
        mindist = norm(x(i,:)-c(mini,:));
        % because centroid has moved, recalc the mindist
        if mindist<closest_inputd(mini,1),
          closest_inputd(mini,1) = mindist;
          closest_inputi(mini,1) = i;
        end;
      end;
      % the problem with this is that when the centroid moves, it may
      % undo its mindist to another centroid which was previously the 
      % winner. Therefore, if there is a string of movements before
      % leaving the loop, it could be poor. More iterations should reduce
      % this problem. (also, an annealing so that movement slows and stop)
      
      
      if 0,
  if nargin==5,
    % create the cy vector
    % closest_inputd,
    cx = x(closest_inputi,:); % closest input unit to this cluster
    cy = y(closest_inputi,:); % y vector for this closest input unit
    % (the category for the cluster is the same as the category for the
    % closest input to the category, ie. nearest neighbour)
  end;
end;

