function [c,cy,member,xpkohonen,kstress] = ...
   kohonen_anneal(x,grid_side, lc, steps, number_of_loops, ...
                  y,c,cy);
%      
% c is the position of the clusters.
% y is: labels supplied for each x (each data point)
% lc is: learning constant
% nlc is no longer used
% neighbsize is the size of the grid neighbourhood
%
% Note that to anneal the neighbourhood (which is generally required
% if you want the net to come out untangled), use kohonen_anneal. kohonen_anneal
% puts an annealing routine around kohonen, and continuously calls kohonen
% with progressively smaller neighbourhoods
%
% EXAMPLE
%
% % Display topographic mapping ability: The centroids move towards the datapoints
% % but also retain local topological consistency
% 
% % input space datapoints are arranged in a gaussian
% x=rand(20,2)-0.5;x=[x exp(-x(:,1).^2-x(:,2).^2)];
% y=x(:,1);
% [c,cy,member] = kohonen_anneal(x,7,0.005,30,10,sum(x')');
%
% % to solve something useful (the two gaussians problem)
% x = rand(20,2).*4-2;y=x(:,1).*exp(-x(:,1).^2-x(:,2).^2);
% [c,cy,member] = kohonen_anneal(x,10,0.005,30,10,y);
% % now look at c,cy and c,correcty
% correcty = c(:,1).*exp(-c(:,1).^2-c(:,2).^2);
% figure(3);clf;plot(1:100,cy);hold on;plot(1:100,correcty,'r-');
% [cy(1:10) correcty(1:10)]
% 
% [c,cy,member,xpkohonen,kstress] = kohonen_anneal(x,grid_side, lc, steps, number_of_loops, y, c,cy);


  dist = dist2(x,x);
  number_of_clusters = grid_side.^2;
  datapoints = size(x,1);
  
  if nargin<7,
  c  = kohonen_initialize_centroids(x,number_of_clusters,2);
  cy = kohonen_initialize_centroids(y,number_of_clusters,2);
  end;
  % this initializes centroids to span the first two dimensions of 
  % x. This is useful for visualization, however may not give 
  % good clusters
  
  nlc = 0;

  neighbsize      = grid_side./2;
  neighbdecrement = neighbsize ./ steps;
  laststress=inf;kstress=inf;
  
  fprintf('kohonen anneal\n');
  fprintf(' neighbstart %d neighbdecrement %f\n',neighbsize,neighbdecrement);
  if nargin<7, fprintf(' initializing grid\n'); else fprintf(' using supplied centroids\n');end;
  fprintf(' learning rate %d\n',lc);
  fprintf(' %d anneal steps with %d inner adjust loops\n',steps,number_of_loops);
  fprintf('\n');
  
  for ii=1:steps,
     
     neighbsize = neighbsize - neighbdecrement; 
     if neighbsize<1, neighbsize = 1; end;

%while 1,
%if kstress>laststress, 
%  neighbsize = neighbsize + neighbdecrement;
%else
%  neighbsize = neighbsize - neighbdecrement;
%end;

    figure(1);
    myplot_kohonen(x,y,'ro',c,cy,'bo',1,2,'b-');
    laststress=kstress;
    fprintf('neighbsize %f\n',neighbsize);

    [c,cy,member] = kohonen(x,number_of_clusters, ...
                            lc,nlc,number_of_loops,y,c,cy, ...
                            round(neighbsize));
  
  % multidim scaling kohonen net works by keeping the grid 
  % uniform, and then just adjusting its tuning curves, then 
  % we can do the following. Read off the grid coords of each 
  % centroid, and report those as mapping the space at each fitness
  % level. 
  
  % Calculate stress
  % ----------------
  
  % now we just need to select the particular centroids in that grid which
  % the training points mapped to
  % clusters always have their 2D positions (!)
  
  for i=1:datapoints,
     % determine (x,y) grid position of this stimulus input
     % ie. the centroid which picked up that datapoint, get its grid position
     
     mini = member(i,1); % datapoint i's closest centroid
     posy = floor((mini-1)/grid_side)+1; 
     posx = rem((mini-1),grid_side)+1;
     % hardcoded square grid from kohonen & also myplot_kohonen
     % we should turn this into a function [x,y] = neighbourhood(id)

     xpkohonen(i,1:2) = [posx,posy];
   end;    
   
   %keyboard
   % now determine the stress for these projected points, versus
   % the original
  
  kdist = dist2(xpkohonen,xpkohonen);
  kstress = stress(dist,kdist);
  fprintf('\nkohonen stress %f\n',kstress);
  
  % display the high dimensional object
  figure(3);
  % note that the 2D representation of the object is always 2D
  % because we are using a 2D mesh. If we used 3D meshes or a 1D
  % linear neighbourhood chain, we'd have 3D or 1D mappings of the
  % high D object respectively
  xp = [xpkohonen zeros(size(xpkohonen,1),1)];
  plot_fully_connected(xp);
  view(90,90);axis('equal');axis('off');
  drawnow;

end;

