function c = kohonen_initialize_centroids(x,number_of_clusters,gridtype);
%
% c = kohonen_initialize_centroids(x,number_of_clusters,gridtype);
%
% c is initialized to the coordinates of x(:,1;2) (the first two
% dimensions of x), h
% gridtype=2 == 2D grid
%

  if nargin<3, gridtype=2; end;
  
  xdim = size(x,2); % this is the dimension of the input space
  minx = min(x);maxx = max(x);
  xrange = maxx-minx;
  
  
  
  % 2D Mesh grid
  % ------------

  if gridtype==2,
     
     grid_side = number_of_clusters.^0.5;
     
     if xdim==1, 
       % there's only one dimension, but our grid is in 
       % two dimensions. Therefore, initialize a two D grid.
       % eventually we should implement the simple 1D kohonen net
       % (each member is simply equal to its ID)
     
       minx  = [minx minx];
       maxx  = [maxx maxx];
       xrange = [xrange xrange];
    end;
  
    % initialize c to its posx,posy coordinates, 
    % and below we will scale this grid to xrange(:,1:2)
    [posx,posy] = id2grid([1:number_of_clusters]',grid_side);
    c = [posx posy];
    
    % c now equals its grid (1..grid_side across, 1..grid_side down)
    
    c = c - ones(number_of_clusters,1)*min(c); % c starts from zero
    mult = xrange(:,1:2)./max(c);
    c = c.* (ones(number_of_clusters,1)*mult); % scale it to match x's range
    c = c + ones(number_of_clusters,1) * minx(:,1:2); % add the min, 
    
    % c should now exactly span x(:,1:2)
    
    % the other dimensions of c are not yet defined. We could set these
    % to zero, or the mean. Here we set to the mean   
    if xdim>2,
       extra_entries = ones(number_of_clusters,1)*mean(x(:,3:xdim));
       % extra_entries = zeros(number_of_clusters,currentdims-2);
       % for zeros   
    else
       extra_entries = [];
    end;
    c = [c extra_entries];
    
 end;
 
 if gridtype==1,
    disp('gridtype 1 is not supported');
    keyboard
 end;
 
 if xdim==1,
    c = c(:,1);
 end;
      
      
return