function x = monomds(xoriginal,target_dimensions);
%
% function x = monomds(x,target_dimensions);
% 
% Performs monotonic multidimensional scaling of the high-dimensional
% data matrix x, and transforms it into a lower dimensional matrix of
% dimension target_dimensions, such that the distances of points in
% the new space minimize Kruskal's Stress function Sum (d-o)^2/d^2
%
% 

n = size(xoriginal,1);

% Get starting coords of x
% -------------------------

% get the distances for the original matrix x
[mini,dist] = find_closest2(xoriginal,xoriginal);

% select a starting x coordinate configuration in the low dimensional
% space using Torgerson's method of minimizing stress. Distances are
% the same

x = torgerson(dist,target_dimensions);
distp = real(dist2(x,x));


% Perform MDS optimization
% ------------------------

sumdist = sum(sum(dist)');
dist = dist ./ sumdist; 
% multiply all dist elements by a factor, so that 
% the sum of the elements equals 1

% now take square root of these and call this dist. 
% This means that the square of dist has elements
% which when summed equal 1

dist = real(dist.^0.5); 
d    = mat2triulist(dist);

alpha       = 0.001;
young_const = -2; % (sum(sum(dist.^2)'));

err = [];
iteration = 0;
while 1,
   
   iteration = iteration+1;
   fprintf('iteraton %d\n',iteration);

   % standardize the sum of squared distances to equal 1
   % ---------------------------------------------------

   sumdist        = sum(sum(distp)');
   distp          = real((distp ./ sumdist).^0.5); 
   distp_original = distp;


   % Sort distances
   % --------------

   % also known as the non-metric stage, or 
   % optimal scaling

   dp = mat2triulist(distp);

% ********** INVEstigate warnings
% ********* faster method to read in and out of linear array
% (remove trilower, (:), remove last n^2/2 zeros of sorted list
%

   % sort by d,dp and retain the index of each
   % (so if d is tied then order is on db)

   [both,i1] = sortrows([d dp]);
   % i is the sorted order of d,dp

   d  = both(:,1);
   dp = both(:,2);

   j1=find_reverse_index(i1); 

   % Average distances to retain monotonicity
   % -----------------------------------------

changes=1;
while changes>0,
   changes=0;
   
   for m=2:size(dp,1),
   % pass through the list and average ds which are out of order
   
     if dp(m-1,1)>dp(m,1),
       % we've found a set which is out of order
       % read forward and find out whether new element is less than mean
       % (if this is the case, it would mean that after performing mean
       % we'd have to mean that element next turn). Thus, keep looking
       % until we find an element which is in order. Then mean the group
       
       i=m+1;
       while i<=size(dp,1),
         dmean = mean(dp(m-1:i-1));
         if dp(i)<dmean, 
           % if we performed mean, i would be out of order,
           % therefore, keep reading ahead
           i=i+1;
         else
           % i would be greater than dmean, which means that
           % i will be in the correct order. Therefore, we
           % can stop the mean here, and 
           % rest assured that i is a correct element
           break
         end;
         % if its the last element and we still haven't found
         % an element which would be in order, we mean it anyway
       end;
  
       dmean = mean(dp(m-1:i-1));
       dp(m-1:i-1) = ones(i-m+1,1) * dmean;
       m = i;

       % instead of the above we can do
       % dmean = (dp(m-1,1) + dp(m,1)) ./ 2;
       % dp(m-1,1) = dmean;
       % dp(m,1) = dmean;

       changes=changes+1;
     end;
   end; % next distance element
   fprintf('%d distances averaged\n',changes);
end;

% read back into matrix
% not quite, these were sorted. dp
% use dp
% and dp were sorted

dp = dp(j1,:); % put back into previous order
% d =  d(j1,:);
% check this is occurring properly
distp = triulist2mat(dp); % read back into triupper
distp = distp + distp'; % adds in the trilower


% Optimize
% --------


if 0,

  % Davis algorithm
  % ---------------

  %distp_original(i) = distp(i);
  % find all distances which might cause divisions by zero
  % sets bottom distp_original to the same as the top, distp
  % so that when divide they equal 1

  i=find(distp_original==0);
  distp_original(i) = distp(i);
  dterm = (1-distp./distp_original);

  for i=1:size(x,1),
    for k=1:size(x,2),
      deltax(i,k) = -1./size(x,1) .* dterm(i,:) * (x(i,k)-x(:,k));
    end;
  end;
end;

% Young algorithm
% ---------------

if 0,
i = find(distp_original==0);
distp_original(i) = ones(size(i,1),1)*inf;
for h=1:size(x,1),
  for e=1:size(x,2),
    deltax(h,e) = young_const .*  ...
                     ((distp(:,h)-dist(:,h))./distp_original(:,h))' * ...
                     (x(:,e)-x(h,e));
  end;
end;

if 0,
if iteration>2,
  if err(size(err,1),1)<err(size(err,1)-1,1),
    % speed up
    alpha = alpha + 0.1 .* alpha;
  else
    % brake
    alpha = alpha ./ 10;
  end;
end;
end;

x = x + alpha .* deltax;

end;
x = torgerson(distp,2);


% Todo:
%
% take averaging out to create
% 
% x = torgerson(dist,2);
% % report stress
% 
% standardize_distmat(dist);
% standardize_distmat(distp);
% distp_new = monotonic_average(dist,distp);
% % report stress
%
% % given a desired distance matrix, dist, and the present dist
% % matrix, distp, first order distp, and then average distp if it 
% % does not respect the monotonicity of dist order
% %
%
% gradient_optimize(distp_new,distp,x,alpha);
% % adjusts x by a delta vector, given the new distp_new
% % report stress
%
% monomds is
% % the above steps, iterated
%
% monotorgerson is
% % torgerson, monotonic_average, torgerson
%
% % 


% random data (5 points)
% torgerson                                                      4.8
% torgerson plus monotonic averaging then torgerson recovery     4.4
% torgerson, monotinic averaging, one iteration gradient descent 3.6
%
% skandiabanken
% torgerson                                                      2.0
% torgerson, monotonic averaging, torgerson recovery             1.0
% torgerson, monotonic averaging, one iteration gradient descent 3.78
% torgerson, nelder-mead                                         1.7
%
% random data (20 points)
% torgerson                                                      33.2
% torgerson, monotonic averaging, torgerson recovery             50.4
% torgerson, monotonic averaging, one iteration gradient descent 18.1 
% torgerson, nelder-mead                                         13.2
%
% random data (50 points)
% torgerson
% torgerson, monotonic averaging, torgerson recovery             159
% torgerson, monotonic averaging, one iteration gradient descent 839

figure(1);clf;plot(x(:,1),x(:,2),'o'); hold on;
text(x(:,1),x(:,2),num2str([1:size(x,1)]'));

% calculate new distances

distp = dist2(x,x);
f = stress(dist,distp);

fprintf('stress %f \n',f);
err = [err;f];
figure(2);plot(1:size(err,1),log(log(err+1)+1),'-');
%fprintf('press any key\n');
%pause

keyboard
end; % end of while 1


% unravel dist and disp into sequential rows d and dp sorted by d
%d  = dist(:);
%dp = disp(:);
%both = [d dp];
%[both,i] = sortrows(both,
%[d,i] = sort(d);
%dp = disp(:);
%dp = dp(i);

% fix ties
% ties between d(m) and d(m+1) are resolved by putting in order of dp

%for i=2:size(d,1),
%      
%   if d(i-1)==d(i), 
%      
     % found a tie
     % -----------
      
      % find the extent of this block of ties
%      m=i;
%      while d(i-1)==d(m),
%         m=m+1;
%      end;
%      m=m-1;
      % m is now pointing at the last equal d in the block.
%      
%      subd  = d(i-1:m,1);
%      subdp = dp(i-1:m,1);
%      
      % sort this subgroup of equal distances
%      [a,neworder] = sortrows([subd subdp]);
%      
%      d(i-1:m,1)  = a(:,1);
%      dp(i-1:m,1) = a(:,2);



