function myplot_kohonen(x,y, colorpoints, c, cy, colorcentroids, ...
                        n, m, colorlines);
%
% myplot_kohonen(x,y, colorpoints, c, cy, colorcentroids, n, m, ...
%                colorlines);
%
% color of the points, color of the centroids, figure n for input space
% figure m for output space
% c centroid in input space
% cy centroid in output space
%
% EXAMPLE
% 
% x=rand(10,2);y=rand(10,2);
% [c,cy,member] = kohonen(x,25,0.3,0.1,30,y);
% close all
% myplot_kohonen(x,y,'ro',c,cy,'bo',1,2,'b-');
% 

  % add zero dimensions if less than threeD
  % ---------------------------------------

  % Ideally we want to plot this in three dimensions. 
  % If the input is 1 or 2 dimensions, then we can still 
  % use the plot3 function. Fill in the other components
  % of x and c as zeros. Also, we can the rotate the
  % three d plot to a flat viewing (view(0,90))

  xdim = size(x,2);ydim=size(y,2);
  if xdim<3,
    x  = [x  zeros(size(x,1),3-xdim)];
    c  = [c  zeros(size(c,1),3-xdim)];
  end;
  if ydim<3,
    y  = [y  zeros(size(y,1),3-ydim)];
    cy = [cy zeros(size(cy,1),3-ydim)];
  end;

  % Plot datapoints
  % ---------------

  figure(n);
  clf;plot3(x(:,1),x(:,2),x(:,3),colorpoints);
  hold on;
  %plot(c(:,1),c(:,2),colorcentroids);

  figure(m);
  clf;plot3(y(:,1),y(:,2),y(:,3),colorpoints);
  hold on;
  %plot(cy(:,1),cy(:,2),colorcentroids);


  % Plot grid
  % ---------

  grid_side = sqrt(size(c,1));
  mini=[1:size(c,1)]';
  posy = floor((mini-1)./grid_side)+1; posx = rem((mini-1),grid_side)+1;

  pass=0;
  while pass<2,

    pass=pass+1;
    if pass==1,nx=1;ny=0;end;
    if pass==2,nx=0;ny=1;end;

    % mini is the list of all centroids
    mini=[1:size(c,1)]';
    i = find(posy+ny>grid_side | posy+ny<1 | ...
             posx+nx>grid_side | posx+nx<1);
                  
    neighb = (posy+ny-1).*grid_side + posx+nx-1 + 1;
    % find the id for its +nx, +ny neighbouring centroid
                  
    if ~isempty(i),
      % rule out any neighbouring centroids which are 
      % off the grid
      neighb(i,1) = mini(i,1);
      % draw from it to itself
      % neighb = neighb(i,:);
      % mini=mini(i,:);
    end;
		  
    i=find(neighb>=1 & neighb<=size(c,1));
    neighb = neighb(i,:);
    mini   = mini(i,:);
    % rule out any neighbouring centroids which have ids which
    % are outside the maximum range
		  
    figure(n);
    %plot([c(mini,1) c(neighb,1)]',[c(mini,2) c(neighb,2)]',colorlines);
    line([c(mini,1) c(neighb,1)]',...
         [c(mini,2) c(neighb,2)]',...
         [c(mini,3) c(neighb,3)]',...
         'color',[0 0 1]); 
		  
    figure(m);
    plot3([cy(mini,1) cy(neighb,1)]', ...
         [cy(mini,2) cy(neighb,2)]', ...
         [cy(mini,3) cy(neighb,3)]', colorlines);

  end; % next pass
	


