function [neighbs,dist] = neighbour(mini,neighbsize,neighbfunc,maxID);
%
% [neighbs,ndist] = neighbour(mini,neighbsize,neighbfunc,maxID);
%
% returns centroid IDs which are the neighbours
% of centroid with id mini. Does not return mini's ID with the
% neighbours, but just neighbours only. Neighbours are defined by
% assuming a Square Grid. The transform from CentroidID
% to GridX GridY position is performed in this function.
%
% neigbs is a list of neighbourIDs of mini
% ndist  is a list of distances from mini to neighbourID
% neighbfunc == 1 means grid
% neighbsize == size of the neighbourhood
% maxID      == maximum number of IDs
%
% EXAMPLE 
% neighbs = [2;8;4;...], dist = [1.5,0.2,2.4,...];
%
% SEE ALSO
% [c,cy,member] = kohonen(...);

  grid_side = maxID.^0.5;

  % determine x,y position of mini (the center of the neighbourhood)
  posy = floor((mini-1)/grid_side)+1; posx = rem((mini-1),grid_side)+1;  
  
  % determine ids of the neighbours
  % -------------------------------

  dist    = [];
  neighbs = [];

  for nx = -neighbsize:1:neighbsize,
	 for ny = -neighbsize:1:neighbsize,
       
       if (nx==0 & ny==0) ...
          | posx+nx>grid_side | posx+nx<1 ...
          | posy+ny>grid_side | posy+ny<1,

            % don't include the 0,0 case, since we are 
            % returning the neighbours of mini and not mini
            % itself

       else
         % determine id of neighbouring neuron at (posy+ny, posx+nx)
         nID = (posy+ny-1)*grid_side + posx+nx-1 + 1;
         % fprintf('%f %f\n',posx+nx,posy+ny);

         if nID>=1 & nID<=maxID,
         % only include if the id is valid (ignore centroids) on the border of the grid
           neighbs = [neighbs ; nID];
           dist    = [dist    ; sqrt(nx.^2 + ny.^2)];
         end; % if neighbID is valid
       end; % if not 0,0
       
     end; % next ny
   end; % next nx
   
% keyboard
  % [neighbs,i] = unique(neighbs);
   
   % remove non-unique IDs
   
return
