function tot = polyff(c,DEG,x)
%
% yest = polyff(c,degree,x);
% 
% evaluate polynomial c at point x using degree vector degree
% degree is either a scalar (eg. 1,2), or a vector, eg.
% [1 3 1 2 1]. if degree is a vector, then each degree column is
% applied to the corresponding dim of the data matrix, ie. that
% degree is the degree of that dim.
%
% Example
%
% % estimating a multidimensional mapping, (and we happen to 
% % guess the relationship correctly)
%
% x = rand(100,4); y=x(:,1).^2 + x(:,2).^3 - 5.*x(:,3);
% figure(1);clf;plot(x(:,1),y,'x');
% degree = [2 3 1 1];
% w=polyfitf(x,y,degree);
% yest=polyff(w,degree,x);
% figure(1);hold on;plot(x(:,1),yest,'o');
% 


dims = size(x,2);
c    = c';

if size(DEG,2)==1,
  % turn the scalar into a degree array
  % ie. every dim has this as its max power
  DEG = ones(1,size(x,2)) .* DEG;
end;

% multiply out
% ------------

col_id = 2;  % keeps track of what coeff column we are up to
tot    = 0;  % total result

for i=1:dims,
  % calc result for each dim group separately
  for degree=1:DEG(1,i),

    tot=tot+ (x(:,i).^degree) .* ...
             (ones(size(x,1),1) * c(1,col_id));

    col_id = col_id + 1;
  end;
  % continue for other dim groups
end;

% now add the constant
tot = tot + ( ones(size(x,1),1) * c(1,1) );

  
