function dist = standardize_distmat(dist);
% 
% dist = standardize_distmat(dist);
% given a distance matrix, sets its elements
% so that the sum of their elements squared
% is equal to 1
%
% EXAMPLE
%
% dist=rand(20,20);
% dist=standardize_distmat(dist);x=torgerson(dist,2);stress(dist,standardize_distmat(dist2(x,x))),
% d=mat2triulist(dist);distp=standardize_distmat(monotonic_average(dist,dist2(x,x),d));
% xx=torgerson(distp,2);stress(dist,standardize_distmat(dist2(xx,xx))),
%
% SEE ALSO
% 
% dist = standardize_distmat(dist);


sumdist = sum(sum(dist)');
dist = dist ./ sumdist; 
% multiply all dist elements by a factor, so that 
% the sum of the elements equals 1

% now take square root of these and call this dist. 
% This means that the square of dist has elements
% which when summed equal 1
dist = real(dist.^0.5); 

return;