function s = stress(d,dp);
%
% s = stress(d,dp);
%
% Given d and dp (original distance and disparity distance)
% calculates lack of fit between two sets of distances
% Present calculation is sum (d-dp)^2 / dp^2
% To change this to (d-dp)^2 / d^2 (which I think makes more sense - 
% so the stress is the percent difference between d and dp), 
% just call with the d,dp switched, so call stress(dp,d);
%
% EXAMPLE
%
% a = rand(5,2);   
% b = rand(5,2);
% d  = dist2(a,a); 
% dp = dist2(b,b);
% stress(d,dp),


% used to be
% i=find(dp==0);
% dp(i) = ones(size(i,1),1);
%
% if dp is zero, and d is zero, then 0/1 will not contribute to stress
% if dp is zero, and d is a number, then stress will be the difference
% between d and 1. This is a bit weird. 
% Better system is: divide by dp^2 + eps

% treat nans as missing values, they exert no stress on the distance function
ii=find(isnan(d) | isnan(dp)); % can cut out dp because the projection will always
% be filled in
if ~isempty(ii),
  dp(ii) = zeros(size(ii,1),1);
  d(ii)  = zeros(size(ii,1),1);
end;

s = real(sqrt(sum(sum( (d-dp).^2 ./ (dp.^2+eps) )')));

return
