function [s,newxp] = stress_wrapper(xp,dist,dim);
%
% [s,xp] = stress_wrapper(xplinear,dist,targetdims);
%
% Take linear xplinear set and calculate stress with respect
% to dist. Assemble xplinear into an xp coordinate matrix, and 
% find the disparity of the xp coordinates, dp, and 
% call stress(dp,dist);
% 
% Returns newxp (assembled xp coord matrix) and s (stress of 
% that coord matrix when compared to original dist)
%
% This function is called by fmins to optimize the linear
% xplinear vect.
% 
% EXAMPLE
%
% currentdims = 2; targetdims = 1; datapoints = 5;
% x = rand(datapoints,currentdims);
% [mini,dist] = find_closest2(x,x);
% xp = torgerson(dist,targetdims); xptorg = xp; % starting xp
% xp = xp(:);
% xp = fmins('stress_wrapper',xp,foptions,[],dist,targetdims);
% % xdisparity, xoriginal, targetdimensions
% [s,xp] = stress_wrapper(xp,dist,targetdims);
% figure(1);plot(x(:,1),x(:,2),'.');text(x(:,1),x(:,2),num2str([1:datapoints]'));
% [xp xptorg]
%
% % check these against the plot
%
% SEE ALSO
% 
% x_in_2d = direct_optimize_kruskal(x,targetdims);
%
%

rows = size(xp,1)./dim;
newxp = zeros(rows,dim);
for i=1:dim,
  newxp(1:rows,i) = xp( (i-1).*rows+1: i.*rows, 1);
end;

dp = dist2(newxp,newxp);

if 0,
% treat nans as missing values, they exert no stress on the distance function
ii=find(isnan(dp) | isnan(dist));
if ~isempty(ii),
  dp(ii)   = zeros(size(ii,1),1);
  dist(ii) = zeros(size(ii,1),1);
end;
%end;
% check that this doesn't take too long. We could always pre-specify where the nans
% will be in the dist matrices, (but this will be a bit icky to pass in)
end;

s = stress(dist,dp);

return
