function [x,torgdist,torgstress] = torgerson(d,target_dimensions);
%
% function [x,torgdist,torgstress]=torgerson(d,target_dimensions);
%
% Performs Torgerson's metric reconstruction of the underlying
% coordinate vectors, given a matrix of their distances. Projects
% the distance matrix of data from high dimensional space
% to whatever low dimensional space the user desires. 
% Does this by taking the first k principal components 
% of the distance (coordinate) matrix.
%
% Torgerson's algorithm effectively minimizes
% (d-o)^2. Because it minimizes squared discrepency, it should
% tend to favour fitting larger distances rather than getting
% the fine detail
%
% Does Torgerson's algorithm also minimize scalings and translations
% of the distances? (l(dist) = dij * m + c ?
% Scale presumably is handled because d1<d2 -> md1<md2, ie. central 
% scaling does not change the resulting shape of distances.
% d1 < d2 -> d1+c < d2+c So the offset should not change the resulting
% shape of distances. But why then is it mentioned that c is estimated
% and subtracted?
%
% EXAMPLE
%
% xx = rand(5,2);
% figure(2);clf;text(xx(:,1),xx(:,2),num2str([1:size(xx,1)]'));
% [mini,dist] = find_closest2(xx,xx);
% x = torgerson(dist,1); % projects 2D data to 1D
% x, % display those vectors
% % compare those vectors to the plot
%  

if size(d,1)~=size(d,2), 
  disp('matrix is not a distance matrix');return;
end;

%fprintf('performing torgerson''s projection\n');

% double center distance matrix
% -----------------------------

n       = size(d,1);
dcolsum = mean(d.^2);
drowsum = mean((d').^2)';
dsum    = sum(sum(d.^2)') ./ (n.^2); % grand mean

d = d.^2;                      % square distance
d = d - (drowsum * ones(1,n)); % subtract drowsum
d = d - (ones(n,1) * dcolsum); % subtract dcolsum
d = d + dsum;                  % add grand mean

d = -0.5 .* d;
% er, we have to multiply this by -0.5 now?

% d now has a mean of 0 for both row and columns

[u,s,v] = svd(d);
% note that v=u';

x = u * ( [s(:,1:target_dimensions) zeros(n,n-target_dimensions)].^0.5);
x = real(x(:,1:target_dimensions));

torgdist = real(dist2(x,x));
% torgdist = real(x*x'); % why isn't this correct?
% because its no longer double centered after we take the square root

torgstress = real(stress(d,torgdist));

% keyboard

