function [c,cy,mini] = ...
         vam(x,number_of_clusters, lc,number_of_loops,y,c,cy);
%
% function [c,cy,member] = vam(x,number_of_clusters,lc,iterations,y);
% x is a matrix (set of input vectors)
% number_of_clusters = scalar, set by user
% lc = learning rate (generally 0.3)
% iterations is amount of training
% y is the output space (associations are to be learned between input and output)
%
% Batch vam. The algorithm calculates distance
% from all points to each centroid in turn, and then adjusts each winning
% centroid towards all the input rows.
% c contains the locations of the centroids
% cy is category label (or the output vector) that goes for each centroid
% mini is: for each input, the ID of the winning centroid
%
% % VAM Examples
% % train on a set of (x,y)
% 
% clusters=4;iterations=5;lc=0.3;
% x=[1:10;1:10]'; y=x(:,1).^4;
% [c,cy]=vam(x,clusters,0.3,iterations,y);
% xtest=[1:8 9.5 9.9;1:10]';ytest=xtest(:,1).^4;
% [yest] = k_nearest_neighbour(xtest,c,cy,1);
% figure(1);clf;plot(1:10,y,'r-');hold on;plot(1:10,yest,'b-');
%
% figure(2);clf;plot(x(:,1),x(:,2),'ro');hold on;plot(c(:,1),c(:,2),'x');
% axis([1 10 1 10]); % help axis
% set(gca,'xtick',1:10);
% set(gca); % returns a list of the settings which you can adjust on the figure
% set(gca,'fontname','times'); % etc

  % initialize clusters to the k random vectors in the input set
  % ------------------------------------------------------------

  if nargin==7, 
    % if cs supplied, don't initialize
  else
    % otherwise select random inputs for vectors
    i = ceil(rand(number_of_clusters,1).*(size(x,1)-1));
    c = x(i,:);
    cy = y(i,:);
  end;

  % kmeans loop
  % -----------

  for count=1:number_of_loops,
    disp([num2str(count)]); % this just prints out number of loop
    mini = find_closest2(x,c);
    % find closest centroids (mini is a vect of these ids)

    c(mini,:) = c(mini,:) + lc .* (x-c(mini,:));
	cy(mini,:) = cy(mini,:) + lc .* (y-cy(mini,:));
	
    % adjust the mini centroids
  end; % number_of_loops

return % return those centroids


