function [corr,meanabs,meanerr,rankcorr,corr_series,meanabs_series,meanerr_series]= ...
   accuracy_twoseries(act,pred,fig,deciles);
% [corr,meanabs,meanerr,corr_series,meanabs_series,meanerr_series]= ...
%   accuracy_twoseries(actual,predicted,fig,deciles);
%
% Calculate warehouse-level timeseries accuracy, given several item forecasts
% - Plots a graph of summed forecast total quantity versus actual
%   quantity each day.
% - Calculates average percent error across all items
%
% Both series are timeseries, each row down is a different item, going across is time
%

if nargin<2,
   help accuracy_twoseries;
   disp(' ');
   return   
end;
if nargin<3,
   fig=1;
end;
if isempty(fig),
   fig=1;
end;
if nargin<4,
   deciles=100;
end;
if isempty(deciles),
   deciles=100;
end;

te=[1:size(act,2)];
its = [1:size(act,1)];
no_items = size(act,1);
decrange = [1:deciles];

% Plot aggregated amount forecasted versus actual each day
% --------------------------------------------------------

% Plot the aggregated totact and totpred series. These are aggregated
% amounts on each day, and the amount we were off by
figure(fig);clf;
plot(te,sum(pred),'r:');hold on;
plot(te,sum(act),'b-');
legend('predicted','actual');xlabel('day');
ylabel('qty in next 30 days');
title('Own price to predict demand (Aggregate forecast vrs actual 10 items)');

% Calculate the total amount forecasted versus actual for each item
% ------------------------------------------------------------------
% Turn this into % over actual

% Total amount forecasted for each item
sumpred = sum(pred')'; % this gives the total forecasted for item1,item2, etc going down
sumact  = sum(act')'; % total amount in each actual series

% We now have a vertical vector of total amounts. Get the percentage error
% (after 200 days)

% actual was 0 and we forecasted something. This is 100% err
i=find(sumact==0 & sumpred~=0); 
if ~isempty(i),
  meanabs(i,1) = ones(size(i,1),1);
  meanerr(i,1) = ones(size(i,1),1); % we over forecasted by 100%
end;

% actual was 0 and we forecasted 0
i=find(sumact==0 & sumpred==0);
if ~isempty(i),
  meanabs(i,1) = zeros(size(i,1),1);
  meanerr(i,1) = zeros(size(i,1),1);
end;

% actual was nonzero, so we have a legitimate percent error
i=find(sumact~=0);
if ~isempty(i),
  meanabs(i,1) = abs(sumact(i,1)-sumpred(i,1))./sumact(i,1);
  meanerr(i,1) = (sumact(i,1)-sumpred(i,1))./sumact(i,1);
end;

% Calculate correlation between act and pred series, for each item
% ----------------------------------------------------------------

% Correlation of the two series
for i=1:size(pred,1);
   corr(i,1) = correlation(act(i,:)',pred(i,:)'); 
   rankcorr(i,1) = rank_correlation(act(i,:)',pred(i,:)',4);
  % gives a correlation matrix with i,j 
  % being the corr between totalactual and totalpred
end;

% Save the series results
% ------------------------

meanabs_series = meanabs;
meanerr_series = meanerr;
corr_series = corr;
% rankcorr_series = rankcorr;

figure(fig+1);clf;

subplot(3,1,1);
%title('Distribution of absolute percent errors');
% [n,x] = hist(round(100.*meanabs_series),deciles);
x = [100./deciles:100./deciles:100];
n = hist(round(100.*meanabs_series),x);
bar(x,round(100.*n./no_items));
% v=axis;v(1,1) = 0;v(1,2) = 200;axis(v);
xlabel('% error of forecast');
ylabel('% of items');

subplot(3,1,2);
%title('Distribution of forecast bias');
%[n,x] = hist(round(100.*meanerr_series),deciles);
x = [-200+400./deciles:400./deciles:200];
n = hist(round(100.*meanerr_series),x);
bar(x,round(100.*n./no_items));
%v=axis;v(1,1)=-200;v(1,2)=200;axis(v);
xlabel('% bias of forecast');
ylabel('% of items');

subplot(3,1,3);
%title('Distribution of forecast correlations');
x = [-1+2./deciles:2./deciles:1];
% [n,x] = hist(corr_series,deciles);
n = hist(corr_series,x);
plot(x,round(100.*n./no_items));
% v=axis;v(1,1) = -1;v(1,2)=1;axis(v);
xlabel('correlation');
ylabel('% of items');
grid on;

subplot(3,1,3);
%title('Distribution of forecast rank correlations');
x = [-1+2./deciles:2./deciles:1];
% [n,x] = hist(corr_series,deciles);
n = hist(rankcorr_series,x);
plot(x,round(100.*n./no_items));
% v=axis;v(1,1) = -1;v(1,2)=1;axis(v);
xlabel('rank correlation');
ylabel('% of items');
grid on;

% Return the average percent errors and average correlations over all items
% --------------------------------------------------------------------------

meanabs = mean(meanabs);
meanerr = mean(meanerr);
i=find(isnan(corr) | isinf(corr));
corr(i,1) = zeros(size(i,1),1);
corr = mean(corr);
i=find(isnan(rankcorr) | isinf(rankcorr));
rankcorr(i,1) = zeros(size(i,1),1);
rankcorr = mean(rankcorr);

% if we have these bad elements in the corr, should we be plotting those corrs?
