function [str]=analyse_item(itemno,wmat,item,price,itemdescript, ...
                             mincorrelation,no_to_report);
%
% [str]=analyse_item(itemno,wmat,item,price,itemdescript, ...
%                     mincorrelation,no_to_report);
%
% Note: Item must be a single number. This is the item number
% in the w matrix for the item we want to look at competitor 
% relationships with
%
% EXAMPLE 
% analyse_item(7,corrmat2(pricegroup',itemgroup'),itemgroup,pricegroup,group,0.1,15);
% analyse_item(7,wmat,itemgroup,pricegroup,group,0.1,15);
%
% SEE ALSO
% analyse_store, analyse_group, analyse_item

% Parameters
% ----------

diary off
if exist('out_analyse_item'),
   delete('out_analyse_item');
end;
diary out_analyse_item
diary on

%if nargin<3,
%  itemdescript = ones(size(wmat,1),1).*32;
%end;
if isempty(itemdescript),
  itemdescript = ones(size(wmat,1),1).*32;
end;

fprintf('analyse_item\n');
fprintf(' Report for:\t%s\n Item matrix id position:\t%d\n', ...
         char(itemdescript(itemno,:)),itemno);
fprintf('------------------------------------------------\n\n');


% Remove own-price elasticity
% ---------------------------

wmat = wmat-diag(diag(wmat));
itemdescript = char(itemdescript);

% Extract my strategy and other item's strategies on me
% -----------------------------------------------------

% Problem: This only picks up a small number of the interactions, 
% the ones which were predictive in the model. If I use the 
% correlation matrix, I should get a better list of competitor
% and cooperator items

str = zeros(size(item,1),11); % initialize
itemB_strategy = full(wmat(:,itemno)); % item's effect on other products
itemA_strategy = full(wmat(itemno,:)'); % other product's effect on item

competitor     = find(itemB_strategy>0  & itemA_strategy>0);
parasite       = find( (itemB_strategy>=0 & itemA_strategy<0) | ...
                       (itemB_strategy>0 & itemA_strategy==0));
sucker         = find( (itemB_strategy<0  & itemA_strategy>=0) | ...
                       (itemB_strategy==0 & itemA_strategy>0));
cooperator     = find(itemB_strategy<0  & itemA_strategy<0);
validrelation  = find(itemB_strategy~=0 | itemA_strategy~=0);

if ~isempty(competitor),
  str(competitor,1:11) = ones(size(competitor,1),1)*double('competitor ');
end;
if ~isempty(parasite),
  str(parasite,1:11)   = ones(size(parasite,1),1)  *double('parasite   ');
end;
if ~isempty(sucker),
  str(sucker,1:11)     = ones(size(sucker,1),1)    *double('sucker     ');
end;
if ~isempty(cooperator),
  str(cooperator,1:11) = ones(size(cooperator,1),1)*double('ally       ');
end;

% Now we only want to show the strongest of each
% ----------------------------------------------

maxeffect = max(abs([itemB_strategy itemA_strategy])')'; 
% remove low support maxeffects
% i=find(s(itemno,:)<sthresh);
% maxeffect(i,1) = zeros(size(i,1),1);

% vertical vector of maxeffects
[temp,validrelation] = sort(-1.*maxeffect);
validrelation = validrelation(1:no_to_report,1);
% sorts from highest effect to lowest
% filter based on support also

fprintf('wmat cross-effects\n');
fprintf('(strong relationships but perhaps not the only effects)\n');
fprintf('-------------------------------------------------------\n\n');
fprintf('Matrix pos\tItem\tDesignation\tMy cross-deriv\tTheir cross-deriv\n');
for i=1:size(validrelation,1),
  if itemA_strategy(validrelation(i),1)==0 & ...
     itemB_strategy(validrelation(i),1)==0,
     break;
  end;
  fprintf('%d\t%s\t%s\t%f\t%f\n', ...
          validrelation(i),...
          itemdescript(validrelation(i),:), ...
          char(str(validrelation(i),:))',...
          itemA_strategy(validrelation(i),1)',...
          itemB_strategy(validrelation(i),1)');
end;
%keyboard

diary off;
return






% =======================================================================
if 0,

% If we sort in order of relationship(parasite), My strategy, we'll get
% Competitors, the competitors I hurt the most.
% Cooperators, the cooperators I help the most.
% If we sort in order of relationship, Their strategy, we'll get
% Competitors, the competitors who hurt me the most
% Cooperators, the cooperators who help me the most
% Parasites, the worst parasites
% Suckers, the best suckers

% specify item,price,mincorrelation,no_to_report

corr = corrmat2(price',item(itemno,:)')';
% Correlation of demand series for itemno with demand series of other products
% If one product's sales go up, what happens to item's sales?
% We have free license to do this with cross-demand series, because
% we're not using this for prediction in any way. We just want to find
% out about the interactions

% Zero out correlations less than the minimum
i=find(abs(corr)<mincorrelation | isnan(corr) | isinf(corr));
if ~isempty(i),
  corr(i,1) = zeros(size(i,1),1);
end;

% This allows us to designate cooperators or competitors only
% Because the correlation matrix is symmetric, it only shows
% positive correlation between series or negative correlation 
% between series

[a,id] = sort(-1.*corr); % sorts from pos to neg
a=-1.*a; % nans are at the bottom

% If their price increases, my sales increase. I cannibalize them.
% If their price drops, my sales decrease. They cannibalize me.

competitor = id(1:no_to_report,1);
ns = find(isnan(a));
if isempty(ns), 
  maxdown = size(a,1);
else
  [m,maxdown] = min(ns); % pick the first nan
end;
cooperator = id(maxdown-no_to_report+1:maxdown,1);

% Mutual competitors and cooperators
% ----------------------------------

%fprintf('Matrix pos\tItem\t
for i=1:size(competitor,1),
  if corr(competitor(i),1)==0, break; end;
  fprintf('%d\t%s\t%s\tMy strategy:\t%f\tTheir strategy:\t%f\n', ...
          competitor(i,1),...
          itemdescript(competitor(i,1),:), ...
          'mutual competitor',...
          corr(competitor(i,1),1)',...
          corr(competitor(i,1),1)');
end;
for i=1:size(cooperator,1),
  if corr(cooperator(i),1)==0, break;end;
  fprintf('%d\t%s\t%s\tMy strategy:\t%f\tTheir strategy:\t%f\n', ...
          cooperator(i,1),...
          itemdescript(cooperator(i,1),:), ...
          'mutual cooperator',...
          corr(cooperator(i,1),1)',...
          corr(cooperator(i,1),1)');
end;


% Problem is, increasing in unison could be caused by advertising
% together.
% If your sales decrease when the other's increase, it could
% also be because you're not in the same advertising pool as the
% other.
% Cross-price gets around this. If their price drops & your sales
% drop, then that's a real effect. So too if their price drops &
% your sales increase, then that is also probably real...

end;


