function wmatgraph = analyse_store(price,item,itemdescript, itemcategory, ...
                      model,modelspec,w,wmat,cons, ...
                      cost,no_ranks);
%
% [wmatgraph] = analyse_store(price,item,itemdescript, itemcategory, ...
%                              model,modelspec,w,wmat,cons,cost,no_to_list);
%
% This function is ready to rock
%
% Perform retail store analyses
% - best demand/profit items
% - anomalous items (due to badly formed models)
% - strongest impact on demand/profit
% - strongest competitors/cooperators from wmat
% - graphdraw competitors/cooperators from wmat
% - removal candidates
% - priceopt results **
% 
% price, item, cost must all come in with days down (normal array is days across)
% make sure you only send in 365 days of price, otherwize the seasonal items
% such as soup get counted more times than the summer products
%
% EXAMPLE
% analyse_wmat(pricegroup(:,1:365)',itemgroup(:,1:365)',itemdescript,[],[],[],[],zeros(size(price(:,1:365)))',5);
% analyse_store(pricegroup(:,1:365)',itemgroup(:,1:365)',group,model,[],w,wmat,cons,zeros(size(itemgroup(:,1:365)')),20);
% analyse_store(filled_price(catall,1:365)',item(catall,2:366)',itemdescript(catall,:), ...
%   itemcategory(catall,:),model,[],w,wmat,cons,(singlecost(catall,1)*ones(1,365))',5);

diary off;
if exist([pwd '/out_analyse_store'])==2,
   delete([pwd '/out_analyse_store']);
end;
diary([pwd '/out_analyse_store']);
diary on;

% Best and worst profit, qty, score items
% ----------------------------------------

totalqty = sum(item);
totalprof = sum(item.*(price-cost));
totalrev  = sum(item.*price);
meancost = mean(cost);
meanmargin = mean(price)-meancost;
score = data2zscore(full(totalprof'))+data2zscore(full(totalqty'));

display_items(itemdescript,itemcategory,totalqty,meanmargin,'selling items','units sold',no_ranks);
display_items(itemdescript,itemcategory,totalprof,meanmargin,'profit items','total profit',no_ranks);
display_items(itemdescript,itemcategory,score,meanmargin,'profit and sales performers','average zscore',no_ranks);
display_items(itemdescript,itemcategory,totalrev,meanmargin,'revenue items','revenue',no_ranks);

% Anomalous items 
% ---------------

% give weight matrix, wmat, return list of descriptions sorted in size of
% own-elasticity weight possibly, also new cleaned wmat
if 0,
if ~isempty(w),
ownprice = full(diag(wmat)'); % horizontal vector
abovezero=find(ownprice>0 | cons<=0); % cons is a horizontal vector also
% make sure these two vectors are both the same direction
fprintf('There are %d anomalous items\n',size(abovezero,2));
if ~isempty(abovezero),
  fprintf('Items with own price elast > 0 or negative maximum demand\n');
  [price_elast,ii] = sort(-1.*ownprice(abovezero));
  fprintf('Rank\tItem\townprice coefficient\tintercept\n');
  for rank=1:min([no_ranks size(abovezero,2)]),
    fprintf('%d\t%s\t%f\t%f\n',rank, ...
             char(itemdescript(abovezero(ii(rank)),:)), ...
             ownprice(1,abovezero(ii(rank))), ...
             cons(1,abovezero(ii(rank))));
  end;
  % zero out those models
  %wmat(1:size(wmat,1),abovezero(ii(rank))) = zeros(size(wmat,1),1);
  %cons(1,abovezero) = zeros(1,size(abovezero,1));
else
  fprintf('no anomalous item models in wmat\n');
end;
end;
end;

% Gradients
% ---------

  % Strongest pos & neg global revenue gradients, profit gradients,
  % quantity gradients
  % do this externally
  
  fprintf('using prices on day %d as last day\n',size(price,1));
  lastprice = price(size(price,1),:);
  lastqty   = item(size(price,1),:);
  lastcost  = cost(size(cost,1),:);

  [critval,dcritval,ddcritval,wmat1,cons1,qty,rev,prof,dqty,drev,dprof, ...
   totalqty,totalrev,totalprof,dtotalqty,dtotalrev,dtotalprof, ...
   ddtotalqty, ddtotalrev, ddtotalprof] = ...
   demand_model_eval(lastprice,model,modelspec,w,'totalqty',lastcost,wmat,cons);

display_items(itemdescript,itemcategory,dtotalqty,meanmargin,'Effect on global store sales','+1price equals +x sales',no_ranks);
display_items(itemdescript,itemcategory,dtotalprof,meanmargin,'Effect on global store profit','+1price equals +x profit',no_ranks);
% display_items(itemdescript(catall,:),itemcategory(catall,:),dtotalqty(1,catall)',meanmargin(catall,1),'Effect on global store sales','+1price equals +x sales',306);
% display_items(itemdescript(catall,:),itemcategory(catall,:),dtotalprof(1,catall)',meanmargin(catall,1),'Effect on global store profit','+1price equals +x profit',306);




% Removal candidates
% ------------------

% suggest n items which should be removed. Select strong grads, then 
% replace each until we get best effect on prof
%
if 0,
[totalqty,totalrev,totalprof]=contribution(lastprice,wmat,cons,lastcost);

%display_items(itemdescript,dtotalqty,meanmargin,'Removal candidates','Contribution',no_ranks);
display_items(itemdescript,category,dtotalprof,meanmargin,'Profit Removal candidates','Contribution',no_ranks);
end;

% Strongest cross-weights in wmat
% -------------------------------

fprintf('Price cannibalization (wmat)\n');
fprintf('----------------------------\n\n');
keyboard
display_correlations(wmat,no_ranks,[],[], ... 
                     itemdescript,itemdescript, ...
                     ['Driver item\tAffected item\t' ...
                      '+1price = +x affected item demand']);
fprintf('(*) wmat is probably a small version of the relations which are probably going on.\n');
fprintf('see Price cannibalization (corr) for more complete cannibalizations\n');

% Graphdraw wmat
% --------------
% wmat is too big to do a graphdraw from. But we can graph the strongest

  wmatgraph = find_sig_corrmat_els(wmat,no_ranks);
  % extracts out all non-zero cross-elasticities. We'll now graph these
  % call graphdraw
  
  
end;


% Price optimization results
% --------------------------

diary off
return












% ==========================================================
% analyse_item
% parasite_host ----- Single item situation [item or group, requires wmat or wmatgroup]
% - parasite/host summary
% - strongest competitors (CPG: strongest threats, allies you could team with or share ad costs with)
% - drivers and items it drives cross-elasticity, affinity (Retailer: to increase sales of this item, use drivers)
% - forward-backward graph
%
% analyse_store
% analyse_wmat  ------ Item effect on store analysis [item,wmat or group,wmatgroup ]
% - top/bottom sellers, profit, score (profit table - items which you should be pushing more, item table - drawcards)
% - anomalous items (pos elasticities or other deformations)
% - removal items (Retailer: remove these products & increase profit)
% - strongest deriv effects on store profit/sales (Retailer: use these to increase sales or increase profit)
% - (price optimization results table) (Retailer: suggested prices)
%
% analyse_itemgroup
% competitor_anal_script ---- Group-level relationships [group, no wmat]
% - hierarchies (items which should be together)
% - MDS (similar selling items or similar basket items)
% - graph draw similarities (because of the limited number)
%
% Basket level
% - true substitution products
% Attribute-tagging required & basket-level
% - attribute hierarchy (group by attribute & run substitution hierarchy on basket-level data)
% 



