function [prof_contrib,rev_contrib,qty_contrib, ...
          baseline_prob] = contribution_cooc(cooc,qty,price,cost,support);

% [prof_contrib,rev_contrib,qty_contrib,baseline_prob] = ...
%   contribution_cooc(cooc,qty,price,cost,support);
%
% Using the cooc matrix as a model for item joint probabilities,
% determine the total profit,qty contribution for a given item.
% This contribution also provides the removal cost for removing that
% item. 
% Contribution is like an enhanced profit number. Contribution gives 
% the total profit plus secondary purchase probabilities 
%(the item plus its immediate links)
%
% INPUTS and OUTPUTS
% qty is a normal basket form matrix with items across, transactions down
% price is a vector 1,items across, cost is same. Pass in the average
%  price & cost for the timeperiod to ensure prof contribution is accurate
% cooc is a cooccurrence matrix created below
% prof_contrib is a vector 1,items across of total contributions
%
% EXAMPLE
% support=5; cooc = cooccurrence(qty,qty,support);
% prof_contrib = contribution_cooc(cooc,ones(1,size(cooc,2)),...
%   zeros(1,size(cooc,2)),support);
% % actually returns sales contribution, (price was 1, cost 0)
% 
% % Compare prof_contrib against prof for each item
% ii=find(prof
% 

% Zero out items below minimum support
% ------------------------------------

no_occurrences = sum(abs(sign(qty)));
% we'll now zero out items with no_occurrences below support

ii=find(no_occurrences<support);
if ~isempty(ii),
  price(1,ii) = zeros(1,max(size(ii)));
  cost(1,ii)  = zeros(1,max(size(ii)));
  no_occurrences(1,ii) = zeros(1,max(size(ii)));
  % we really want to zero out the ii across entries in cooc, 
  % but its slow to do so. The no_occurrences above will
  % lead to baseline_prob being zero, and will ensure a 0 on
  % this item. Price and cost as 0 will also relieve some pressure
  % in the prof calculation.
end;

baseline_prob = no_occurrences ./ size(qty,1);
% baseline probability of each item appearing in a basket
% if no_occurrences is 0, the baseline_prob will be 0

prof=price-cost;
% I guess that we should have price, cost as matrices of
% same size as qty. To factor in past price points, pass in 
% a price and cost which are averages for the period. 

% Calculate conribution
% ---------------------

% baseprob * prof from own sale + baseprob * prob*prof of other item
prof_contrib = baseline_prob .* (prof  + (cooc * prof')');
rev_contrib  = baseline_prob .* (price + (cooc * price')');
qty_contrib  = baseline_prob .* (1     + sum(cooc'));
% qty_contrib  = baseline_prob .* (cooc * ones(size(cooc,1),1))';
% check that transpose occurs before .*

% Multiply this number by no_transactions to get total losses
% over no_transactions transactions
% so if prof_contrib is 5.25, then every transaction at the store
% this item contributes +5.25. So to get total losses resulting
% from pulling this item, calculate
% no_transactions_per_6_months .* prof_contrib
%

return

% ===============================================
% Bayes theorem
% p(h|e,i) = p(h|i)*p(e|h,i)/p(e|i)
% p(i&e->h) = p(i->h)*p(i&h->e)/p(i->e)
% posterior = prior * liklihood / all possible outcomes
%
% Product rule
% p(a,b|i) = p(a|b,i)*p(b|i) = p(i->b) * p(b&i->a)
%          = p(b|a,i)*p(a|i) = p(i->a) * p(a&i->b)
%
% starting with the product rule, we can derive bayes theorem:
% p(a|b,i)*p(b|i) = p(b|a,i)*p(a|i)
% p(a|b,i) = p(b|a,i)*p(a,i)/p(b|i)
% = bayes theorem
% 
% p(b) = p(a)*p(a->b), assuming no other events influence b
% p(b) = sum (p(a)*p(a->b)) forall a which could be influencing b
% which rule is this called? prior * liklihood
%

