function convert_relational_price2_price_timeseries;
%
% (item, day, price, qty) -> price timeseries
%                            qty timeseries
%
%
% Analysis script
% 

% sqlldr80 userid=brendan/brendan control=control.ctl data=data.dat direct=true

% load mat_priceqty.txt

% Sequential code for rows / items going down
% ------------------------------------------

  [itemcode,vals,mapping] = recode(mat_priceqty(:,1));
  % need a unique sequential code from 1..n to be the
  % row number for the matrix
  % itemcode is a sequential number, 1..n, vals are the origitemvals for each code 1
  % after we convert into our sequential series, we will put vals -> sequentials
  
% Sequential code for columns (time going across)
% -----------------------------------------------

  % If dates are in sequential order, but don't start from 1
  % mintime = min(min(price(:,2:size(price,2))'));
  % price = [price(:,1) price(:,2)-mintime+1];
  % price = [itemcode mat_priceqty(:,[2 3])];
  % item  = [itemcode mat_priceqty(:,[2 4])];

  % If dates are not in order eg. 950101, 
  [datecode,datevals,datemapping] = recode(mat_priceqty(:,2)); % find all daycodes where any transact occurred
  % datecode is a sequential number 1..number of time periods
  % datevals are the original vals of each sequential time period, eg. 
  % 950101 = 5
  price = [itemcode datecode mat_priceqty(:,3)];
  item  = [itemcode datecode mat_priceqty(:,4)];

  % If dates are in sequential order and start from 1,
  price = [itemcode mat_priceqty(:,[2 3])];
  item  = [itemcode mat_priceqty(:,[2 4])];



% keep track of those mappings, as we may 
% want to retrieve the descriptions

% to retrieve description,
% say s2 is the description vector IN ITEM SORTED order
% Then, because recoding recodes in sorted order too,
% the descriptions should go directly with the price,
% item series


% Spconvert
% ---------

price = spconvert(price);
item  = spconvert(item);

item = [vals item]; % vals (original value) -- mapping (recode value) 
price = [vals price];
% firstly, when we do spconvert, it gets sorted into mapping
% order. The mappings are 1:n, and match exactly the order
% of the vals. (100 1000 1500, etc]
% Therefore, the vals can be placed as the key
%

% If price is an amt, fix this
% ----------------------------

qty=item;
price2=price(:,2:size(price,2));qty2=qty(:,2:size(qty,2)); [x,y]=find(qty2>0);for i=1:size(x,1),a=x(i,1);b=y(i,1);price2(a,b) = price2(a,b)./qty2(a,b);end

% Round price to two decimal places
% ---------------------------------

%price2 = round(price2 .*100)./100;
%price = [vals price2];

% If price is in dollars:
price = [vals round(price(:,2:size(price,2)).*100)./100];
% Check the format of price before performing this rounding operation.
% eg. if price is in cents, then this op should round without multiplying by 100

% If price is in cents
price = [vals round(price(:,2:size(price,2)))];



% Anaysis checks
% --------------

% Flick through the item and price series
for i=1:100:size(price,1), figure(1);clf;subplot(2,1,1);bar(item(i,2:size(item,2)));grid on;subplot(2,1,2);bar(price(i,2:size(price,2)));grid on;drawnow;pause;end;
% Display autocorrelations between demand and demand at lag
for i=1:100:size(price,1), figure(1);clf;dispfourier(item(i,2:size(item,2)));drawnow;fprintf('%d ',i);pause;end;


% Fill price series (memory intensive!)
% ------------------------------------

filled_price = [price(:,1) fill_price_array2(price(:,2:size(price,2))];
% price2 is a full array, will consume memory


% Reduce price and item via change_resolution (makes these smaller series, with binned values)
% --------------------------------------------------------------------------------------------

price_month = [price(:,1) change_resolution(filled_price(:,2:size(price,2)),30,1)]; % price is a mean
item_month  = [item(:,1)  change_resolution(item(:,2:size(item,2)),30,0)];

price_week = [price(:,1) change_resolution(filled_price(:,2:size(price,2)),7,1)]; % price is a mean
item_week  = [item(:,1)  change_resolution(item(:,2:size(item,2)),7,0)];
   
   
% Find the correlation between price series and a particular item series
% -----------------------------------------------------------------------

corr = corrmat(price(:,2:size(price,2))'  , item(1,2:size(item,2))');

% Display the best correlations
% -----------------------------

[a,i] = sort(corr);
disp('best correlations: (correlation, seriesID)');
[ a(1:10,1) i(1:10) ],
disp('worst correlation: [correlation, seriesID)');
[ a(size(a,1)-9:size(a,1)) i(size(a,1)-9:size(a,1))],
% really display the two series

% STEPWIZE MODEL CREATION BELOW
% =============================

% Get correlations for an item
% -----------------------------

path(path,'f:\consulting\bkitts\archive\_estimators');
path(path,'f:\consulting\bkitts\archive\_priceopt');

it=10000;

corr=corrmat2(filled_price(:,2:size(filled_price,2))',item(it,2:size(item,2))'); 
% calc correlation
[a,i]=sort(corr); % sort correlations
figure(1);plot(a(1:100)); % top correlations
figure(1);plot(a(size(a,1)-330:size(a,1)-230)); % bottom correlations
fprintf('%f',price(it,1)); % display the itemcode

% Stepwize on most strongly correlated items to predict demand of it
% -------------------------------------------------------------------------------------

% Assume we've calculated a correlation matrix, 
% and got a by sorting the correlations, ie.

% --- DAILY

it = 1; % CHANGE THIS
corr=corr1; % CHANGE THIS (or do above);

[a,i] = sort(corr); 
lowest_corr = min(find(isnan(a)));
trainrange = 1:250;testrange =251:412; % 300 day train, 100 day test
vars = [ it; i([1:30 lowest_corr-30:lowest_corr],1)];                  % top 30, bottom 20
maxdeg = 2;
[v,deg,crit] = stepwize(filled_price(vars,trainrange)',item(it,trainrange)',2,0.01,maxdeg);
if ~ismember(1,v), v=[1 v]; deg=[1 deg];end; % adds the first var, which is it
fprintf('best vars were\n');
vars(v),

w=polyfitf(filled_price(vars(v),trainrange)',item(it,trainrange)',deg);

% test on testrange
yest=polyff(w,deg,filled_price(vars(v),testrange)');

correlation(yest,item(it,testrange)'),
figure(1);clf;plot(item(it,testrange),'b-');hold on;plot(yest,'r-');
figure(1);clf;subplot(2,1,1);plot(item(it,testrange)','b-');subplot(2,1,2);plot(yest','r-');

% --- MONTHLY

it = 1000; % CHANGE THIS
corr=corr1000_month; % CHANGE THIS (or do above);

[a,i] = sort(corr); 
lowest_corr = min(find(isnan(a)));
trainrange = 2:10;testrange =11:14; % 300 day train, 100 day test
vars = [ it; i([1:30 lowest_corr-30:lowest_corr],1)];                  % top 30, bottom 20
maxdeg = 2;
[v,deg,crit] = stepwize(price_month(vars,trainrange)',item_month(it,trainrange)',2,0.01,maxdeg);
if ~ismember(1,v), v=[1 v]; deg=[1 deg];end; % adds the first var, which is it
fprintf('best vars were\n');
vars(v),

w=polyfitf(price_month(vars(v),trainrange)',item_month(it,trainrange)',deg);

% test on testrange
yest=polyff(w,deg,price_month(vars(v),testrange)');

correlation(yest,item_month(it,testrange)'),
figure(1);clf;subplot(2,1,1);plot(item_month(it,testrange)','b-');subplot(2,1,2);plot(yest','r-');
figure(2);clf;bar(item_month(it,2:size(item_month,2)));hold on;plot(price_month(it,2:size(price_month,2)),'--');


% Compare to ordinary elasticity model
% -------------------------------------------------

% assume we have loaded standard elasticity
% load f:\consulting\common\projects\lucy\lucy2_standard_elasticity.mat

p=price_filled(it,testrange); % the price timeseries
% this must be filled to prevent zero prices

% look up the demand of each of these
for i=1:size(p,2),
  pmatch = find(prices(it,:)==p(1,i));
  yest(i,1) = qties(it,pmatch);
end;
 
correlation(yest,item(it,testrange)'),
figure(1);clf;plot(item(it,testrange),'b-');hold on;plot(yest,'r-');


% Compare to model of its own price over time
% -----------------------------------------------------------

deg = 1;
w=polyfitf(price(it,trainrange)',item(it,trainrange)',deg);
yest=polyff(w,deg,price(it,testrange)');

correlation(yest,item(it,testrange)'),
figure(1);clf;plot(item(it,testrange),'b-');hold on;plot(yest,'r-');


% Compare to reduced resolution timeseries (monthly)
% ---------------------------------------------------------------------

% try this on whatever looks promising


% Objective
% =========

% determine if there are any good correlations between price_other and item. 
% look at the two series to check this is real.
%
% prediction with 100 day holdout, based on price_own, price_other (best), 
% demand at lag, and stepwize with vars.
%
% try daily, 1 week and 1 month resolutions for all of the above.
%
% Run standard price elasticity function: Is (price,mean_qty) a better model?
%
% If (price,mean_qty) is better, then it means that cross-effects are very weak,
% and also means that models should not go to the lengths to include these terms
% based on timeseries, etc. This also means an explicit optimization cannot be
% performed, since cross-effects are too flukey/weak to model.
%
% On the other hand, if a good model can be found for each using the timeseries
% and cross-prices, then this will need to be put into the optimization model.
% Optimization model will use demand models for every item, incorporating cross-effects.
%
%


