function [corr,support,lift,no_occurrences,reverse]=cooccurrence(r1,r2);
%
% [cooc,support,lift,no_occurrences,reverse]=cooccurrence(r1,r2);
% 
% returns a matrix of joint probabilities. These are the probabilities
% that r1 and r2 both occurred (had values >0).
% The form of the output matrix is the same as corrmat2
% 
% EXAMPLE
%
% SEE ALSO
% 
% corr = corrmat2(r1,r2),
% jointprop = cooccurrence(r1,r2),
%

% the vanila cooc matrix has the number of times itemi was
% purchased on the diagonal.
% (ie. number of hits or occurrences). But this is actually
% given by the support matrix anyway (on its diagonal).
% Instead we'll now change the diagonal to be the number
% of times purchase i leads to another purchase of i, or 
% probability of second purchase

if size(r1,1)~=size(r2,1),
   disp('rows in r1 and r2 are different');
   disp('routine gives the correlation between each dim of r1 and r2.');
   disp('this is equal to the correlation between a set of vectors across dimensions 1..r');
   return
end;

% Generate the diagonal (own purchase affinities)

own = r1>1; % finds all entries for each item which are 2 or more qty
% labels those with 2 or more with a 1. Now we just have to count
support_diag = sum(own); % number of times >1 were purchased
% cooc_diag    = sum(own)./sum(sign(abs(r1)));
% probability of a >1 purchase. This is actually calculated automatically
% below where we get corr=support./r1total

r1 = sign(abs(r1)); % forces r1 and r2 to have matrices of 1s and 0s (and -1s)
r2 = sign(abs(r2));
support = (r1'*r2);
no_occurrences = diag(support);
support = support-diag(no_occurrences);
support = support+diag(support_diag);
r1total = ones(size(r2,2),1)*sum(r1);
r1total = r1total'; % r1 down r2 across
corr = support ./ (r1total+eps); % prevents the divide by zero

if nargout>=3, % lift affinities
  r2total = ones(size(r1,2),1)*sum(r2);
  lift = corr ./ ((r2total+eps)./size(r2,1));
% note, this is another normalization, divide the affinity by the average
% affinity. a->b / mean(a->b). This will give the percentage over normal affinity.
% Lift affinity, or a->b/p(b) will give the occurrence after seeing b over
% the baseline occurrence in the population
% lift2 = corr ./ (ones(size(corr,1),1)*mean(corr));

% We should also generate zscore affinities
end;

if nargout>=5, % reverse attachments
  reverse = support' ./ (r2total'+eps);
end;

%corr=(r1'*r2) ./ (   ((sum(r1.*r1)'.^0.5)*ones(1,size(r2,2))) .*(ones(size(r1,2),1)*sum(r2.*r2).^0.5));

