function [critval,constraints] = demand_model_constr(price,  ...
                                  penalize, lower, upper, ...
                                  model,w, ...
                                  critstr,cost,wmat,cons);

% [critval,constraints] = demand_model_constr(price,  ...
%                                 penalize, lower, upper, ...
%                                 model,w, ...
%                                 critstr,cost,wmat,cons);
%
% Puts a wrapper around demand_model_eval. This wrapper returns
% constraint violations, or if instructed, penalizes the returned
% function evaluation by adding millions to its return value.
%
% This function is used by fmins to attempt to optimize price whilst
% keeping price within its fixed boundaries.
%


% Eval demand_model_eval
% ----------------------

critval = full(demand_model_eval(price,model,  ...
                            [],w,critstr, ...
                            cost,wmat,cons));

constraints=[];

% Add penalty if instructed
% -------------------------

if penalize,
  constraints = full(max([lower-price ; price-upper]));
  % if the constraints become positive its a violation
  i=find(constraints>0);
  if ~isempty(i),
    critval = critval + sum(constraints(i)) * 1e6;
  end;
end;

return
