function pricevect=discrete_price_optimize(pricevect, ...
    prices, different_prices, ... 
    additional_prices, ... 
    no_simultaneous_changes, lower1, upper1, ...
    model, modelspec,w,critstr,cost,wmat,cons);

% discrete_price_optimize(pricevect, prices, different_prices, 10, ...
%                         no_simultaneous_changes, lower, upper, model, ...
%                         modelspec,w,critstr,cost,wmat,cons);
%
% Optimizes price using gradients and discrete price points. The method
% reads the gradient and adjusts the items with the strongest gradient.
% But it adjusts up or down by skipping to a higher or lower pricepoint.
% This avoids a lot of transitional movement, forces the method to be
% tied to the pricepoints.
%
% price is a horizontal price matrix, with one price for each item 
% going across
%
% prices is a matrix with prices (items) going down, and unique price
% values going across. prices across must be sorted from ascending, 
% without 0 appearing in the first position. Code for generating this
% is in price_opt_script
%


% Strip excessive zeros and add back a controlled number on price
% ---------------------------------------------------------------

  disp('stripping extra zeros');
  [a,b] = find(prices~=0);
  prices = prices(:,1:max(b));
  
  % Add 10 columns of zeros onto the end to give space for adding
  % new prices

  fprintf(['Adding the user specified number of additional price ' ...
           'positions (%d extra possible prices) to the prices ' ...
           'lookup matrix'],additional_prices);

  prices = [prices zeros(size(prices,1),additional_prices)]; 
  % Manually ensure that there are no prices with every day filled


  prev2adjust    = zeros(1,size(pricevect,2));
  prev1adjust    = zeros(1,size(pricevect,2));
  prev2pricevect = pricevect+eps;
  prev1pricevect = pricevect;  

while 1,

  % Read off gradients
  % ------------------

  [critval,dcritval] = ...
   demand_model_eval(pricevect,model,modelspec,w,critstr,cost,wmat,cons);

  dcritval = -1.*dcritval;
  % the reason for this. Below if the gradient points positive, then
  % we adjust positive. This is actually the reverse of what we want
  % to do (if the grad is pos we should go to a lower price). Because
  % we're doing the opposite of what we should be doing, we flip the 
  % sign on this gradient


  % Select the strongest gradients
  % ------------------------------

  % these gradients have the strongest effect on store revenue
  % We will move those items to a higher or lower price point (whatever
  % direction they want to go)

  disp('check for bound');
  nochange = find((dcritval>0 & pricevect==upper1) | ...
                  (dcritval<0 & pricevect==lower1));
  % rule out items which are at their upper bound, and the gradient
  % is taking them higher. ie. we can't possibly do anything with 
  % those items anyway, so don't even consider them. These items'
  % gradients are forced to zero, and in the sort won't be considered.

  full([upper1(1:10);pricevect(1:10);lower1(1:10)]),
  fprintf('constrained bounded\n');
  nochange',
  pause

  dcritval(1,nochange)=zeros(1,size(nochange,2));
  % zero out the gradients of items which are at their upper or lower
  % bound, and are trying to move further out of bound. We won't 
  % respect those gradients (since we can't act on them anyway).
  % Therefore, select a lower strength gradient which can move.

  [strong_gradient,id] = sort(abs(dcritval));
  % Gradients from weakest to strongest

  id=fliplr(id); 
  % Strongest to weakest

  adjust = id(1:no_simultaneous_changes);
  % read off the strongest no_simultaneous_changes gradients

  prev2adjust = prev1adjust;
  prev1adjust = zeros(1,size(pricevect,2));
  prev1adjust(1,adjust) = ones(1,size(adjust,2));

  % Move higher and move lower items
  % --------------------------------

  posadjust = find(dcritval(1,adjust)>0)'; % horizontal vector
  negadjust = find(dcritval(1,adjust)<0)'; % horizontal vector


  % Find the next price point for "move higher" gradients
  % -----------------------------------------------------

  posremaining = posadjust;
  higher=[];
  higherval=[];
  newpricepos=[];
  newpriceval=[];

  for across=1:size(prices,2),
    % We're reading across the prices table, which maintains a list
    % of actual prices for each item
    % look at each pricepoint entry

    z = find(prices(adjust(posremaining),across)~=0);
    % look for prices at this across position with non-zero entries

    if ~isempty(z),
      posremaining = posremaining(1,z);
      % This removes any prices which are now 0 (at the end of the
      % prices across

      % Find higher prices
      % ------------------

      h = find(full(prices(adjust(posremaining),across)) > ...
               full(pricevect(1,adjust(posremaining)))');

      % Finds all prices that are greater than our current price
      % Once we've found one that is higher, because prices is in
      % sorted order from lowest to highest, we know there are no
      % more prices which are even higher. Therefore we report
      % this price as the new move to price

      h,
      if ~isempty(h),
        % Record any found prices into the higher list
%        higher(posremaining(h),1) = ones(size(h,2),1).*across;
        newpricepos(posremaining(h),1) = ones(size(h,2),1).*across;
        % this is the pos across that the final price lives at.
        % We can also record the actual value
%        higherval(posremaining(h),1) = ...
 %                    prices(adjust(posremaining(h)),across);
        newpriceval(posremaining(h),1) = ...
                     prices(adjust(posremaining(h)),across);
        posremaining = setdiff(posremaining,posremaining(h));
        % Remove those prices from further consideration, (since
        % we've found their new price)
      end;
    end; % isempty(z)
  end; % across

  % Find the next price point for "move lower" gradients
  % -----------------------------------------------------

  negremaining = negadjust;
  lower=[];
  lowerval=[];
  % across = different_prices(negadjust,1); 
  % start at the rightmost position of each prices list. 
  % Now work our way back
  %while 1,
    
   for across=size(prices,2):-1:1,
    if ~isempty(negremaining),
      l = find(full(prices(adjust(negremaining),across)) < ...
               full(pricevect(1,adjust(negremaining))') & ...
               full(prices(adjust(negremaining),across))~=0);

      if ~isempty(l),
%        lower(negremaining(l),1) = ones(size(l,2),1).*across;
        newpricepos(negremaining(l),1) = ones(size(l,2),1).*across;
        
%        lowerval(negremaining(l),1) = ...
 %                    prices(adjust(negremaining(l)),across);
        newpriceval(negremaining(l),1) = ...
                     prices(adjust(negremaining(l)),across);
        negremaining = setdiff(negremaining,negremaining(l));
      end;
    end; % negremaining not empty
  end; % across

  % We've now found new prices for each adjustable item
  % Now we'll effect the change


  fprintf('selected for change due to strong derivatives:\n');
  adjust,

  if ~isempty(newpricepos),
 
    % Move pricevect(adjust) to new pricepoints
    % -----------------------------------------

    % Assign pricevect to new prices found in the up or 
    % down direction
    validprice = find(newpricepos~=0); % ids of adjust which made it
    pricevect(1,adjust(validprice)) = newpriceval(validprice)';

    % Record the position of the new price in prices
    down = adjust(validprice)';       % gives the down location in pricevect and prices
    across = newpricepos(validprice); % the position across
    seqid    = down + (across-1).*size(pricevect,2);
    seqid_max = down + size(pricevect,2).*(size(prices,2)-1);

  else
    % No new prices could be moved to
    disp('no new pricepoints found');
%    break;
  end;


  % Look for prices which are back to the same state
  % ------------------------------------------------

  change = adjust(validprice);
  % ids into pricevect which underwent a movement to a new pricepos

  if ~isempty(change),
    same = find(pricevect(1,change)==prev2pricevect(1,change));
    if ~isempty(same),

      % Create new price
      % ----------------

      disp('about to create a new price');

      % make room for new price in prices vector. Note 
      % we continue to respect the sorted ascending order
        
      for p=1:size(same,1),

        % Check if we will move any prices off the end of the
        % list by making room for the new price (shifting the old
        % ones towards the right)

        if prices(seqid_max(same(p)))~=0,
          fprintf(['warning: prices moving off the end of prices ' ...
                  'list. item %d price %f deleted. Increase ' ...
                  'no_additional_prices to allow larger prices. ' ...
                  'This error could be a problem if other variables ' ...
                  'after this one are subject of optimization. ' ...
                  'In this case, this var might need to move again, ' ...
                  'and won''t have these price points to work with. ' ...
                  'On the other hand, the opt would have terminated ' ...
                  'here without the ability to add new pricepoints, so ' ...
                  'by adding points, we''ve gotten further than we ' ...
                  'would have\n'],change(same(p)),prices(seqid_max(same(p))));

          % The last item on the right side is NEVER deleted. The
          % reason is, at a later time this variable may be able to use
          % the dynamic range (the pricepoints) over on the right hand side.
          % The RHS will be impoverished because pricepoints were being
          % deleted from it. However, if we've retained the maxprice
          % on the RHS, we will always be able to create new pricepoints.
          % We pick the maxprice, and then create a new point in between.
          % Therefore, we always have full movement, and can recover
          % in-between points.
 
        end;

        if prices(seqid(same(p)))<prev1pricevect(1,change(same(p))),
          
          % destination price is lower than our last visited price
          % and we are creating a new price. Therefore, 
          % make room to the left
          
          % Adding an item to the left
          % low current others
          %  |
          % seqid(same(p)) destination price
          % low current others
          %              ^ .... become equal to
          %         ^ .....
          % low space   current 
          % newitem goes into position seqid(same(p))+size(pricevec,2)
          % 
          % Adding an item to the right
          % others current high others
          %                 |
          %                 seqid(same(p)) destination price
          %                        ^ ..... become equal to 
          %                 ^ .....
          % others current space high others
          % newitem goes into position seqid(same(p))
          %
          % Sideeffect: Lower bound remains intact and becomes
          % very fine-grained. High upper bounds will be pushed off
          % the end of the finite list. Therefore, they will be lost. 
          % If we start optimizing on a different variable, the ability
          % to move that other var may be impaired a lot.
          %
          % Could get around this by starting each off from their
          % individualized different_price right extremes. Then we 
          % won't have an array crunch & can handle the extra size
          % 
          
          addposition = seqid(same(p))+size(pricevect,2);
        else
          addposition = seqid(same(p));
        end;


        savevec = [addposition:size(pricevect,2): ...
                   seqid_max(same(p))-size(pricevect,2)];
prices(savevec),

        prices(addposition+size(pricevect,2) : size(pricevect,2): ...
               seqid_max(same(p))) = prices(savevec);
               % move the high prices to the right one space

%      keyboard

        % Form Newprice as the average
        % This is half-way between the price 1 turn ago and the 
        % price we're trying to move to (the price 2 turns ago)

        newprice = (pricevect(1,change(same(p))) + ...
           prev1pricevect(1,change(same(p))))./2;

        prices(addposition) = newprice;
        % insert the new prices into the old loc

        pricevect(1,change(same(p))) = newprice;
        % Assign the current price to the newprice in the middle

      end; % create the next price
      fprintf('created %d new prices\n',size(same,1));
    end; % no prices the same
  end; % no price movement recorded

  prev2pricevect = prev1pricevect;
  prev1pricevect = pricevect;

  fprintf('new price after price creation\n');
  pricevect(1,adjust),

  % Keep track of a switch which has gone pos and negative.
  % and update the new item to be half-way

  prev1adjust(1:10),
  prev2adjust(1:10),
critval,
pause,
end; % next









  


% -----------------------------------------------------------------
% NOTES
%
  
  changes=[];

  % adjust(actual_change) are the ids of components in 
  % pricevect which are to be changed (which are to get their 
  % new prices)

  % figure out sequential id location of the prices entry at
  % down,across. This is the prices entry we will be switching to
  % for the adjust(actual_change) entry in pricevect

  actual_change_higher = [];
  if ~isempty(higher),
    % Record non-zero higher
    actual_change_higher = find(higher~=0);
  end;

  if ~isempty(actual_change_higher),
    changes = [changes;adjust(actual_change_higher')];
    down    = adjust(actual_change_higher)';
    across  = higher(actual_change_higher,1);
    seqid    = down + (across-1).*size(pricevect,2);
    seqidmax = down + size(pricevect,2).*(size(prices,2)-1);
    full(prices(seqid));

    fprintf('updating the following higher\n');
    adjust(actual_change),
    fprintf('existing price\n');
    full(pricevect(1,adjust(actual_change))),

    pricevect(1,adjust(actual_change)) = prices(seqid)';

    fprintf('new price\n');
    full(pricevect(1,adjust(actual_change))),
  end;

  pause,
  disp('update lower');

  % Update lower
  % ------------
  
  actual_change_lower = [];
  if ~isempty(lower),actual_change_lower = find(lower~=0);end;
  
  changes_lower=[];
  if ~isempty(lower),
  actual_change = find(lower~=0);
  if ~isempty(actual_change),
    changes_lower=[changes_lower;adjust(actual_change)']; 
    down   = adjust(actual_change)';
    across = lower(actual_change,1);
    seqid_lower = down + (across-1).*size(pricevect,2);
    seqid_lowermax   = down + size(pricevect,2).*(size(prices,2)-1);
    full(prices(seqid_lower));

    fprintf('able to update the following\n');
    adjust(actual_change),
    fprintf('existing price\n');
    full(pricevect(1,adjust(actual_change))),

    pricevect(1,adjust(actual_change)) = prices(seqid_lower)';

    fprintf('new price\n');
    full(pricevect(1,adjust(actual_change))),
  end;
  end; % isempty(lower)
pause,

