function display_corrmat(corr,latent_class,levels,fig);
% 
% display_corrmat(corr,varnames,levels,fig);
%
% takes a correlation matrix, and displays it as an image with 
% shaded squares
%
% variable_names    is a string matrix of names for each of the size(corr,2) vars
% intensity_levels  is the number of different shading gradations.
%                   The more shading gradations, the finer the ability to discriminate differences
% relative==1       means show matrix with correlations relative to each other,
%                   so a 0.4 is max, and -0.2 min
% relative==0       means show matrix with fixed scale -1..1
% slip    ==1       show correct range
%         ==2..10   show half range, third range, and so on
% fig               starting figure (1)

remove_diagonal=0;
% set this to 1 if we want to remove the diagonal. This is 

corr_orig=corr;

for slip=1:20,
for relative=0:3,
   
   corr=corr_orig;
   
if relative==0, % Absolute -1..1 correlation
   cmin = -1;
   cmax =  1./slip;
   offset  = 1;
   tit = 'Correlation';
end;
if relative==1, % min(corr)..max(corr)
   offset  = -1.*min(corr(:));
   d    = diag(diag(corr)); % remember the diagonal
   corr = corr - d;
   cmin = min(corr(:)); % (min(corr(:))+offset).*multiplier;
   cmax = max(corr(:))./slip; %(max(corr(:))+offset).*multiplier;
   % add back the ones on the diagonal
   corr = corr + d;
   tit = 'Correlation (relative scale)';
   % to make this work we need a log scale, where we have a lot of resolution
   % through the middle ranges, and little on the high ranges. We could take
   % this decile1 approach to get an equal number of values in each level.
   % code them into decile1, then replace with rank
   % This would look pretty with the blue and red of the jet colormap,
   % but not for now
end;
if relative==2, % positive correlations
   d = diag(diag(corr));
   corr = corr - d;
   ii=find(corr<0);corr(ii)=zeros(size(ii,1),1);
   cmin = 0;
   cmax = max(corr(:))./slip;
   offset=0;   
   corr = corr + d; % add back the ones on the diagonal
   tit = 'Positive correlations only';
end;
if relative==3, % negative correlations
   d = diag(diag(corr));
   corr = corr - d;
   ii=find(corr>0);corr(ii)=zeros(size(ii,1),1);
   corr = -1.*corr; % note that these were negatives
   cmin = 0;
   cmax = max(corr(:))./slip;
   offset=0;   
   corr = corr + d;   % add back the ones on the diagonal
   tit = 'Negative correlations only';
end;
if relative==4, % min(corr)..max(corr)
   offset  = -1.*min(corr(:));
   d    = diag(diag(corr)); % remember the diagonal
   corr = corr - d;
   cmin = median(corr(:))-median(abs(corr(:)-median(corr(:)))); % (min(corr(:))+offset).*multiplier;
   cmax = median(corr(:))+median(abs(corr(:)-median(corr(:))))./slip; %(max(corr(:))+offset).*multiplier;
   % add back the ones on the diagonal
   corr = corr + d;
   tit = 'Correlation std lower and upper';
   % to make this work we need a log scale, where we have a lot of resolution
   % through the middle ranges, and little on the high ranges. We could take
   % this decile1 approach to get an equal number of values in each level.
   % code them into decile1, then replace with rank
   % This would look pretty with the blue and red of the jet colormap,
   % but not for now
end;

range = cmax-cmin;

% spread range through 1000 intensity levels
multiplier = levels ./ range;


% Show intensity figure
% ----------------------

figure(fig+relative);clf;
image(round((corr+offset).*multiplier)+1);
colormap(pink(levels+1));
set(gca,'ytick',[1:size(latent_class,1)]');
set(gca,'xtick',[1:size(latent_class,1)]');
set(gca,'yticklabel',latent_class);
set(gca,'xticklabel',latent_class(:,1:2));
set(gca,'fontsize',8);
cbar=colorbar;
ytick = get(cbar,'ytick');
set(cbar,'yticklabel',num2str((ytick'-1)./multiplier-offset));
title(tit);

% we need to call a graph drawing package (my own code) now
% to view the correlation graph which results (at this slip threshold)
% Items below thresh would be breaks. This would be nice

% Three-d correlation version (not very useful)
% ---------------------------------------------

if 0,
im_range = [1:size(latent_class,1)]';
figure(fig+4+relative);clf; % 4 is max relative
surf(round((corr+offset).*multiplier)+1);
colormap(pink(levels+1));
xlabel('x');
    ylabel('y');
    set(gca,'YAxisLocation','right');
    set(gca,'XAxisLocation','bottom');
    set(gca,'ytick',im_range');
    set(gca,'xtick',im_range');
    set(gca,'yticklabel',latent_class(im_range,:));
    set(gca,'xticklabel',latent_class(im_range,:));
    set(gca,'fontsize',6);
    ytick=get(gca,'ztick');
    set(gca,'zticklabel',num2str((ytick'-1)./multiplier-offset));
    colorbar;
    cbar = get(gcf,'children');
    ytick=get(cbar(2),'ytick');
    set(cbar(2),'yticklabel',num2str((ytick'-1)./multiplier-offset));
title(tit);
end; % if 0

end; % next relative

fprintf('%d ',slip);pause;

end; % next slip