function price=fill_price_array(price);
%
% p=fill_price_array(price);
%
% a price array is a timeseries where a price or qty
% is recorded on a particular day. The days in-between
% purchases have no price recorded (are zero), even though the
% price was implicitly active. This routine fills those holes by
% cascading the price across the days where no price is recorded
%
% price series is on its side: time goes across
%                         item1
%                         item2
% 

for i=1:size(price,1),
   % find the first price. 
   % We need this to know what the initial currentprice is
   start=1;
   while price(i,start)==0, 
      start=start+1; 
   end;
   currentprice = price(i,start);
   % set the current price
   
   % backfill
   price(i,1:start) = ones(1,start).*currentprice;
   
   % while keeping track of the current price, fill in the zeros
   for t=start:size(price,2),
      if price(i,t)==0,
         price(i,t) = currentprice;
      else
         currentprice = price(i,t);
      end;
   end;
end;

    