function g = generate_graph_format(edge,nodedescript,directed,formatstr);
%g = generate_graph_format(edge,nodedescript,directed,formatstr);
%
% formatstr == 'tomsawyer'
%           == 'malf_in'
%           == 'tomsawyercomplex'
% directed == 1 means draw arrows, == 0 means no arrows
%
% EXAMPLE
%
% 
% corr  = corrmat2(itemgroup',itemgroup');
% corr2 = corr-tril(corr); % since its symmetric, don't report the 
% num=100;corrlist = find_sig_corrmat_els(corr2,num);
% generate_graph_format(corrlist,group(:,:),0,'tomsawyercomplex');

%fprintf('generate_graph_format\n');
%fprintf('generating output file. Results written to out_graph\n');

% Parameters
% ----------

if ~strcmp(formatstr,'tomsawyer') & ~strcmp(formatstr,'malf_in') & ...
   ~strcmp(formatstr,'tomsawyercomplex'), 
   disp('didn''t recognize the output graph format.');
   disp('possible formats are tomsawyer, malf_in, tomsawyercomplex');
   return
end;

% Graph formats can't deal with spaces in descriptions. Replace with _
% --------------------------------------------------------------------

nodedescript = char(nodedescript);
if ~strcmp(formatstr,'tomsawyercomplex'), 
   % tomsawyercomplex is the only format that doesn't have problems with spaces
   for i=1:size(nodedescript,1),
   % deblank it
     st = deblank(nodedescript(i,:));
     % remove any spaces inside that string
     ii=find(st==' ');
     st(ii)=char(double('_')*ones(1,size(ii,2)));
     nodedescript(i,1:size(st,2)) = st;
  end;
end;

%nodedescript = nodedescript(:,1:25); 
% truncate it at 20 (graph drawing package seems to have trouble)


% Switch on diary
% ----------------

%if exist([pwd '/out_graph']),
%   delete([pwd '/out_graph']);
%end;
delete out_graph; % better hope this deletes the right file
diary off
diary out_graph
diary on

existing_nodes = unique([edge(:,1);edge(:,2)]);
% list of nodes listed somewhere in the edge list

%keyboard
% tomsawyer format
% ----------------

if strcmp(formatstr,'tomsawyer'),
   
   % node clause
   fprintf('// nodes\n');
   for i=1:size(existing_nodes,1),
      fprintf('%s\n',nodedescript(existing_nodes(i,1),:));
   end;
   
   % edges clause
   if size(edge,2)>=3,
      % edge weight, nodename, nodename
      fprintf('// named edges\n');
      for i=1:size(edge,1),
        fprintf('%s\n',num2str(edge(i,3))); % edge title
        fprintf('%s\n%s\n',nodedescript(edge(i,1),:),nodedescript(edge(i,2),:));
      end;   
   else
      % nodename, nodename
      fprintf('// edges\n');
      for i=1:size(edges,1),
        fprintf('%s\n%s\n',nodedescript(edge(i,1),:),nodedescript(edge(i,2),:));
      end;   
   end;
end;

if strcmp(formatstr,'malf_in'),
   % node clause
   for i=1:size(existing_nodes,1),
      fprintf('%d %s\n',i,nodedescript(existing_nodes(i,1),:));
   end;
   
   % edges clause
   fprintf('#\n');
   edgecount=1;
   for i=1:size(edge,1),
     % find source % dest on existing_nodes list
     source = find(edge(i,1)==existing_nodes);
     dest   = find(edge(i,2)==existing_nodes);
     % note, source,dest edge(i,1),edge(i,2) if we are allowed to use their
     % real ids
     if source~=dest, % graph drawing prog seems to have trouble with loops
        fprintf('%d %d %d %d\n',edgecount,directed,source,dest); % edge title
        edgecount=edgecount+1;
     end;
   end;   
end;

if strcmp(formatstr,'tomsawyercomplex'),
   % prior to the header there are a lot of font definitions, and so on
   % these can be skipped, or prepended. Examples are in the numerous tomsawyer
   % files
   
   % directed/undirected
   fprintf('// undirected\n');
   if directed==1,
      fprintf('// FALSE\n');
   else
      fprintf('// TRUE\n');
   end;
   
   % header
   fprintf('// Editor Digraph\nVersion 3.0\nUntitled\n// Digraph Background Color\n16777215\n// Topology\n// coordinates\nTRUE\n');
   
   % nodelist
   for i=1:size(existing_nodes,1),
      fprintf('// Nodes\n// Node\n// name\nNode%s\n// x\n%d\n// y\n%d\n// width\n%d\n// height\n%d\n', ...
         num2str(i),1,1,300,23); % WAS 250 WAS 180
      fprintf('// Editor Node\nVersion 3.0\n%s\nTSETextView\n0x400080\n0xff00\n1\n', ...
         nodedescript(existing_nodes(i,1),:));
   end;
   
   % edges
   fprintf('#\n');
   for i=1:size(edge,1),
     % find source % dest on existing_nodes list
     source = find(edge(i,1)==existing_nodes);
     dest   = find(edge(i,2)==existing_nodes);     
     % note, source,dest edge(i,1),edge(i,2) if we are allowed to use their
     % real ids
     
     fprintf('// Edges\n// Edge\n// name\nEdge%d\n// fromNodeName\nNode%d\n// toNodeName\nNode%d\n', ...
        i,source,dest);
     fprintf('// Path\nBegin\n// Edges\n// Edge\n// name\nPathEdge%d\n// fromNodeName\nNode%d\n// toNodeName\nNode%d\n', ...
        i,source,dest);
     fprintf('// Path\nEnd\n// fromPadX\n-69\n// fromPadY\n-262\n// toPadX\n-97\n// toPadY\n-109\n// Labels\nBegin\n');
     fprintf('// Label\n// name\n%5.2f\n// width\n36\n// height\n19\n// x\n-104\n// y\n-178\n// style\nLABEL_LEFT_OF_EDGE\n', ...
              edge(i,3));
     fprintf('// association\nLABEL_TOWARDS_BOTTOM_OF_EDGE\n// Editor Label\nTSELabelView\n0x800000\n0xffff\n0\n1\n');
     fprintf('// Labels\nEnd\n');
   end;   
   
end;

diary off
if nargout>0, 
   g = char(string_load('out_graph')); 
   % problem: PC and Unix systems may or may not include a tab
end;

return
