%
% Analysis Script: graph_dmm_affinity
% Author: Brendan Kitts
% Date: August 12, 1999
%


filepath = 'h:\consulting\common\projects\AdKnowledge\data';
siteAfile = 'siteA.txt';
siteBfile = 'siteB.txt';
datfile   = 'siteMeasures.txt';

siteA = string_load([filepath '\' siteAfile]);
siteB = string_load([filepath '\' siteBfile]);
dat=load([filepath '\' datfile]);

% Positions in the dat file
supportA = 1;
supportB = 2;
coocAB   = 4;
lift     = 8;

maxdown = max([size(dat,1) size(siteA,1) size(siteB,1)]);
% find the longest record (all should be the same size)
i=find(siteA==0);siteA(i)=ones(size(i,1),1).*32;
i=find(siteB==0);siteB(i)=ones(size(i,1),1).*32;
% remove 0s in the strings. This is caused because of my string_load algorithm
% also, watch out for an extra blank at the beginning or the end. My string load
% could add that

% fill cols with spaces or zeros, if they are less than full size
if size(siteA,1)<maxdown,
   siteA(size(siteA,1)+1:maxdown,1:size(siteA,2)) = ones(maxdown-size(siteA,1),size(siteA,2)).*32;
end;
if size(siteB,1)<maxdown,
   siteB(size(siteB,1)+1:maxdown,1:size(siteB,2)) = ones(maxdown-size(siteB,1),size(siteB,2)).*32;
end;
if size(dat,1)<maxdown,
   dat(size(dat,1)+1:maxdown,1:size(dat,2)) = zeros(maxdown-size(dat,1),size(dat,2));
end;

siteA=char(siteA);
siteB=char(siteB);

% Create nodedescript
% -------------------
% this is a lookup with all the unique values of site descriptions
% each site then is recoded as a number with an id into this unique list

vals = unique([siteA;siteB],'rows');
% we just want to get a unique id for all the values in these two sites
% these vals become the nodedescript list

% Create nodeID
% -------------
% this is a vector of ids into the nodedescript list
% siteA gets recoded into nodeID(:,1), and nodedescript(siteA,:)
% retrieves the original siteA string

% foreach val, recode siteA and siteB to this respective val
nodeID = zeros(size(siteA,1),2);
tobeassignedA = [1:size(nodeID,1]';
tobeassignedB = [1:size(nodeID,1]';
for i=1:size(vals,1),
   % find entries in siteA and siteB which are equal to vals(i,:)
   ii=strmatch(vals(i,:),siteA(tobeassignedA,:));
   if ~isempty(ii),
      nodeID(ii,1) = ones(size(ii,1),1).*i;
      tobeassignedA = setdiff(tobeassignedA,tobeassignedA(ii));
   end;
   
   ii=strmatch(vals(i,:),siteB(tobeassignedB,:));
   if ~isempty(ii),
      nodeID(ii,2) = ones(size(ii,1),1).*i;
      tobeassignedB = setdiff(tobeassignedB,tobeassignedB(ii));
   end;
   fprintf('%d/%d ',i,size(vals,1));
end;

if ~isempty(tobeassignedA) | ~isempty(tobeassignedB), 
   fprintf('Problem: Some of the siteA or siteB strings did not match up\n');
   fprintf('with the unique values in vals. There must be a nodeID for these\n');
   % nodeID(:,1) = nodeID(setdiff([1:size(siteA,1)],tobeassignedA);
   % we could delete them, as above
end;

% check that all nodeIDs were converted
i=find(nodeID(:,1)<=0);
if ~isempty(i), 
   fprintf('Problem: siteA pos %d down could not find a unique value which matched in vals\n', i(1));
   fprintf('look at i for more values\n');
end;
i=find(nodeID(:,2)<=0);
if ~isempty(i), 
   fprintf('Problem: siteB pos %d down could not find a unique value which matched in vals\n',i(1));
   fprintf('look at i for more values\n');
end;


% Generate graph
% ---------------

affinity_measure = lift; % affinity measure to show
n                = 500;  % number of lifts to show
minsupport       = 100000;

abovesupport = find(dat(:,supportA)>minsupport & dat(:,supportB)>minsupport); % 0.0001
[a,i] = sort(-1.*dat(abovesupport,affinity_measure));
i=i(1:n);
%i=i(1:2:n); % if we have duplicates
generate_graph_format([nodeID(abovesupport(i),1:2) dat(abovesupport(i),affinity_measure)],vals,1,'tomsawyercomplex');

