function [group,dataout,count] = group_by(key,data,operation);
% [group,dataout,count] = group_by(key,data,operation);
% operation=='sum', 'median', etc
%
% performs a group by operation on the data
% 
% EXAMPLE
% [group,pricegroup,count] = group_by(itemcategory,filled_price,'mean');
% 

% Parameters
% ----------

% Sort key and data so that groups are together
% ---------------------------------------------

fprintf('sorting the key values into order (groups are now together)\n');
[key,kid] = sortrows(key);
% sort key so that the duplicates are together. We can then find where the
% group boundaries occur by doing a diff operation. We can then just skip through
% the list, only looking at the groups, and apply the operation to each group
data      = data(kid,:);
% data has to be put into key order

% Find the break points - the points at which the key transitions to a new key
% -----------------------------------------------------------------------------

fprintf('differencing the sorted key to find boundaries where the key changes\n');
keydiff = diff(key); % difference key(i+1)-key(i), 
% if keys i is different to i-1, this will show up as nonzero entry at i-1 (we add 1 below to get i)
fprintf('searching for key change boundaries\n');
keystart = find(sum(abs(keydiff)')'~=0);
keystart = [1;keystart+1;size(key,1)];
% keystart is the list of start ids for each group.
% 1 is added, because the first group in the sort was actually omitted because
% the difference subtracted 2-1. ie. it only saw the transition from 1 to 2, not
% the start group with 1.
% size(key,1) is added, so that below we will always sum from i to i+1
% however, this last group isn't calculated - it is just an end stop.

% Perform the operation on each group
% -----------------------------------

count = zeros(size(keystart,1)-1,1);
fprintf('performing %s operation on %d groups\n',operation,size(keystart,1)-1);
for i=1:size(keystart,1)-1, 
% these are the number of different groups
% skip the last one
  group(i,1:size(key,2)) = key(keystart(i,1),1:size(key,2));
  count(i,1) = 1+keystart(i+1,1)-keystart(i,1);
  if keystart(i+1,1)-keystart(i,1)>1,
    str = ['dataout(i,1:size(data,2)) = ' operation '(data(keystart(i,1):keystart(i+1,1)-1,:));'];
    eval(str);  
  else
    eval(['dataout(i,1:size(data,2)) = data(keystart(i,1),:);']);
  end;
  if rem(i,100)==0, 
    fprintf('%d/%d ',i,size(keystart,1)-1);
  end;
end;
fprintf('\n');
