function [prof, dprof] = logistic_priceprofit_curve(price,cost,qty,b);
%
% [prof, dprof] = logistic_priceprofit_curve(price,cost,qty,b);
% 
% SYNOPSIS
% 
% Return the profit, and the derivative of profit, assuming a  
% profit equation of the following form:
% 
%      prof = winprob * qty * (price-cost)
% 
% where winprob is a logistic function of price, L(price,b(1),b(2))
% The profit and derivative of profit allows an optimum price to be 
% found using the gradient_optimize routine
% 
% EXAMPLE
%
% %%%% Plot the profit curve
% 
% qty=1;price=[1:0.1:10];cost=0.8;b(1)=-5;b(2)=1;
% prob = 1./(1+exp(b(1)+b(2).*price));plot(price,prob,'-');
% % prof=qty.*(price-cost).*prob;
% [prof,dprof] = logistic_priceprofit_curve(price,cost,qty,b);
% figure(1);clf;hold on;
% plot(price,prob,'-');
% plot(price,prof,':');hold on; plot(price,dprof,'--');
% 
% %%%% Optimize the profit curve
% 
% [x1,err] = gradient_optimize('logistic_priceprofit_curve',2,[],[],1,0.1,inf,inf,1,0,0,cost,qty,b);
% fprintf('%f\n',x1);
% [prof,dprof] = logistic_priceprofit_curve(x1,cost,qty,b),
% 
% SEE ALSO
%
% gradient_optimize, newton_optimize
% 

%keyboard;

prob  = 1./(1+exp(b(1)+b(2).*price));
prof  = qty.*(price-cost).*prob;
dprof = (price - 1./(b(2).*exp(b(1)+b(2).*price))) + (cost + 1./b(2));

% Done because we want to maximize 
prof = -1.*prof;
dprof = 1.*dprof;

return;

% plot(price,prob,'-');
% hold on;plot(price,prof,':');


% Form funstr must be
% [f(x), df(x)] = funfcn(x,otherparams)
% x and df are horizontal vectors of the same size
%  f(x)  = scalar
%  x     = [x1 x2 x3 ... xn]
%  df(x) = [df/dx1 df/dx2 df/dx3 ... df/dxn]


%function [err,dtotalerr, ddtotalerr,derr,dderr,y] = ...
%          adaline_eval(w,x,ytarget,caseweights);