function [str]=parasite_host(w,itemno,itemdescript);
% [str]=parasite_host(w,itemno,itemdescript);
%
% Note: Item must be a single number. This is the item number
% in the w matrix for the item we want to look at competitor 
% relationships with
%

% Parameters
% ----------

if nargin<3,
  itemdescript = ones(size(w,1),1).*32;
end;
if isempty(itemdescript),
  itemdescript = ones(size(w,1),1).*32;
end;


% Extract my strategy and other item's strategies on me
% -----------------------------------------------------

% Problem: This only picks up a small number of the interactions, 
% the ones which were predictive in the model. If I use the 
% correlation matrix, I should get a better list of competitor
% and cooperator items

itemB_strategy = w(:,itemno); % item's effect on other products
itemA_strategy = w(itemno,:)'; % other product's effect on item

competitor     = find(itemB_strategy>0  & itemA_strategy>0);
parasite       = find(itemB_strategy>=0 & itemA_strategy<0);
sucker         = find(itemB_strategy<0  & itemA_strategy>=0);
cooperator     = find(itemB_strategy<0  & itemA_strategy<0);
validrelation  = find(itemB_strategy~=0 | itemA_strategy~=0);

if ~isempty(competitor),
  str(competitor,1:11) = ones(size(competitor,1),1)*double('competitor ');
end;
if ~isempty(parasite),
  str(parasite,1:11)   = ones(size(parasite,1),1)  *double('parasite   ');
end;
if ~isempty(sucker),
  str(sucker,1:11)     = ones(size(sucker,1),1)    *double('sucker     ');
end;
if ~isempty(cooperator),
  str(cooperator,1:11) = ones(size(cooperator,1),1)*double('ally       ');
end;

for i=1:size(validrelation,1),
  fprintf('%d\t%s\t%s\tMy strategy:\t%f\tTheir strategy:\t%f\n', ...
          validrelation(i),...
          itemdescript(validrelation(i),:), ...
          char(str(validrelation(i),:))',...
          itemA_strategy(validrelation(i),1)',...
          itemB_strategy(validrelation(i),1)');
end;

% If we sort in order of relationship(parasite), My strategy, we'll get
% Competitors, the competitors I hurt the most.
% Cooperators, the cooperators I help the most.
% If we sort in order of relationship, Their strategy, we'll get
% Competitors, the competitors who hurt me the most
% Cooperators, the cooperators who help me the most
% Parasites, the worst parasites
% Suckers, the best suckers


% specify item,price,mincorrelation,no_to_report

corr = corrmat2(item(itemno,:)',price');
% Correlation of demand series for itemno with demand series of other products
% If one product's sales go up, what happens to item's sales?
% We have free license to do this with cross-demand series, because
% we're not using this for prediction in any way. We just want to find
% out about the interactions

% Zero out correlations less than the minimum
i=find(abs(corr)<mincorrelation);
corr(i,1) = zeros(size(i,1),1);

% This allows us to designate cooperators or competitors only
% Because the correlation matrix is symmetric, it only shows
% positive correlation between series or negative correlation 
% between series

[a,id] = sort(-1.*corr); % sorts from pos to neg
a=-1.*; % nans are at the bottom

% If their price increases, my sales increase. I cannibalize them.
% If their price drops, my sales decrease. They cannibalize me.

competitors = id(1:no_to_report);
[m,maxdown] = min(find(isnan(a)));
cooperators = id(maxdown-no_to_report+1:maxdown);

% Mutual competitors and cooperators
% ----------------------------------

for i=1:size(competitors,1),
  fprintf('%d\t%s\t%s\tMy strategy:\t%f\tTheir strategy:\t%f\n', ...
          competitors(i),...
          itemdescript(competitors(i),:), ...
          'mutual competitor',...
          corr(competitor(i),1)',...
          corr(competitor(i),1)');
end;
for i=1:size(cooperators,1),
  fprintf('%d\t%s\t%s\tMy strategy:\t%f\tTheir strategy:\t%f\n', ...
          competitors(i),...
          itemdescript(competitors(i),:), ...
          'mutual cooperator',...
          corr(competitor(i),1)',...
          corr(competitor(i),1)');
end;


% Problem is, increasing in unison could be caused by advertising
% together.
% If your sales decrease when the other's increase, it could
% also be because you're not in the same advertising pool as the
% other.
% Cross-price gets around this. If their price drops & your sales
% drop, then that's a real effect. So too if their price drops &
% your sales increase, then that is also probably real...

