% function price_opt_script(w,model);
%
% This script can be copied from the 
%
% Assume that:
% w, model have been generated
% --------------------------------------------------------------
%Cook, S. A. and Mitchell, D. G. ``Finding Hard Instances of the
%Satisfiability Problem: A Survey.'' In Satisfiability problem: theory
%and applications (Piscataway, NJ, 1996). Theoret. Comput. Sci., Vol. 35.
%Providence, RI: Amer. Math. Soc., pp. 1-17, 1997. 

% Prevent positive weights from being used
% make it maximize

% Remove items from the optimization which are below a certain Rsq
% confidence. Removed by removing its model (wmat(:,i) is 0)
%

% Explicitly calculate current revenue
% ------------------------------------

disp('current revenue is');
% we need access to the item timeseries
% actual_rev_lastday = item30(:,size(item30,2))' * pricevect;
actual_rev_lastday = nan;


% Initialize pricevect and upper and lower constraints
% ----------------------------------------------------

pricevect_initial = price30(:,size(price30,2))'; 
pricevect_initial = price30(:,1)';
                   % price on last day of timeseries
cost              = sparse(size(pricevect,1),size(pricevect,2));
critstr           = 'totalrev'; 
                    % Criterion to optimize


upper             = max(price30');
lower             = min(price30');

% The problem here is that we're dealing with the smugged price series
% This means that the prices reported here as uppers and lowers will
% be different from the actual max and min pricepoints

% If you have access to prices, the matrix with pricepoints, replace
% this code with:

upper = max(prices'); % this is expensive
temp=prices;
f=find(temp==0);
temp(f)=ones(size(f,1),1).*inf;
lower = min(temp');
clear temp;



% Eval demand function at present price position
% ----------------------------------------------

% Get model prediction for current rev given current pricevect
% Build the wmat and cons vectors
% Get some idea on how long this takes to run one function eval

disp('\nprobing demand function with price on last day');
t0=clock;

  [critval,dcritval,ddcritval,wmat,cons] = ...
  demand_model_eval(pricevect,model,[],w,critstr,cost);

time_for_1_eval = etime(clock,t0);
fprintf('time for one function eval = %d seconds\n\n',time_for_1_eval);


% Gradient descent
% ----------------

lc            = 0.0001;
secs          = 120;
iterations    = 10000;
max_noimprove = 10;
adaptive      = 0;
linesearch    = 0;
conjugate     = 0;

predicted_rev_lastday = critval;

fprintf(['Actual    %s on last priceday is %f\n' ...
         'Predicted %s on last priceday is %f\n' ...
         'Initiating gradient descent\n'], ...
         critstr, full(actual_rev_lastday), ...
         critstr, full(predicted_rev_lastday));

pricevect = pricevect_initial;
[gradopt_price,gradopt_prof] = ...
  gradient_optimize('demand_model_eval',...
                     pricevect, lower,upper, ...
                     lc,secs,iterations,max_noimprove, ...
                     adaptive,linesearch,conjugate, ...
                     model,[],w,critstr, ...
                     cost, ...
                     wmat,cons); 

fprintf('Grad Descent predicted %s on last priceday is %f\n', ...
        critstr, full(gradopt_prof));
     
% ACTUAL REVENUE
% --------------

sum(item30(1:4800,size(item30,2)).*price30(1:4800,size(item30,2))),

% MULTIPLE RESTART, MULTIPLE METHOD
% ----------------------------------



% We have to find most sig differences in price vector & gradopt_price

% Newton optimization
% -------------------

secs          = 2;
iterations    = 100;
max_noimprove = 10;

pricevect=pricevect_initial;
[newtonopt_price,newtonopt_prof] = newton_optimize('demand_model_eval', ...
                          pricevect, lower,upper,...
                          secs,iterations,max_noimprove, ...
                          model,[],w,critstr,cost,wmat,cons);


% systematic search
% (calc how many combs there are, calc time for one eval, get time for
% testing all possible combs)

% We may want to restart multiple times at different random prices within
% lower and upper

% implement genetic

% most significant differences

% record the sig differences, then average those differences after
% multiple runs to determine reliable direction changes


% Systematic search
% -----------------

% test every possible price combination

% fact about systematic search
% 
% adv: uses actual price points, ie. those pricepoints 
% can carry qualitatively different consumer behaviour, eg. 89c vers
% 87 which grad descent might try


  % Build prices using price points
  % -------------------------------
  % (if we have access to find_price_changes results)
  
  [startprice,  endprice,  startchange,  endchange, ...
   qtybefore,   qtyafter,  stderrbefore, stderrafter, ...
   transactions_before, transactions_after, ...
   prices, qties, transacts, stderr,different_prices, ...
   changelist, firsttrans, lasttrans] =
   find_price_changes(price(:,2:size(price,2)),qty(:,2:size(qty,2)));

  % Build sorted order prices matrix
  % --------------------------------

  % Note that this isn't as good as running the full find_price_changes
  % which also gets demand at each of those points. So if you have this
  % data laying around, don't run this procedure
 
  % prices must be in sorted order for 

% DO we set this up as prices_sorted?

  prices = sparse(size(price,1),size(price,2));
  for i=1:size(price,1),
    % Collapse the item price timeseries into a set of unique values
    newprices = unique(full(price(i,2:size(price,2))));
    % This also sorts newprices into ascending order. 
    % This is used by discrete_price_optimize to know where to lookup

    % Remove 0 from these price points (the 0 gets put in
    % since there are gaps. Its possible there could be a price of 0
    % however
    z = find(newprices~=0);
    newprices = newprices(1,z);

    prices(i,1:size(newprices,2)) = newprices;
    different_prices(i,1)         = size(newprices,2);
    if rem(i,100)==0,
      fprintf('%d/%d ',i,size(price,1));
    end;
  end;
  fprintf('\n');
  % Remove excessive zeros at end of prices
  [a,b] = find(prices~=0);
  prices = prices(:,1:max(b));
  clear a b;

  % Ensure there are no excessive zeros on prices
  % ---------------------------------------------

  [a,b] = find(prices~=0);
  prices = prices(:,1:max(b));

  % Add 10 columns of zeros onto the end to give space for adding  
  % new prices

  prices = [prices zeros(size(prices,1),10)];

  % Estimate time it will take to run systematic search
  % ---------------------------------------------------
  
  % after we have prices matrix

  total_evals = prod(different_prices);
  fprintf(['%d evaluations are required to test every possible ' ...
           'price combination. Multiply this by the time it takes ' ...
           'for 1 function eval (go to demand_model_eval and ' ...
           'test it'],total_evals);

  total_evals .* time_for_1_eval,

  % for just 42 items total evals is 21,666,124,089,262,080  
  % 21 thousand trillion.
  % so don't even run systematic search

gradient on pricepoints
need to capture common up and down and change amount patterns


      
  for i=1:  
  [critval,dcritval,ddcritval,wmat,cons] = ...
  demand_model_eval(pricevect,model,[],w,critstr,cost);
  

% Simps method
% ------------

options(1)  = 1;    % display progress
options(2)  = 0.01; % Relative x-tolerance
options(3)  = 0.01; % Relative f-tolerance
options(14) = 50;   % Max. number of f-evaluations per internal

pricevect = pricevect_initial;
x = simps('demand_model_constr', full(pricevect), ... 
           [1:size(pricevect,2)],options,full(lower),full(upper), ...
           0,lower,upper,model,w,critstr,cost,wmat,cons);

f = demand_model_eval(x,model,[],w,critstr,cost,wmat,cons),


% Nelder-Mead with constraint penalty
% -----------------------------------

options( 1)  = 0;    % no display
options(14)  = 100;  % no of function calls allowed
options( 2)  = 0.01; % x change tolerance (1c in price change)
options( 3)  = 0.01; % y change tolerance (1c in profit)

pricevect = pricevect_initial;
x = fmins('demand_model_constr',pricevect,...
           options,[],1,lower,upper,model,w,critstr,cost,wmat,cons);

f = demand_model_eval(x,model,[],w,critstr,cost,wmat,cons),


% Single item optimize (on smugged demand functions)
% -------------------------------------------------

  [price_prof,price_rev,price_profunc,price_revunc] = ...
   single_item_optimize(pricevect,cost,wmat,cons,lower,upper);

  % test this with constraints implemented as set to higher or lower,
  % or constraints implemented as don't touch

% Single item optimize (on pricepoint induced demand functions)
% -------------------------------------------------------------
  % Assume qties, prices, different_prices

  % create the demand functions using pricepoint data

  fprintf('creating demand functions from pricepoint data for each item\n');
  for i=1:size(price,1),
    ww=polyfitf(prices(i,1:different_prices(i,1)), ...
               qties(i,1:different_prices(i,1)), 1);
    cons_pricepoint(1,i) = ww(1,1);
    w_pricepoint(i,1) = ww(2,1);
    if rem(i,100)==0, fprintf('%d/%d ',i,size(price,1));
  end;
  fprintf('\n');
  % wmat is diag(w)

  % Get the optimum for each function

  pricevect = pricevect_initial;
  [price_prof,price_rev,price_profunc,price_revunc] = ...
    single_item_optimize(pricevect,cost, ...
      diag(w_pricepoint),cons_pricepoint,lower,upper);

% Selective pricing (using pricepoint data)
% -----------------------------------------

  revs          = prices.*qties;
  profs         = revs - costs.*prices;
  profid        = max(profs')';     % find the best prof
  price_prof    = prices(:,profid); % list off prices at that pos
  revid         = max(revs')';      % find the best rev
  price_rev     = prices(:,revid);  % list off prices at that pos

% NOTE we have a conflict between this prices and the prices
% used in the discrete_price_optimize
%

  

% ================================================================
% NOTES

  
  % The following code works

  % Force prices to be in sorted ascending order
  % --------------------------------------------
  
  % This may conflict with price point data: Therefore, keep this prices
  % variable separate
  % The following operation is slow.
  
  for i=1:size(prices,1),
    newprice = unique(full(prices(i,:))); % this sorts it ascending
    nz = find(newprices~=0);
    prices(i,1:size(nz,2))=newprice;
  end;


  % Build different_prices vector for each prices entry
  % ----------------------------------------------------

  % Find last element for each prices table
  rows             = [1:size(prices,1)]';
  different_prices = zeros(size(prices,1),1);

  for i=1:size(prices,2),
    lastel  = rows(find(prices(rows,i)==0),1);
    if ~isempty(lastel),
      different_prices(lastel,1) = ones(size(lastel,1),1).*(i-1);
    end;
    rows = rows(find(prices(rows,i)~=0),1);
  end;



