function [newdata,vals,count,mapping]=recode(data,vals,missingVal);
% [newdata,vals,count,mapping]=recode(data,vals,missingVal);
%
% Converts string data to numeric codes
% If vals is supplied, it uses this as the list of values, otherwise it takes them 
% from the data
% 
% EXAMPLE
% [custid,custvals,custcount]=recode(cust);
% % custid is a sequential 1..n number recoding for cust
% % custvals is a lookup from custid into the true custval key
% % custcount is a count for each custval, this is number of items bought (ommitting doubles)
%
% SEE ALSO
% [newdata,vals,count,mapping] = recode(data,KnownValues,missingVal);
% [newdata,vals,count,mapping] = recode(data);

if nargin<2,
  vals=unique(data,'rows');
  % vals should be in sorted order
end;
if nargin<3, 
  missingVal = -9999;
end;

newdata = zeros(size(data,1),1);
remaining = [1:size(data,1)]';
for i=1:size(vals,1),
   jj=strmatch(vals(i,:),data(remaining,:));            % find(data==vals(i,:));
   newdata(remaining(jj),1) = ones(size(jj,1),1).*i;
   count(i,1) = size(jj,1);
   remaining = remove_ids(remaining,jj);
   if rem(i,100)==0,fprintf('%d/%d ',i,size(vals,1));end;
end;

mapping=[1:size(vals,1)]';

leftover = size(data,1) - sum(count);
if leftover>0, 
  fprintf(['\n **** recode WARNING: %d / %d rows could not be converted to codes.\n ' ...
           'Its likely that the data you generated\n ' ...
           'your unique values with is different to the data you are ' ...
           'now using.\n You''ll be able to spot the values that failed\n ' ...
           'to be recoded because they will be equal to missingVal = %d.\n ' ... 
           'Successfully coded codes begin at 1.\n'],leftover,size(data,1),missingVal);
  i = find(newdata==0);
  newdata(i) = ones(size(i)).*missingVal;
end;