function s=time_to_next_spike(series);
%
% s=time_to_next_spike(series);
%
% recode a timeseries to a second timeseries where the 
% series gives the time until the next spike. An imaginary series
% component is also returned which is the height of the spike.
% 
% This routine is designed for spikey timeseries
% where there are a small number of separated spikes, and the task
% involves predicting the next spike and level
%


for i=1:size(series,1),

  % find all spikes
  spike=sort(find(series(i,:)>0)')'; % is this a down vector?

  for spikeID=1:size(spike,1)-1,
    left  = spike(spikeID);
    right = spike(spikeID+1);
    halfway = spike(spikeID)+floor((right-left)./2);
    s(i,left:halfway)    = 0:halfway-left;
    s(i,halfway+1:right) = right-halfway-1:-1:0;
  end;
  s(i,spike(size(spike,1)):size(series,2)) = ...
    0:size(series,2)-spike(size(spike,1));
  s(i,1:spike(1)) = spike(1)-1:-1:0;


% the old way
% -----------

if 0,
  for t=1:size(series,2),
   tseries = t.*ones(size(spike,1),1);
   [timetospike,nearest_spike] = min(abs(tseries-spike));
   heightofspike = series(i,spike(nearest_spike));
   s(i,t) = timetospike + heightofspike.*j;
  end; % for
end; % if 0

end;
    
    

    


