function series = time_to_spike2_series(time_to_spike);
%
% series = time_to_spike2_series(time_to_spike);
%
% Converts a time_to_spike series back to an ordinary timeseries
% Note that the new series returned has a total energy (that is,
% total sum of series) equal to 1. 
% If you know the original spikey series, then you can convert
% the series you get back and put it on the original scale (that
% is, ensure that the total spikes are equal to the total qty
% forecasted in the original series). Do this by multiplying the
% series by the sum(original_series). eg.
%
% EXAMPLE
%
% % Convert timeseries to time_to_spike series, and back again
%
% % create sparse spikey timeseries
% series = rand(1,400); i=find(series>0.1);
% series(1,i)=zeros(1,size(i,2)); 
% series = round(series .*100);
% figure(1);clf;plot(series);
%
% % convert to time_to_spike series
% spseries2 = real(time_to_next_spike(series));
% figure(2);clf;plot(spseries2);
%
% % convert the timetospike series back to a normal series
% newseries = time_to_spike2_series(spseries2);
% figure(3);clf;plot(newseries);
% title(['unscaled - sum = unity (' num2str(sum(newseries)) ')']);
%
% % scale back to original series' energy (total qty in series)
% newseries = newseries .* sum(item(1,2:size(item,2)));
% figure(4);clf;plot(newseries);
% title(['scaled - sum = ' num2str(sum(newseries)) ...
%        ' original sum = ' num2str(sum(series)) ]);
%
% also try: create_dependent on spikey series to create a
% continuous stepped series. Run arma on that
% 

% Assume original series has a uniform size of 1 total
% If we knew the original series energy (total sum), we can 
% scale all results we get out of this by that factor.

time_to_spike = 1./(time_to_spike+1);
alpha = 1 ./ sum(time_to_spike')'; % unity divided by the sum of the energy
% in the time_to_spike series


% now scale all time_to_spikes by alpha. This will result in the desired
% sum equalling 1.

series = time_to_spike .* (alpha * ones(1,size(time_to_spike,2)));


