function yest = blend_prototypes(weight,y,func,varargin);
% 
% yest = blend_prototypes(weight,y,func,varargin);
%
% performs a weighted average of y in order to return a value yest
% The weighted average is the weight./sum(weight)
% We also provide the ability to pass a function to the weighted average, such that 
% after the weighting is performed, it then warps the result. This can sharpen the 
% weighted average. For instance, squaring the result will cause greater weight to the 
% highest weights. Logarithm will cause less weight to the highest weights.
% 
% EXAMPLE
%
% yest = blend_prototypes(w,y,'power',2);        % Squares the weights before calculating the weighted average
% yest = blend_prototypes(w,y,'exp');            % Exponentiates the weights before calculating the weighted average
% yest = blend_prototypes(w,y,'identity');       % Unmodified weighted average
% yest = blend_prototypes(w,y,'dirac_nearest');  % Classic nearest neighbour
%
% blend_prototypes([1 2 3 4 5]',[2 2 4 4 1]','log'),
% blend_prototypes([1 2 3 4 5]',[2 2 4 4 1]','identity'),
% blend_prototypes([1 2 3 4 5]',[2 2 4 4 1]','power',4),
% blend_prototypes([1 2 3 4 5]',[2 2 4 4 1]','power',40),
% blend_prototypes([1 2 3 4 5]',[2 2 4 4 1]','dirac_nearest'),
% 

  wnew = feval(func,weight,varargin{:});
  sumw = sum(wnew);
  if sumw==0,
    wnew = zeros(size(wnew));
  else
    wnew = wnew ./ sumw;
  end;
  yest = sum(y .* [wnew*ones(1,size(y,2))]);
  
return;
  