function [dist,numwords1,numwords2] = edit_distance(str1,str2,case_sensitive,perm)
% 
% [dist,numwords1,numwords2] = edit_distance(str1,str2,case_sensitive,allpermutations)
%
% case_sensitive==1 means D and d are different. This is optional; default is not case sensitive
% allpermuations==1 means test all permutations between str1 and str2. Default is test all permutations
% numwords1 is the number of collocated words that comprise str1
% 
% SEE ALSO 
%
% sequential_edit_distance
% tokenize
%

if nargin<3,case_sensitive = 0;end;
if nargin<4,perm=1;end;

delim = ' ';

[str1,removed_nonasc1] = tokenize(str1,delim); % breaks collocations into multiple words
[str2,removed_nonasc2] = tokenize(str2,delim);

%if removed_nonasc1,
% If we're doing exact or phrase (sequential) matches, we shouldn't perform this operation

str1=sort(str1);
str2=sort(str2);

numwords1 = length(str1);
numwords2 = length(str2);
min_numwords = min([numwords1 numwords2]);
max_numwords = max([numwords1 numwords2]);
if numwords1==min_numwords,
  smaller=str1;
  larger=str2;
else
  smaller=str2;
  larger=str1;
end;

if 0,
  maxprime  = 100;
  primelist = [];
  while length(primelist)<max_numwords,
    maxprime = maxprime.*2;
    primelist = primes(maxprime);
  end;
  primelist = primelist(1:max_numwords);
  primelist = 2.*ones(size(primelist));
else
  primelist = 2.*ones(1,max_numwords);
end;

if perm==1 & max_numwords>1,
  if 0,
    combination = fullfact(max_numwords.*ones(1,max_numwords)); % An exponential number of combinations. max_numwords in first place, * max_numwords in second, and so on
    products = prod([[ones(size(combination,1),1) * primelist].^combination]')';
    product_we_want = prod(primelist.^[1:max_numwords]);
    matches = find(products==product_we_want);
    combination = combination(matches,:); % Only care about those combinations
    % We need a better way to cut these combinations
  else
    combination = permutate(max_numwords);
  end;
else
  combination = [1:max_numwords];
end;

bestcomb=1;
maxdist = -inf;
for comb=1:size(combination,1),
  dist = sequential_edit_distance(smaller,{larger{[combination(comb,:)]}});
  if dist>maxdist,
    maxdist=dist;
    bestcomb = comb;
  end;
end;

% Return this edit distance

dist = maxdist;

