function [dist,number_of_words] = editdist2(keywordstr1,keywordstr2,case_sensitive,allperms,sparsemode);
%
% [dist,number_of_words] = editdist2(keywordstr1,keywordstr2,case_sensitive,allperms);
% 
% Creates a distance matrix dist where dist(a,b) represents the edit distance between 
% word a and word b. If allperms is being used, then this matrix is symmetric
% number_of_words is the number of collocated words in each {} collocation and is a vector of size keywordstr1
% 
% SEE ALSO
%
% edit_distance	 The edit distance between two {} collocations, where we test all permutations of words
% sequential_edit_distance  The edit distance between two {} collocations, where we use the original sequential order 
% 

if nargin<3,case_sensitive = 0;end;
if nargin<4,allperms = 1;end;
if nargin<5,sparsemode=0;end;

if ~sparsemode,
  dist = zeros(length(keywordstr1),length(keywordstr2));
end;

number_of_words = zeros(length(keywordstr1),1);

for i1=1:length(keywordstr1),
  for i2=1:length(keywordstr2),

    if i1==i2, 
      continue; 
      % We know the edit distance will be 1.0
    end;    

    if allperms & i1<=i2, 
      % no need to perform the lower triangular
      continue; 
    end;
          
    [dist(i1,i2),number_of_words(i1),number_of_words(i2)] = edit_distance(keywordstr1{i1},keywordstr2{i2},case_sensitive,allperms);
    % Otherwise, go ahead and calculate the edit distance between these two words
    % The second number_of_words is needed to fill in a cell that would otherwise not be filled in 
    % because of our tests above that prevent us from doing extra work
  end;
  fprintf('%d/%d ',i1,length(keywordstr1));
end;

if sparsemode,
  fprintf('NOT IMPLEMENTED YET\n');
  if allperms==1,
  end;
  
else

  if allperms==1, 
    % We skipped the lower triangular. We now need to fill that in.
    dist = dist + dist';
  end;

  dist = dist + eye(size(dist)); % Add 1s along the diagonal
end;

return;