function [arm,m2,dep,                     ...
          biasa,  biasm,                  ...
          corra,  perra,  corrm,  perrm,  ...
          corra2, perra2, corrm2, perrm2, ...
          corrs,  perrs,  corrs2, perrs2] = compare_methods(series, ...
                                                           training_window, ...
                                                           initrange, ...
                                                           delay, ...
                                                           window, ...
                                                           days, ...
                                                           wivect );
%
% [arm,m2,dep,                             ... 
%          biasa,  biasm,                  ...
%          corra,  perra,  corrm,  perrm,  ...
%          corra2, perra2, corrm2, perrm2, ...
%          corrs,  perrs,  corrs2, perrs2] ...
%  = compare_methods(series,               ...
%                    training_window,      ...
%                    initrange, ...
%                    delay, ...
%                    window, ...
%                    days );
%

%system_dependent(7);
maxdate = size(series,2);

% Moving average comparison
% -------------------------

m1 = movingavg(filter_outliers(series,4),window,delay); 
% 7 day moving average, no filtering

%m1 = movingavgweek2(filter_outliers(series,4),window,delay);
% 2 day moving average, filtered

%m1 = movingavgweek(series,window,delay);
%m1 = filter_outliers(m1,4);
% tests filtering on the moving average (forecasts), not the actuals

% m1 = mean(series')' * ones(1,size(series,2)); 
% if we want to test the mean
% problem: this mean takes in the zeros at the beginning of the series

m2 = create_dependent(m1,delay); % sums up that average
% this also shifts the moving average backwards in time.
m2 = [zeros(size(m2,1),delay) m2(:,1:size(m2,2)-delay) ];
% m2 gets shifted back to original position


dep = create_dependent(series,delay); 
% creates 7 day sums. This is the correct actual level

seasonal1 = filter_outliers(series,2);
seasonal2 = create_dependent(seasonal1,delay);
% sums up into 7 day total quantities
% no need to shift it forwards in time, since we have the seasonal
% levels


% create day filter, this is a 1 (smoothly falling off) separated by 6
% days, with zeros. To get different day, add 0 to start then reapply
% to series. This gets the correlation between day filter and series.
dayfilter = zeros(1,size(series,2)+7); % the 7 at the end is safety padding
dayfilter(1,3:7:size(series,2)+3) = 1.0 .* ones(1,floor((1+size(series,2))./7)+1);
% convolve with gaussians to get gaussian off-center
dayfilter(1,2:7:size(series,2)+2) = 0.008 .* ones(1,floor((1+size(series,2))./7)+1);
dayfilter(1,1:7:size(series,2)+1) = 0.002 .* ones(1,floor((1+size(series,2))./7)+1);
dayfilter(1,4:7:size(series,2)+4) = 0.008 .* ones(1,floor((1+size(series,2))./7)+1);
dayfilter(1,5:7:size(series,2)+5) = 0.002 .* ones(1,floor((1+size(series,2))./7)+1);

% Loop over each series
% ---------------------

for i=1:size(dep,1),

% default to moving average if forecast zero (ie default to recent 
% history if the total over the next 7 days is zero (could be unusual,
% go back to recent activity))
%[a,b] = find(seasonal2(i,1:range+1)==0);
%seasonal2(ones(size(b)).*i,b) = m2(ones(size(b)).*i,b+365);

irange = initrange(1,1)-delay-days : initrange(1,size(initrange,2))+delay;
%arma = dispar(series(i,:),training_window,irange,days,delay);

DAILY_MULTIMODEL               = 1;
DAILY_SINGLEMODEL              = 2;
DAILY_FIXEDWEIGHTS             = 3;
ACCUMULATED_WEEKLY_SINGLEMODEL = 4;
DIFFERENCED_SINGLEDAY          = 5;

ARMASTDEVS = 4;                        % used for filtering outliers

% bias correction is in tunear

MODEL = DAILY_FIXEDWEIGHTS;

if MODEL==DIFFERENCED_SINGLEDAY,
  % predict differenced series at a lag of 7

  filteredseries = filter_outliers(series(i,:),ARMASTDEVS);
  [start,diff] = difference(filteredseries,delay);
  % filtering should be as aggressive as in usual moving average

  diff = [start diff];
  % diff drops 7 days, but start replaces it. Start from 0 0 0 start
  
  armadiff = dispar(diff, ...  
                    training_window,irange,days,delay);

  % problem is we need a new model for tunear. Instead of counting
  % back to the first double zero block, it could be common for
  % no change in level.

  % armadiff should have given the difference prediction at each point
  % in the test range. Therefore, just add the actual starting values 
  % for the test range, and this should give us the actual predicted 
  % level.
  arma(1,1:delay) = zeros(1,delay);
  arma(1,delay+1:size(series,2)) = ...
                    filteredseries(1,1:size(series,2)-delay) + ...
                    armadiff(1,1:size(series,2)-delay);

  arm   = create_dependent(arma,delay);
end;

if MODEL==ACCUMULATED_WEEKLY_SINGLEMODEL, 
  % fit to dependent
  arma = dispar(filter_outliers(dep(i,1:size(dep,2)-7),ARMASTDEVS), ...
                training_window,irange,days,delay);
  arma = [arma zeros(1,7)];
  arm = arma;
end;

if MODEL==DAILY_MULTIMODEL,
  % model for each days, 1..7, then sum it to get the accumulated total
  
  DEPENDENT_MA = 5;
  for future=1:delay,
    armf(future,:) = tunear(filter_outliers(series(i,:),ARMASTDEVS), ...
                     training_window,irange(1,1)-DEPENDENT_MA+1: irange(1,size(irange,2)),days,future-1);
    % need larger range because we are taking mean of previous 6 values
    % of the forecasts. Therefore we need these previous 6
  end;
  arma = armf(1,:); % pick 7 days in the future as the day model
  arm_dep = sum(armf(1:delay,:)); % for each day, sum days 1..7 (the predictions
  % for this day at this lag in the future. This is the total for that
  % day
  
  % we want to remove the high frequency components now (found that high
  % up and downs reduce error). Want a line straight through those. Do this
  % by elimination of high freqs. Have to do the fft operation at each point,
  % and use last.
  
  if 0, % no smoothing of final 7day sum
     arm = arm_dep;
  else % smooth it
    arm = zeros(size(arm_dep));
    if 0, % use lowpass filter to remove highfrequency spikes
      for iii=5:size(arm_dep,2),
         if sum(arm_dep(1,1:iii))>0,
           l = lowpass(arm_dep(1,1:iii),0.9);
           %bias = (sum(l(1,1:iii)') ./ sum(arm_dep(1,1:iii)')); 
           arm(1,iii) = l(1,iii); % ./ bias;
           % here we want to normalize the first fourier component
           % so that series is centered
        end;
      end;
    else % use moving average to remove high frequency spikes
      for iii=DEPENDENT_MA:size(arm_dep,2),
        %arm(1,iii) = 0.5 * arm_dep(1,iii) + 0.5 * mean(arm_dep(1,iii-6:iii-1)');
        arm(1,iii) = mean(arm_dep(1,iii-DEPENDENT_MA+1:iii)');  
      end;
    end;
  end;

  %keyboard
end;

if MODEL==DAILY_SINGLEMODEL,
  % 7-day model arma, gets summed into the prediction over days 1..7
  arma = tunear(series(i,:), ...
                training_window,irange,days,delay);
  arm   = create_dependent(arma,delay);
end;

if MODEL==DAILY_FIXEDWEIGHTS,
  arma = dispar(filter_outliers(series(i,:),ARMASTDEVS), ...
                training_window,irange,days,delay);
  arm   = create_dependent(arma,delay);
end;

if find(arma),
   %arma = 0.5 .* arma +  0.5 .* m1(i,:);
   %arm  = 0.5 .* arm  +  0.5 .* m2(i,:);
   % arm   = [zeros(1,delay) arm(1,1:size(arm,2)-delay)];
   
   % use m1 for the level, and arma for the difference.
   % note that the tunear routine which gets called must not retrend
   % the arma series. arma should be fitted on detrended / stationary
   % series, and the result then not retrended. m1 then comes in with
   % the level (trend)
   
   [start,diff] = difference(arma,1);
   arma(1,initrange) = m1(i,initrange) + diff(1,initrange); % initrange(1,1)-1:initrange(1,size(initrange,2))-1);
   
   [start,diff] = difference(arm,1);
   arm(1,initrange) = m2(i,initrange) + diff(1,initrange); % initrange(1,1)-1:initrange(1,size(initrange,2))-1);
   
d   = dep(i,initrange);
m   = m2(i,initrange);
% s   = seasonal2(i,initrange+1-365); % 1:seasonal_range+1);

figure(2);clf;

subplot(2,1,1);
plot(initrange,d,'r-');hold on;
plot(initrange,m,'g-');
plot(initrange,arm(1,initrange));
%plot(initrange,s,'c-');
if nargin==7,
  title(['wi ' num2str(wivect(i,1),8) ' ' num2str(wivect(i,2),8) ...
         ' timeseries ' num2str(i)]);
  disp(['wi '  num2str(wivect(i,1),8) ' ' num2str(wivect(i,2),8)]);
else
  title(['timeseries ' num2str(i)]);
end;

[corrm(i,:),perrm(i,:),totm(i,:)] = display_accuracy(m,d,0);
[corra(i,:),perra(i,:),tota(i,:)] = display_accuracy(arm(1,initrange),d,0);
%[corrs,perrs,tots] = display_accuracy(s,d,0);

st1 = sprintf('\n%d day accuracy\narma      \t %5.2f\t %5.2f%%\nmovingavg \t %5.2f\t %5.2f%%\n', ...
   delay, corra(i,:),perra(i,:),corrm(i,:),perrm(i,:));
%disp(st1);

%disp(['seasonal  ' num2str(corrs(i,:)) sprintf('\t') num2str(perrs(i,:))]); % sprintf('\t') num2str(tots)]);


% day by day
% ----------

d = series(i,initrange);
m = m1(i,initrange);
%s = seasonal1(i,initrange+1-365);

subplot(2,1,2);

% Day by day forecast versus actual
% ---------------------------------

plot(initrange,d,'r-');hold on;
plot(initrange,m,'g-');
plot(initrange,arma(1,initrange));
% plot(initrange,s,'c-');
drawnow;

[corrm2(i,:),perrm2(i,:),totm2(i,:)] = display_accuracy(m,d,0);
[corra2(i,:),perra2(i,:),tota2(i,:)] = display_accuracy(arma(1,initrange),d,0);
%[corrs2(i,:),perrs2(i,:),tots2(i,:)] = display_accuracy(s,d,0);

% bias only relevant on the day by day level
biasa(i,:) = 100 .* 100.* ( ( sum(arma(1,initrange)') - sum(d') ) ./ sum(d')) ./ size(d,2);
biasm(i,:) = 100 .* 100.* ( (sum(m') - sum(d')) ./ sum(d') ) ./ size(d,2);
           % 100 days * 100% of total diff between d and m, expressed as a
           % percentage of d, taking this as an average % error each day  
           
           disp([st1]);
disp('daily accuracy');
disp(['arma       ' num2str(corra2(i,:)) ' ' num2str(perra2(i,:)) '% ' num2str(biasa(i,:)) '% per 100 days']);
disp(['movingavg  ' num2str(corrm2(i,:)) ' ' num2str(perrm2(i,:)) '% ' num2str(biasm(i,:)) '% per 100 days']);

  
if 0,
% display on graph
figure(3);clf;
bar(1,perrm(i,:),'g-');hold on;
bar(2,perra(i,:));
bar(4,biasm,'g-');
bar(3,biasa);
set(gca,'XTick',[1:4]);
set(gca,'XTickLabel',['merr';'aerr';'acrp';'mcrp']);
line([0.5 4.5],[0 0]);
ylabel('%');
axis([0.5 4.5 -100 100]);
title('aggreg stock');

figure(4);clf;
bar(1,corrm2(i,:),'g-');hold on;
bar(2,corra2(i,:));
set(gca,'XTick',[1:2]);
set(gca,'XTickLabel',['mcor';'acor']);
ylabel('%');
axis([0.5 2.5 -1 1]);
title('turn correlation');
% keyboard
% sprintf('\t') num2str(totm2(i,:))]);
%disp(['seasonal  ' num2str(corrs2(i,:)) sprintf('\t') num2str(perrs2(i,:)) sprintf('\t') num2str(tots2(i,:))]);
% disp(['bias after ' num2str(initrange(1,size(initrange,2))-initrange(1,1)) ' days: ' sprintf('\t') num2str(
end;

% Day Of Week
% -----------

% calculate correlation with day of week
dayfilter_apply = dayfilter;
maxdayval = inf;maxday = -1;
dd = d-mean(d'); % d./(d+eps);
for i=1:7,
% shift day by 1 each time
  dayfilter_apply = [ 0 dayfilter_apply ];
  daycorr(i,1) = (dd*dayfilter_apply(1,1:size(d,2))') ./ ...
                 ( norm(dd) * norm(dayfilter_apply(1,1:size(d,2))) );
  if daycorr(i,1) > maxdayval, 
    maxdayval = daycorr(i,1);
    maxday = i;
  end;
end;

if 0,
prtstr = ['dayofweek '];
for i=1:7,
  prtstr = [ prtstr ' ' num2str(daycorr(i,1),2) ];
end;
disp([prtstr]);
end;

figure(3);clf;
bar(1:7,daycorr(1:7,1)');
title('day of week');
axis([1 7 -1 1]);
% recomment these for matlab5
%set(gca,'XTick',[1:7]);
%set(gca,'XTickLabel',['M';'T';'W';'T';'F';'S';'S']);

if size(dep,1)>1, 
  %pause, 
end;

end; % if arma2 is empty

end;

