function [levelseries,time_since_lastspike] = create_levelseries(series,number_to_fill);
%
% [levelseries,timeseries] = create_levelseries(series,number_to_fill);
%
% A levelseries is a timeseries of levels which occurred in customer spending, regardless
% of how long ago. We can also retrieve time since last spike.
%
% For example, 
% [levelseries,time_since_lastspike]=create_levelseries([1 0 0 0 5 4 0 2 0],4);
% levelseries          == [1 5 4 2]
% time_since_lastspike == [1 2 1 4]
%
% The problem with this representation is customers (series) who have fewer than 
% number_to_fill spikes in their series. When this happens levelseries gets 
% zero-filled at the end, eg. levelseries == [0 0 4 2];
% 
% The problem is performing predictions from these zero-entries. We could filter 
% customers so that only customers with > number_to_fill are looked at using this
% method.
%
% We could also go ahead with prediction using the zero-fills. In linear regression,
% this could result in the zeroed-values not changing the constant
% term, so you would think the predictions would be poor. You could also perform a
% constant-less regression.
%

  %spike=find(series>0); % is this a down vector?
  %[a,b]=ind2sub(size(series,1),spike);
  
  time_since_lastspike = zeros(size(series,1),number_to_fill);
  levelseries          = zeros(size(series,1),number_to_fill);
  
  % from very end of series, count back.
  
  
  finishtime  = size(series,2).*ones(size(series,1),1);
  % the time futuren time that a spike occurred. When we count backwards and find a 
  % new spike, we subtract finishtime-day to find out the number of days between
  % spike
  nonfilled   = [1:size(series,1)];
  % customers who have yet to have filled all levelseries positions
  level_pos   = ones(size(series,1),1) .* number_to_fill;
  % current levelseries pos that we are up to (we count backwards from number_to_fill)
  
  for day = size(series,2):-1:1,
     % find all spikes which occurred on this day
     spike = find(series(nonfilled,day)~=0);
     % for these customers, reset their finishtime to this day, and look for a 
     % new spike prior to this.
%     keyboard;
     for cust=1:size(spike,1),
       % I wish I wish there was a way not to do this for loop
       levelseries(nonfilled(spike(cust)),level_pos(nonfilled(spike(cust)))) = series(nonfilled(spike(cust)),day); 
       time_since_lastspike(nonfilled(spike(cust)),level_pos(nonfilled(spike(cust)))) = finishtime(nonfilled(spike(cust)),1)-day;     
     end;
     finishtime(nonfilled(spike),1) = day.*ones(size(spike,1),1);
     % reset the finish time for these customers, so we are counting back from this new date
     level_pos(nonfilled(spike),1) = level_pos(nonfilled(spike),1)-1;
     % we will next put entries in at one position to the left for these customers
     
     nonfilled = remove_ids(nonfilled,spike( find(level_pos(nonfilled(spike))<1)));
     %nonfilled = setdiff(nonfilled,nonfilled(spike( find(level_pos(nonfilled(spike))<1))));
     % find all customers whose level_pos is now zero
     

     %fprintf('%d ',day);
  end; % next day
  
  
