function [inputs,targets,str] = create_multiple_timeseries(series, train, days, delay, skipdays, targetdays, targetseries_id,varstr);
% [inputs,targets] = create_multiple_timeseries(series, train, days, delay, skipdays, targetdays, targetseries_id,varstr);
%
% train = size of training period (1..train will be used), days == days_in_model, delay == minlag, 
% targetdays == number of days to predict 
% 

end_of_patterns = 1+train-(days+delay+targetdays);
slidedown       = [1:skipdays:end_of_patterns];
start_across    = end_of_patterns; % toeplitz is square
targetrange     = start_across+days+delay:start_across+days+delay+targetdays-1;
% end of usable patterns in toeplitz
% rightmost extent of the toeplitz

inputs  = zeros(size(slidedown,2),days.*size(series,1));
targets = zeros(size(slidedown,2),targetdays.*size(targetseries_id,2));

ti = 0;
for i=1:size(series,1),
% these series line up horizontally. Going down we have the sliding
% components
% these series are supposed to be different timeseries for the same customer
% outside we would have a loop which constructs them one customer at a time, and adds
% to a large array

  sliding_series = toeplitz(series(i,1:train));
  % bulk of time spent here, unfortunately
  % see if we can speed this up? 0.88s
  
  % Add input series
  inputs( [1:size(slidedown,2)],(i-1).*days+1:i.*days) = ...
     sliding_series(slidedown,start_across:start_across+days-1);
  % targets(1:end_of_patterns,(i-1).*targetdays+1:i.*targetdays) = sliding_series(1:end_of_patterns,targetrange);
  % if we just wanted the targets to be for each input timeseries. But because we may want the
  % targets to be from just a single series, then we have to have a special counter for 
  % target series
  inputsmember(  1,(i-1).*days+1:i.*days) = i.*ones(1,days);
  if nargin>7,
  str((i-1).*days+1:i.*days,1:size(varstr,2)+5) = [char(ones(days,1)*varstr(i,:)) num2str([1:days]','%05d')];
  end;
  % Add target series
  if ~isempty(targetseries_id),
    if ismember(i,targetseries_id),
       ti=ti+1;
       targets([1:size(slidedown,2)],(ti-1).*targetdays+1:ti.*targetdays) = ...
          sliding_series(slidedown,targetrange);
       targetsmember( 1,(ti-1).*targetdays+1:ti.*targetdays) = i.*ones(1,targetdays);
    end; % ismember(i,targetseris_id)
  end; % isempty(targetseries_id)
  
end; % i