function [trainseries,traintarget,testseries,testtarget,startday,trainstart,trainend,valid,newcusts] = create_traintest_series(rev,trainwindow,targetwindow,delay,sep,include_startday,startday,varargin);
% [ trainseries, traintarget, testseries, testtarget, ...
%   startday,    trainstart,  trainend,   valid,      newcusts] = ...
%   create_traintest_series(rev,trainwindow,targetwindow,delay,sep, ...
%                           include_startday,varargin);
%
% Turns multiple customer timeseries into training and target series
% a separation variable states how far the test set should be offset into the future
% Additional timeseries relating to a customer (eg. a monthseries [1:12]) can be
% included in the varargin, and these will be dutifully created, also.
%
% This does not perform sliding, but instead creates a single series at a random
% point at which it is possible to create a series.
% If I create a sliding version, this will be general purpose
%
% EXAMPLE
% [    trainseries,traintarget,testseries,testtarget, ...
%      startday,   trainstart, trainend,  valid,      newcusts] = ...
%  create_traintest_series(full(dat),3,1,0,3)

% code these as nans
fprintf('zero padding\n');tic,
rev = [zeros(size(rev,1),sep) rev];
for seriesid=1:size(varargin,2),
   a=varargin{seriesid};
   varargin{seriesid} = [zeros(size(rev,1),sep) a];
end;
toc,

% vectsize   = targetwindow+trainwindow+delay;
if nargin<8, 
   extrasize=0;
else
   extrasize=size(varargin,2);
end;

trainseries = zeros(size(rev,1),trainwindow  +trainwindow  .* extrasize);
traintarget = zeros(size(rev,1),targetwindow);
testseries  = zeros(size(rev,1),trainwindow  +trainwindow  .* extrasize);
testtarget  = zeros(size(rev,1),targetwindow);

% sep_above_traintarget = min([sep targetwindow+1+delay]);
if sep>trainwindow,
   sep_above_traintarget = sep-trainwindow+1;
else
   sep_above_traintarget = 1;
end;
if sep==0,
   sep_above_traintarget = 0;
end;

% man, this logic is complicated

% in other words, the minimum separation is either current separation
% (eg. 0 or 2), or the targetwindow+1 (meaning, targetwindow plus one
% training day). If the user's required separation is higher than targetwindow,
% then a minimum of the initial train targetwindown and one training day
% must be captured (the rest of the training series can be lost)

separation = sep_above_traintarget + trainwindow + targetwindow + delay; 
% there must be at least this 
% to perform a prediction. sep is the offset from the training data which 
% the test data appears at

number_of_customers = size(rev,1);
series_size         = size(rev,2);

% separation means train and test training periods must be shifted by at least
% separation days
% if separation>trainwindow, then the two training periods won't have any elements
% in common

% Find firstspike
% ---------------

findspike=0;
if nargin<7,
   findspike = 1;
end;
if isempty(startday) | size(startday,1)~=size(rev,1) | size(startday,2)~=1,
   fprintf('Supplied startday not acceptable\n');
   findspike=1;
end;
if findspike==1,
   fprintf('Finding each customer''s startday (firstspike)\n');
   startday = find_first_spike(rev);
end;
if findspike==0,
   fprintf('using supplied startday\n');
   startday = startday + sep; % because of the zero padding, we have to shift the startdays
end;

fprintf('Estimate that %d customers will be valid\n',...
   size(find(series_size-startday>(trainwindow+targetwindow+delay+sep)),1));

% WE SHOULD BALANCE THIS SET OF NEW INCOMPLETES. ESTIMATE NUMBER OF NEW INCOMPLETES, A
% estimating that might be difficult

% Report new incompletes in original data (end of series)
% -------------------------------------------------------

i=find(startday>series_size-(trainwindow-delay));
fprintf('\n%d/%d (%5.1f%%) of customers at end of series were new incompletes\n', ... 
   size(i,1),number_of_customers,100.*size(i,1)./number_of_customers);
fprintf('Try to aim for roughly this percentage of new incompletes when you create the training set\n');

% select random test period, from startday..series_size-target_window
% we want train and test sets, so
% startday..series_size-(target_window+delay).*2

maxdayrange = series_size-separation+1; % 1 to this if we allow the startday
% rightmost that the series could possibly be
% only requirement - we must always have this much room available
% to build a vector. We need this to build the two vectors. Less than 
% this and we cannot

% targetwindow+delay.*2 available
%rand_day    = round(rand(number_of_customers,1).*(maxdayrange-1));
% makes it 0..maxdayrange-1
% 0 is startday, now rem it

dayrange = maxdayrange-(startday+1); % the +1 here prevents it from ever
% selecting the customer's startday. the dayrange is always one less than
% the proper dayrange. Now when we select a day range, we'll only count back
% a max of one less than the startday

% number of days between startday and rightmost which are available
% if this is zero, then it should be ok
% our beginning of test set could be on maxdayrange, or maxdayrange-1, etc
% until we reach startday.
% the trainwindow will hang over past startday in time, its filled with zeros.

% only requiring
%dayrange = series_size-(targetwindow+delay+trainwindow).*2 - startday; % only requiring
% two periods of targetwindow+delay. We could add trainwindow, which would
% force there to be enough room to run two trainwindows as well.
% range of days that the pos of trainwindow could fall onto
% relative to customer's time of arrival)

valid = find(dayrange>=0);

rand_day = round(rand(number_of_customers,1).*series_size.*2000);
warning off; % we are already filtering the dayrange==0s out
increment = rem(rand_day,dayrange)+1; % allows it to go back to startday
ii=find(dayrange==0);
increment(ii,1) = zeros(size(ii,1),1);
warning on;

% I can appear 
% this is an offset which we add to startday to give us a random 
% day for the trainwindow, which we are assured will have enough room
% to fill the rest of the targetwindow and delay

vectend = series_size-increment; % this - targetwindow+delay+sep must be >= startday (trainwindow not crucial as new custs)
% target1 delay1 target2 delay2 vectend series_size

if ~isempty(find(vectend(valid)+1-separation<startday(valid))),
   fprintf('didn''t work\n');
end;

% Force training data to be only valid ranges (exclude the cases 
% which could not be used because there was not enough data to create
% a single time period
% ---------------------

rev      = rev(valid,:);
startday = startday(valid,:);
vectend  = vectend(valid,:);
if extrasize>0,
   for seriesid=1:extrasize,
      a=varargin{seriesid};
      varargin{seriesid} = a(valid,:);
   end;
end;

trainseries = trainseries(valid,:);
traintarget = traintarget(valid,:);
testseries  = testseries(valid,:);
testtarget  = testtarget(valid,:);
trainstart  = zeros(max(size(valid)),1);
trainend    = zeros(max(size(valid)),1);

number_of_customers = size(valid,1);

% Remove excess zeros at the bottom of the vector (caused by some of the
% training ranges being impossible to meet)

if isempty(rev),
   fprintf('no training data made it into that setting\n');
   return   
end;

% Report new incompletes in new data
% -----------------------------------

newcusts=find(vectend-trainwindow-delay-targetwindow-sep<startday);
fprintf('\nNew training vector series: %d/%d (%5.1f%%) customers were new incompletes\n',size(newcusts,1),number_of_customers,100.*size(newcusts,1)./number_of_customers);
%fprintf('%d customers valid 


% Allocate timeseries using these ranges
% --------------------------------------

% we now know when to count back from for each timeseries. Now allocate these new timeseries

% for each varargin, rev=varargin{seriesid}
for day=1:series_size,
   i=find(vectend==day);
   % find all customers whose vectors finish on this day
   if ~isempty(i),
      % allocate the series by counting back from here
      trainstart(i,1) = ones(max(size(i)),1) .* (day-trainwindow-delay-targetwindow-sep+1);
      trainend(i,1)   = ones(max(size(i)),1) .* (day-            delay-targetwindow-sep);
      trainseries(i,1:trainwindow)  = rev(i,day-trainwindow-delay-targetwindow-sep+1:day-targetwindow-delay-sep);
      traintarget(i,1:targetwindow) = rev(i,day-targetwindow-sep+1:day-sep);
      testseries(i,1:trainwindow)   = rev(i,day-trainwindow-delay-targetwindow+1:day-targetwindow-delay);
      testtarget(i,1:targetwindow)  = rev(i,day-targetwindow+1:day);
      
      % do this for all series
      for seriesid=1:extrasize,
         a = varargin{seriesid};
         trainseries(i,seriesid.*trainwindow+1:(seriesid+1).*trainwindow)   = a(i,day-trainwindow-delay-targetwindow-sep+1:day-targetwindow-delay-sep);
         %traintarget(i,seriesid.*targetwindow+1:(seriesid+1).*targetwindow) = a(i,day-targetwindow-sep+1-include_startday:day-sep-include_startday);
         testseries( i,seriesid.*trainwindow+1:(seriesid+1).*trainwindow)   = a(i,day-trainwindow-delay-targetwindow+1:day-targetwindow-delay);
         %testtarget( i,seriesid.*targetwindow+1:(seriesid+1).*targetwindow) = a(i,day-targetwindow+1-include_startday:day-include_startday);
      end;         
   end;
end;

% All results were calculated on the padded series. Now correct those results back to the original,
% so that outside, our startday and trainstart, trainend will be correct

startday   = startday   - sep;
trainstart = trainstart - sep;
trainend   = trainend   - sep;
