function [lifecycle,lifecyclevis,custs_on_day,series,seriesvis,custs_on_startday] = customer_lifecycle(rev,startday);
% [lifecycle,lifecyclevis,custs_on_day,series,seriesvis,custs_on_starday] = customer_lifecycle(rev,startday);
%
% Generates the standard profile for customers after arriving (or having their first
% purchase recorded). This is done by matching up all customers on each startday,
% and averaging their resulting purchase series after that date
%
% %interactive, startday moves, period remains the same
% figure(1);clf;for i=1:size(seriesvis,2)-1,figure(1);clf;bar(seriesvis(i,:));title(num2str(custs_on_startday(i)));hold on;v=axis;plot([i i],[v(3) v(4)],':');pause;end
%  
% %interactive, startday stays the same, observe any characteristic increases at later time
% for day=1:size(seriesvis,2)-1,i=find(startday==day);figure(1);clf;bar(sum(rev(i,day+1:size(rev,2)))./size(i,1));v=axis;axis([v(1) size(seriesvis,2) v(3) v(4)]);pause;end
%
% % rev after startday (liftrev)
% avgrev = sum(abs(rev))./(sum(sign(abs(rev)))+eps);figure(1);clf;for i=1:size(seriesrev,2)-1,figure(1);clf;bar(seriesrev(i,:)./avgrev);title(num2str(custs_on_startday(i)));hold on;v=axis;plot([i i],[v(3) v(4)],':');pause;end
% 
% %customer lifecycle following startday
% figure(1);clf;plot([1:size(lifecycle,2)].*4,lifecyclevis./custs_on_day,'b-');
% title('Customer lifecycle following startday');xlabel('week after startday');ylabel('visits per month');
%
% figure(1);clf;plot([1:size(lifecycle,2)],lifecyclevis./custs_on_day,'b-')
% title('Customer lifecycle following startday');xlabel('day after startday');ylabel('visits per day');

options=foptions;
figure(1);clf;for i=1:16,figure(1);
   s = seriesvis(i,:); %./custs_on_startday(i); % not required because we normalized below by number of custs (i)
   clf;bar(s);
   title(num2str(custs_on_startday(i)));hold on;v=axis;plot([i i],[v(3) v(4)],':');
   lambda = fmins('exponential_smooth_error',[0 1]',options,[],[i+1:size(s,2)]',s(1,i+1:size(s,2))');
   [err,fit] = exponential_smooth_error(lambda,[1:0.2:size(series,2)]',[1:0.2:size(series,2)]');
   [err,fit,c] = exponential_smooth_error(lambda,[i+1:size(s,2)]',s(1,i+1:size(s,2))');
   fit = exponential_smooth_eval([i+1:0.1:size(s,2)]',lambda,c);
   plot([i+1:0.1:size(s,2)]',fit,'r-','linewidth',3);
   pause;
end


% what is the latency between multiple buyers

if nargin<2,
  fprintf('finding each customer''s startday (firstspike)\n');
  % remove any custs who didn't start in the series
  i=find(sum(rev')'~=0);
  rev = rev(i,:);
  startday = find_first_spike(rev);
else
  fprintf('using supplied startday vector\n');
end;

[id,vals,count] = recode(startday);
figure(1);clf;bar(vals,count);title('startdays in population');

%groupsize = min(count);
%no_groups = size(vals,1);
%trainseries = zeros(groupsize.*no_groups,trainwindow);
%traintarget = zeros(groupsize.*no_groups,targetwindow);
%testseries  = zeros(groupsize.*no_groups,trainwindow);
%testtarget  = zeros(groupsize.*no_groups,targetwindow);

custs_on_day = zeros(1,size(rev,2));
lifecycle    = zeros(1,size(rev,2));
lifecyclevis = zeros(1,size(rev,2));
% show the average lifecycle (series from startday to end)
% for all customers in population
% custs_on_day provides how many customers were present
% on a particular day of that lifecycle, which went into building
% the rev. Normalize by lifecycle./custs_on_day to get average
% level per customer active

series            = zeros(max(vals),size(rev,2));
seriesvis         = zeros(max(vals),size(rev,2));
seriesprob        = zeros(max(vals),size(rev,2));
seriesrev         = zeros(max(vals),size(rev,2));
custs_on_startday = zeros(max(vals),1);
% show the series for each startday customer group, into the future

bins=10;
visits_per_cust = sum(abs(sign(rev))')' ./ size(rev,2);
[avgvisperweek,avgvisbin] = hist(visits_per_cust,bins); % average visits per day per week
% average visits per week irrespective of startdate (across population)

visdistribution        = zeros(max(vals),bins);
% visperweek is a histogram of the visits per week
% we'll look at this as we go forward in time with new starting customers
% or could we use this for the pop by going forward
% will be affected by months

for day=vals(1:size(vals,1)-1)', % do all except the last one
   vect = [day+1:size(rev,2)]; % one day after the spike, what did they do after that
   i=find(startday==day);
   if max(size(i))>1, % having 1 customer would mess up some of the sums below
     % customers enrolled on this day
     lifecycle(1, 1:size(vect,2)) = lifecycle(1,1:size(vect,2)) + sum(rev(i,vect));
     lifecyclevis(1,1:size(vect,2)) = lifecyclevis(1,1:size(vect,2)) + sum(abs(sign(rev(i,vect))));
     custs_on_day(1,1:size(vect,2)) = custs_on_day(1,1:size(vect,2)) + size(i,1).*ones(1,size(vect,2));
     
     % complete series for each startday group
     series(   day,1:size(rev,2))  = [zeros(1,day) mean(rev(i,vect))]; % note this isn't normalized
     seriesvis(day,1:size(rev,2))  = [zeros(1,day) sum(abs(sign(rev(i,vect))))./size(i,1)];
     % note this is normalized, this is literally a probability
     
     seriesrev(day,1:size(rev,2))  = [zeros(1,day) sum(abs(rev(i,vect)))];
     seriesrev(day,1:size(rev,2))  = seriesrev(day,1:size(rev,2)) ./ ...
        [ones(1,day)  sum(sign(abs(rev(i,vect))))+eps];
     % average revenue when a customer turns up
     
     % we need to do something with with rev here. When customers turn up, how much do
     % they spend. This is sum(rev(i,vect))./sum(sign(rev(i,vect)))
     % if inf then nan
     % ie. in the divisor, we only count times when a customer turned up
     % it is possible that visits of "loyal" customers increase over time (or something)
     
     custs_on_startday(day,1) = size(i,1);
     % could do it without the zeros also, or just plot and only show from startday+1
     
     visdistribution(day,:) = hist(sum(abs(sign(rev(i,vect)))')'./size(vect,2),avgvisbin);
     % distribution of visits per day for customers arriving on this startdate
     % problem, expect later customers will have lower visdistribution. But then its
     % normalized by time, so it should still be reasonably fair.
     % If the distribution increases later, it would indicate a successful marketing program
     % But the visits/custs would show this also. 
  end;
  fprintf('%d/%d ',day,size(vals,1));
end;
fprintf('\n');


if 0,
lastspike = size(custs_on_day)-find_first_spike(fliplr(custs_on_day))+1;

lifecycle    = lifecycle(   1,1:lastspike);
custs_on_day = custs_on_day(1,1:lastspike);
end;

clf;plot([1:size(lifecycle,2)],lifecyclevis./custs_on_day,'b-');
title('Customer lifecycle following startday');
xlabel('month after startday');ylabel('visits per month');
