% Customer retention script
% similar to lifecycle, we should merge them
% Displays acquisition versus defection rates over a period of time
% Also shows distribution changes over a set period of time
% customer_lifecycle shows the entire lifecycle probability change, this 
% takes two snapshots
%
% ii=find(startday>=2 & startday<=28 & sum(sign(abs(rev(:,29:56)))')'>=4); size(ii)% & sum(rev(:,300:size(rev,2))')'==0);size(ii)
% find customers who visit at least .5 times per week (or twice per 4 weeks)
% [decdata,d,cutpoint] = decile1((sum(sign(abs(rev(ii,300:size(rev,2))))')'.*7./112),[1],10,1);
% 10% of population in each group, look at the popular bins


% Parameters
% ----------

startdaylow  = 2;  % find customers between this interval. Now monitor them over time
startdayhigh = 28; % find customers between this interval. Now monitor them over time
range1       = 29:56;   % first period to observe customer visits per week
range2       = 380:407; % second period to observe customer visits per week
baseunit     = 'day';
multiplier   = 7;       % if base unit is days, mult by 7, if base unit is months, no mult
multunit     = 'week';  % 
no_bars      = 30;


% Show distribution now and in the future
% ---------------------------------------

ii = find(startday>=startdaylow & startday<=startdayhigh);

% Future visit behaviour
figure(1);clf;
visits_per_week = sum(sign(abs(rev(ii,range2)))')'.*7./size(range2,2);
[a,x] = hist(visits_per_week,no_bars);
figure(1);clf;shading flat;bar(x,100.*a./sum(a),1,'w');shading flat;grid on;
% equal range, look at the distribution
title(['Same customers ' num2str(min(range2)-max(range1)) ' ' baseunit 's into the future (' baseunit 's=' num2str(size(range2,2)) ', n=' num2str(size(ii,1)) ')']);
xlabel(['Visits per ' multunit ]);
ylabel('Percent of customers');
v=axis;

% Initial visit behaviour
visits_per_week = sum(sign(abs(rev(ii,range1)))')'.*multiplier./size(range1,2);
[a,x] = hist(visits_per_week,no_bars);
figure(2);clf;shading flat;bar(x,100.*a./sum(a),1,'w');shading flat;grid on;
% equal range, look at the distribution
title(['Initial visit behaviour (' baseunit 's=' num2str(size(range2,2)) ', n=' num2str(size(ii,1)) ')']);
xlabel(['Visits per ' multunit ]);
ylabel('Percent of customers');
axis(v);

% Show this as a defection percentage
% -----------------------------------

% Initial period
visits_per_week = sum(sign(abs(rev(ii,range1)))')'.*multiplier./size(range1,2);
initial_zero = size(find(visits_per_week==0),1);
initial_vis  = size(ii,1)-initial_zero;

% Future period
visits_per_week = sum(sign(abs(rev(ii,range2)))')'.*multiplier./size(range2,2);
future_zero = size(find(visits_per_week==0),1);
future_vis  = size(ii,1)-future_zero;

figure(3);clf;h=bar([1 2],round(100.*[[initial_zero initial_vis];[future_zero future_vis]]./size(ii,1)));
set(gca,'xticklabel',['initial';'future ']);
legend('defectors','loyals',0);
ylabel('percent of customers');
title(['Comparison between same customers now and ' num2str(min(range2)-max(range1)) ' ' baseunit 's into the future (' baseunit 's=' num2str(size(range2,2)) ', n=' num2str(size(ii,1)) ')']);
text(0.9,round(100.*initial_zero),num2str(round(100.*num2str(initial_zero))));
text(1.1,round(100.*initial_vis),num2str(round(100.*num2str(initial_vis))));
text(1.9,round(100.*future_zero),num2str(round(100.*num2str(future_zero))));
text(2.1,round(100.*future_vis),num2str(round(100.*num2str(future_vis))));


% Acquisition rate
% -----------------

% We've found a way to estimate defection rates for old customers, independent
% of any factors to do with the sampling time period. The strategy was to find
% customers who had transacted, so we knew were valid customers, and then to 
% look at their defection rate at two sequential periods. The difference in 
% defection rates gave the increase in defection for at least old customers.
%
% Acquisition rate cannot be estimated from the data, without access to the 
% true signup date for the customer. The problem is that how do you distinguish
% between a true new customer, and a customer who just has a long visit cycle, and
% is turning up. As you move forward you will appear to "acquire" more and more
% customers. This is an artifact since these are existing customers, where the
% period is long enough that the probability of their turning up, they end up 
% turning up.
% However, despite this problem, in fact there is a way to estimate acquisition and
% defection rates independent of knowledge of signon or signoff date.
% 
% It in fact can. The artifact which is causing us so much trouble is the fact that
% when we calculate new customers, we are influenced by customers 
%
% 
% problem with just looking at the growth in startdays since range1, is that in
% grocery (and elsewhere) we are simply taking customers from a database.
% The startday is their first transaction we have recorded in the period.
% However, for early days, eg. 1..40, we are just picking up customers who 
% have a slower visit period. These aren't real new customers. These are existing
% customers on a slow timescale.
%
% To remedy this, we put as much distance as possible between day 0 and the 
% beginning of an analysis period. Over this anal period, we will record true
% new customers. This will provide the acquisition rate over time.
% Thus, acquisition rate is estimated at the end of the timeseries.
% Defection rate is estimated at the beginning of the timeseries.

%max_startday=20;
 max_startday = max(startday);
min_startday = min(startday);

% Acquisition each day, assuming startday is correct. Look at the graph to check
number_new = [];total_new  =[];
candidate_new  = [1:size(startday,1)]';
for day=min_startday:max_startday,
   newcusts = find(startday(candidate_new,1)<=day);
   number_new(day,1) = size(newcusts,1);
   if day==1, 
      total_new(day,1)=number_new(day,1);
   else
      total_new(day,1) = total_new(day-1,1)+number_new(day,1);
   end;
   candidate_new = remove_ids(candidate_new,newcusts);
   fprintf('.');
end;
%number_new(max(startday)+1,1)=0;
% number_new is the number new each day
%total_new(max(startday)+1,1)=total_new(max(startday));
% total_new is the cumulative new up to this day
figure(1);clf;plot(number_new);
figure(2);clf;plot(total_new);

% Defection per day. Each day we know the rest of the customer's series is blank
% Again this can be incorrect at the end because a customer just hasn't come back 
% for their 7 day cycle
number_defectors=[];
new_defections=[];
defectors  = [1:size(startday,1)]';
for day=max_startday:-1:min_startday,
   purchaser = find(rev(defectors,day)~=0);
   if ~isempty(purchaser),
      defectors = remove_ids(defectors,purchaser);
      new_defections(day,1)=size(purchaser,1);
   else
      new_defections(day,1)=0;
   end;   
   number_defectors(day,1) = size(defectors,1);
   fprintf('.');%fprintf('%d ',size(purchaser,1));
end;
%number_defectors(max(startday)+1) = size(startday,1);
% number_defectors is the total defectors todate
%new_defections(max(startday)+1) = size(startday,1)-number_defectors(max(startday),1);
% new_defections is the new defections
fprintf('\n');
figure(3);clf;plot(new_defections);title('defectors who were good customers yesterday (new defectors)');
figure(4);clf;plot(round(100.*number_defectors./size(startday,1)));title('total defectors identified from now on'); 
ylabel('Percent of population');
xlabel(baseunit);
title(['Number of customers with no further purchases recorded, by ' baseunit]);

halfway_point = ((max_startday-min_startday)./2);

% New defections against new acquisitions per day
figure(5);clf;plot(min_startday:max_startday,new_defections,'r-');
hold on;plot(min_startday:max_startday,number_new,'b-');
v=axis;plot([halfway_point halfway_point],[v(3) v(4)],':');

% Total defections against total acquisitions per day
%figure(6);clf;hold on;plot(min(startday):max(startday)+1,number_defectors,'r-');
%plot(min(startday):max(startday)+1,total_new,'b-');
figure(6);clf;hold on;area([min_startday:max_startday+0]',[number_defectors]',1);
                      area([min_startday:max_startday+0]',total_new',2);
% new defections per day
v=axis;plot([halfway_point halfway_point],[v(3) v(4)],':');

figure(7);clf;hold on;area(min_startday:max_startday+0,total_new-number_defectors);
colormap(summer);v=axis;plot([halfway_point halfway_point],[v(3) v(4)],':');

figure(8);clf;hold on;area(min_startday:max_startday,number_new-new_defections);
colormap(summer);v=axis;plot([halfway_point halfway_point],[v(3) v(4)],':');

diff=number_new'./fliplr(new_defections');
ii=find(isinf(diff));diff(ii)=nan.*ones(size(ii));
figure(9);clf;hold on;area(min_startday:max_startday,diff);%diff./(fliplr(diff)+eps));
colormap(summer);v=axis;%plot([halfway_point halfway_point],[v(3) v(4)],':');
plot([v(1) v(2)],[1 1],':');

mean_missing(diff',nan),
median_missing(diff',nan),
% we want to now calculate the ratio of 

figure(3);clf;plot(round(100.*
% invariance, if day is non-zero, then customer is no longer a defector
% otherwise, we know the customer is still a defector


% -----------------------------------------------------------------------------



% Acquisition versus defection rate
% ---------------------------------

newcusts = find(startday>startdayhigh & startday<max(range2));
% absolute id
figure(3);clf;hold on;
bar(1,-1.*100.*(future_zero-initial_zero)./size(ii,1),'r');
bar(2,100.*size(newcusts,1)./size(ii,1)-100,'b');
%legend('Defection of old customers','Acquisition of new customers',0);
set(gca,'xtick',[1;2]);
set(gca,'xticklabel',['Defection of old customers  '; ...
                      'Acquisition of new customers']);
set(gca,'ygrid','on');
ylabel('Percent of original population');

% Pie graph
% ----------

newinactive = find(sum(sign(abs(rev(ii,startdayhigh:max(range2))))')'.*multiplier./size(range2,2)==0);
% absolute id

pie([size(newinactive,1) future_zero ...
     size(newcusts,1)-size(newinactive,1) size(ii,1)-future_zero], ...
    {'New inactive','Old inactive','New active','Old active'});
 
 figure(1);clf;
 pie([size(newinactive,1) size(newcusts,1)-size(newinactive,1) size(ii,1)+size(newcusts,1)], ...
    {'New inactive','New active','Rest'});
 
 figure(2);clf;
  pie([future_zero size(ii,1)-future_zero size(ii,1)+size(newcusts,1)], ...
    {'Old inactive','Old active','Rest'});
 
 figure(3);clf;
 bar([1 2],[size(newinactive,1) size(newcusts,1)-size(newinactive,1); future_zero ...
       size(ii,1)-future_zero],'stacked');
 
% new who are inactive, old who are now inactive, new who are active, old who are active
set(gca,'xticklabel',['initial';'future ']);
legend('defectors','loyals');
ylabel('percent of customers');
title(['Comparison between same customers now and ' num2str(min(range2)-max(range1)) ' ' baseunit 's into the future (' baseunit 's=' num2str(size(range2,2)) ', n=' num2str(size(ii,1)) ')']);


diff = round(100.*([future_zero future_vis]-[initial_zero initial_vis])./size(ii,1));
figure(4);clf;bar(1,diff(1),'b');hold on;
bar(2,diff(2),'r');
legend('defectors','loyals');
%set(gca,'xticklabel',['defectors';'loyals   ']);
ylabel('percent change in customer population');

%figure(3);clf;shading flat;plot(x,100.*a./sum(a),'r-');shading flat;grid on;
%figure(3);hold on;shading flat;plot(x,100.*a./sum(a),'b-');shading flat;grid on;




