function [fit,a,corr,perr] = dispar(series,                   ...
                          training_window,          ...
                          range,                    ...
                          days_in_model,            ...
                          delay_before_prediction);
%
% function [fit,a]   = dispar(series, 
%                             training_window_size, 
%                             test_range, 
%                             days_in_model, 
%                             delay_before_prediction);
%
% example
% fit = dispar(wi(1:10,:),100,400:500,10,7)
% fits models and tests the result on the first 10 series of wi, 
% using a 300:399 as training data (100 days), 400:500 as test data,
% using a 10 day regression model, and having a 7 day delay from 
% regression set to prediction. (ie. the model is predicting 7 days
% in advance).
%
% fit is (vertically) 10 series which is the result for each wi(1:10
% a is the coefficients for each series. These coefficients are written
% horizontally here, so each row entry is a different set of coefficients
%
% 
%
% rb
%  [W1,b1,W2,b2,nr,err] = solverb(P,T,[10 100 0.02 1]); 
% a = simurb(0.5,w1,b1,w2,b2); 

   range = range(1,1) + delay_before_prediction + max(days_in_model) : ...
                       range(1,size(range,2));

   fit = zeros(size(series));
   a   = zeros(size(series,1),days_in_model);
   m   = zeros(size(series,1),days_in_model/1);

   if isempty(training_window), infer_training_window=1; else infer_training_window=0; end;

for i=1:size(series,1),

   if infer_training_window, % find the first occurance of nonzero data
     for b=1:size(series,2),
       if series(i,b)>0,
         break;
       end;
     end;
     training_window = range(1,1)-b;
   end;
      
  training_range = range(1,1) - training_window : range(1,1)-1;
  % keyboard
  if (training_window < days_in_model + delay_before_prediction) | ...
     (series(i,training_range)==zeros(1,size(training_range,2))),
    disp('no data in training range');
  else
    disp([ num2str(training_window) ' training days']);
    
    if 0, % the old way, just calculate the fit on training data
  a(i,:) = earpyw(series(i,training_range), days_in_model, ... 
             delay_before_prediction)'; 
   end;
   
   % new way we need to know training set fit so that we can calc error
   disp('fitting training data');
   [fit(i,training_range),a(i,:)] = fitar(series(i,training_range), ...
      days_in_model,delay_before_prediction);
   
   disp('fitting model error');
   err = series(i,training_range) - fit(i,training_range);
   [fit(i,training_range),m(i,:)] = fitar(err,days_in_model/1,delay_before_prediction);
   
   
   for p=range,
     fit(i,p)=a(i,:) * flipud(series(i, ...
                                p-delay_before_prediction-days_in_model: ...
                                p-delay_before_prediction-1)') - ...
                                0.0 .* m(i,:) * flipud(series(i, ...
                                p-delay_before_prediction-days_in_model/1: ...
                                p-delay_before_prediction-1)');
                            
   end;

   figure(1);clf;
   plot(range,series(i,range),'r-');hold on;
   plot(range,fit(i,range));
   title(['timeseries ' num2str(i)]);

   [corr(i,:),perr(i,:)] = display_accuracy(fit(i,range),series(i,range),1);
   drawnow;pause

  end; % no data in training range
end;
