function [fit,a] = dispar2(series,                   ...
                          training_window,          ...
                          range,                    ...
                          days_in_model,            ...
                          delay_before_prediction,  ...
                          difference_dist,          ...
                          standardize);
%
% function [fit,a]   = dispar(series, 
%                             training_window_size, 
%                             test_range, 
%                             days_in_model, 
%                             delay_before_prediction,
%                             difference_dist,
%                             standardize);
%
% difference_dist is a vector, eg. [1 1 7 1] means difference
% at delay of 1,1,7 (a weekly adjustment, then 1)
%
% example
% fit = dispar(wi(1:10,:),100,400:500,10,7,0,0)
% fits models and tests the result on the first 10 series of wi, 
% using a 300:399 as training data (100 days), 400:500 as test data,
% using a 10 day regression model, and having a 7 day delay from 
% regression set to prediction. (ie. the model is predicting 7 days
% in advance).
%
% fit is (vertically) 10 series which is the result for each wi(1:10
% a is the coefficients for each series. These coefficients are written
% horizontally here, so each row entry is a different set of coefficients
%
% todo:
% difference
% standardize mean and variance based on training set
% (assumes we've detrended the points, so that training and test
% set are the same)
%

% difference the series

% series of differences are stored in diffseries
% we use the last of these, series, for the fit 
% then retrieve original series, graph it and fit

original_range = range;
%range = range(1,1) + sum(difference_dist') : range(1,size(range,2));
keyboard

original_series = series; % remember it so that we don't 
% have to explicitly retrieve it from the differenced series

original_series_size = size(series,2);

diffseries = [0 series];
for i=1:size(difference_dist,2),
  % series is the current series
  [start,series] = difference(series,difference_dist(1,i));
  % series(:,size(series,2)-i+1) = zeros(:,size(series,2)-1+1);
  diffseries = [diffseries start series zeros(1,i)];
end;
% we now have diffseries. At 1, (original_series_size + 1)*1 + 1
% we have the start, size at each of these is original_series_size -
% i

% series has now been set to the current series (maximally differenced)

keyboard

   training_range = range(1,1) - training_window : range(1,1)-1;
   if find(training_range<1),
     disp('with this training window and test_range there are negative');
     disp('training indexes. Move test_range right, or decrease');
     disp('training window');
     return
  end;
  fit = zeros(size(original_series)); % this is the differenced fit
   a   = zeros(size(series,1),days_in_model);

for i=1:size(series,1),

  a(i,:) = earpyw(series(i,training_range), days_in_model, ... 
             delay_before_prediction)'; 
   
   figure(1);clf;
   plot(original_range,original_series(i,original_range),'r-');hold on;
   title(['timeseries ' num2str(i)]);

   % more efficient way to do this exists! (vectorize it), but won't 
   % bother right now
 
   for p=range,
     fit(i,p)=a(i,:) * flipud(series(i, ...
                                p-delay_before_prediction-days_in_model: ...
                                p-delay_before_prediction-1)');

     % only for difference 
     % fit(i,p) = fit(i,p) + series(i,p-1); % current point is the change
     % plus the value last day

  end; 
  
  disp('just before fit set');
  keyboard
  fit(i,range(1,1):range(1,size(range,2)-1)) = series(i,range(1,1):range(1,size(range,2)-1)); % now do we get hte original series back
  r = range;
  % undifference the fit
  for i=size(difference_dist,2):-1:1,
     range = range(1,1) - difference_dist(1,i) : range(1,size(range,2));
     loc = (original_series_size+1)*(i-1) + 1;
     fit(i,range(1,1)) = diffseries(1,loc+1+1+range(1,1)); 
     % put first val of prev series = new series startval
     
     fit(i,range) = fit(i,range) + ... 
        diffseries(1,range(1,1)+loc+1:range(1,size(range,2))+loc+1-1);
     % this should have integrated it
     % note that loc+1, diffseries offset isn't used
     keyboard   
  end;
  range = r;
  disp('outside');
  keyboard
  
   plot(range,fit(i,range),'c-');

   corr = (fit(i,range) * series(i,range)') / (norm(fit(i,range))*norm(series(i,range)));
  
   s = series(i,range); % - mean(series(i,range)');
   f = fit(i,range); % - mean(fit(i,range)');

if 0,
   r = sum( (f-s).^2 ) ./ ...
       sum( (s-mean(s')).^2 );
end;

   z = find(s==0);
   s(z) = s(z)-1; % forces denominator to be -1
   f(z) = zeros(size(z)); % forces numerator to be 0, therefore we get
   % (-1-0)/-1 = 1. = 100%
   
   perr = 100 .* mean( abs((f-s)./s)' );
   disp(['corr '  num2str(corr) ' perr ' num2str(perr) ]);

  % plot r^2 info
  pause

end;
