function startday = find_first_spike(rev);
% startday = find_first_spike(series);
%
% finds the first spike from the left in the timeseries. This can be used
% to identify when a customer first enrolled in a program, or first made 
% a purchase (their startday). To find the last spike, 
% lastspike = size(series,2)+1-find_first_spike(fliplr(series));

number_of_customers = size(rev,1);
series_size         = size(rev,2);

startday  = nan.*ones(number_of_customers,1);
yettofind = [1:number_of_customers]';

for day=1:series_size,
   firstspike = find(rev(yettofind,day)~=0);
   if ~isempty(firstspike),
      startday(yettofind(firstspike),1) = ones(max(size(firstspike,1)),1).*day;
      yettofind = remove_ids(yettofind,firstspike);
      % setdiff(yettofind,yettofind(firstspike));
   end; 
   fprintf('.');
end;

if ~isempty(find(isnan(startday))), fprintf('Some series had no startday, these were coded as NAN\n');end;