function m = movingavg(series,windowsize,delay,fun,varargin);
% 
% m = movingavg(series,windowsize,delay,fun,varargin);
%
% INPUTS
% 
% series   is a matrix where horizontal vector is equal to a timeseries, and each row is
%          equal to another series that we want to process
% fun      is a function that we want to apply, for instance, movingavg is often the mean, 
%          but we can also write in 'std' to calculate the volatitlity of the series at each point
% varargin you can include any other parameters you want to pass into fun as a comma 
%          delimited set of parameters
% 
% EXAMPLE
%
% r = randn(1,100);
% figure(1);clf;plot(movingavg(r,10,0,'mean'));title('mean');pause;
% figure(1);clf;plot(movingavg(r,10,0,'median'));title('median');pause;
% figure(1);clf;plot(movingavg(r,10,0,'std'));title('std');pause;
% figure(1);clf;plot(movingavg(r,10,0,'var'));title('var');pause;
% figure(1);clf;plot(movingavg(r,10,0,'min'));title('min');pause;
%
% NOTES
%
% The first 1:delay+windowsize columns of the moving average series will be filled with zeros
% You could nan fill them if you wanted (might be more accurate). By filling the first part of the 
% moving average, we are able to directly overlay the moving average against original series
% 
% SEE ALSO
% 
% m = movingavg(series,windowsize,delay,fun,varargin);
% 

m = zeros(size(series));
for i=delay+windowsize+1:size(series,2),
  if nargin<4,
    m(:,i) = mean( series(:,i-delay-windowsize:i-delay-1)' )';
  else
    m(:,i) = feval(fun,( series(:,i-delay-windowsize:i-delay-1)' ))';
  end;
end;
m(:,1:delay+windowsize) = zeros(size(series,1),delay+windowsize);

return;



