function [fit,w1,b1,w2,b2] = rbfar(series,days,delay,targetdays,centroids);

% days is the number of input units
%
% We would like a solution with less than 100 hidden layer neurons  
% which fits the data with a sum squared error of less than 0.02.   
% A radial basis spread of 1.0 should be used, and progress should  
% be displayed every 10 neurons.                                    
%                                                                   
% >> [W1,b1,W2,b2,nr,err] = solverb(P,T,[10 100 0.02 1]);           


fit=[];
serieslen = size(series,2);
end_of_patterns = 1+ serieslen - (days+delay+targetdays); 
start_across = end_of_patterns; % toeplitz is square
% end of usable patterns in toeplitz

for i=1:size(series,1),

  sliding_series = toeplitz(series(i,:));

  inputs = sliding_series(1:end_of_patterns,start_across:start_across+days-1);

  targetrange = start_across+days+delay:start_across+days+delay+targetdays-1;
  targets = sliding_series(1:end_of_patterns,targetrange);

  errtarget = 0.1;
  d = inf; % display every 10 iterations

  spread = var(series') / (centroids * 10);

  disp('fitting rbf');
  figure(1);

% perform crossval training here

  [w1,b1,w2,b2] = solverb(inputs',targets',[d centroids errtarget spread]);
  % spread must be related to the typical variance of qty

  % display the result
  % ------------------

  disp('plotting series');

  figure(1);clf;drawnow;
  range = 30:size(series,2);

  figure(1);clf;
  plot(1:size(series,2),series(i,:),'r-');hold on;
 
  for p=range,
     out = simurb(series(i,p-delay-days:p-delay-1)',w1,b1,w2,b2);
     fit(i,p)=out(1,1); % only use the first estimate if its multidim
   end;
   plot(range,fit(i,range),'c-');
  
   figure(2);clf;
   no_frames = 3;
   for frame=0:no_frames-1,
     subplot(no_frames,1,frame+1);
     z1 = (frame    /no_frames) .* serieslen + 1;
     z2 = ((frame+1)/no_frames) .* serieslen;
     plot(z1:z2,series(i,z1:z2),'c-'); hold on;
     plot(z1:z2,fit(i,z1:z2),'r-');
   end;
    
   pause

% the question is does it generalize. we need to test it on a test set
% if it generalizes that will be pretty amazing, it means the ts has a 
% signature
end;
