function fit = seasonalweight(series,testwindow,delay,no_of_points);
% 
% fit = seasonalweights(series,testwindow,delay,no_of_points);
% 
% fits the following model:
% y(t) = sum of { y(t-1) * w(t-1)./sumw }
% where w is the autocorrelation coefficient
%
% arma tries to find weights so that prediction error is minimized
% this just assumes that seasonal should be added in proportion to its
% correlation throughout the series. 
%
% example
% fit = seasonalweight(wi(3,:),200,-1);
% r=350:500;figure(1);clf;plot(r,fit(1,r),'b-');hold on;plot(r,wi(3,r),'r-');
%
% find the first non-zero entry. This is the start of
% the series

for b=5:size(series,2),
   if series(1,b)>0, 
      disp('here');
      b,
      break;
   end;
end;

if b>=size(series,2),
   disp('series appears to be all zeros');
   return;
end;
%keyboard
series = series(1,b+1:size(series,2));
range = size(series,2)-testwindow+1:size(series,2);
if range(1,1)-1<1, 
   disp('zeros into training range, returning');
   return
end;

training_window = range(1,1)-1;
training_range = 1:training_window;
disp(['training on ' num2str(training_window) ...
      ' points']);

autocorr = zeros(size(series,1),training_window);
fit = autocorr;

%training_range = range(1,1)-training_window:range(1,1)-1;
autocorr = autocorr1way(series(1,training_range));

% now base the present on the weighted sum of 
% the previous seasonal points, where the points
% we take are lag points which have high correlation

half_size = floor(training_window/2);
if no_of_points>half_size, no_of_points=half_size-1;end;
if no_of_points<0, no_of_points=half_size-1;end;

[w,index] = sort(abs(autocorr(1,1:half_size))');
w = autocorr(index)';
% take top no_of_points values for the weights
% (or the lags we will apply)
% skip the first point since we know this is full corr
% (index of 1 = lag of 0)
% keyboard

w = w(half_size-no_of_points:half_size-1,1);
index = index(half_size-no_of_points:half_size-1,1);
sumw = sum(abs(w));

% the model is now w/sum(w) * ts(current-index)
% -1 if we get a full correlation at 1
% perform the lagged weightings to the

% apply model to the test range

for p=range,
  % for each point in the test range
  fit(1,p) = 0.0;
  for i=1:size(w,1),
     if index(i,1)>delay,
     fit(1,p) = fit(1,p) + (w(i,1) ./ sumw) .* series(1,p-index(i,1));
     end;   
  end;
end;

% adjust to seasonal mean and variance
% the weighted fit we've just calculated is biased in terms
% of mean and variance. (to do with weights). Readjust the
% mean and variance to be equal to the mean and variance of
% the training set. Could even readjust in time to be
% equal to mean and variance of complete series up to 
% forecast point in time.

% adjust mean

training_mean = mean(series(:,training_range)');
fit_mean      = mean(fit(1,range)');

% adjust variance

mult = std(series(1,training_range)') ./ std(fit(1,range)');
fit(1,range) = training_mean + mult .* (fit(1,range) - fit_mean);

%keyboard
corr = (fit(1,range) * series(1,range)') / (norm(fit(1,range))*norm(series(1,range)));
disp(['corr ' num2str(corr)]);

fit = [zeros(1,b+training_window) fit(1,range)];
% another way to do this, after identifying useful lags,
% put those into an AR model