function y = timeseries_operation(x,operation);
% 
% y = timeseries_operation(x,operation);
%
% Apply operation to timeseries x. x is a 1xn vector. This is consistent with other 
% timeseries operations, and different from other column vector operations. Operation should be 
% specified in terms of x(t), and should be evaluatable, eg. 'x(t)-x(t-1)' would implement 
% a differencing operation
% 
% EXAMPLE
%
% % Example 1
% % Build a detector to show when a timeseries value has become 
% % significantly high
% x = randn(1,100); % The variable we want to monitor
% z = timeseries_operation(x,'(x(t) - mean(x(1:t-1)))./std(x(1:t-1))');
% figure(1);clf;subplot(2,1,1);plot(x);hold on;subplot(2,1,2);plot(z);
% [a,i] = sort(-1.*z);
% i = i(1:5);
% figure(1);subplot(2,1,1);hold on;plot(i,x(i),'r*');subplot(2,1,2);hold on;plot(i,z(i),'r*');
% 
% % Example 2
% % Difference a timeseries
% x = randn(1,6);
% y = timeseries_operation(x,'x(t) - x(t-1)');
% find([nan diff(x)] ~= y),
% [[nan;diff(x)'] y'],
% 

warning off;
y = nan.*ones(size(x));
for t=1:size(x,2),

  try,
    y(t) = eval(operation);
  catch,
    fprintf('no dice %d\n',t);
    % it is dangerous to put a catch that doesn't do anything, 
    % but there can be cases where our formula is undefined, eg. 
    % x(t-1) could be undefined. Don't want an error to be triggered
    % in that case, so we're doing this
  end;
  
end;
warning on;
  